/* BaseNPAwareShifter does call for isBaseNP w/o stripping tags!
what is the effect for the func tag models?

agh!  so does ModNonterminalModelStructure2 - 
just checks modEvent.parent()
*/

/*
Send mail to Dan re +STOP+ for modifiers
could actually just redo this using only modifier event, but not
really becuase need to have modWordIdx,headWordIdx, etc.
*/


/*
look at how Dan identifies coordination and modifiers in coordinated
phrases.  It must be done somehow.  Collins makes the modifier be the
modifier of the first conjunct (appendix A).  Actually this could be different
for Dan due to the way he does coordination
*/

package danbikel.parser;

/*
For each comparison technique:
just put everything into one big map
then separate out later.  This will
make it easier to get a table like Collins' table 4
in map:
key is relation
Parent%%Head%%Modifier%%Direction
value has:
goldCount
testCount
matching
ArrayList instances
Each member of instances is an array of two
relations [0] from gold, [1] from test
*/
    
    /* 
       for this to work, need to add Gold/Test signifier to Relations (well, nogt
       really, but to be on the safe side) and sentNum
       RelationScores: (key -> 
                               int numGold 
			       int numTest
			       int numMatching
			       ArrayList matchingSamples
                                        eachElement: array [0] goldRelation [1] sentRelation
                               ArrayList missedSamples
                                        eachElement: goldRelation
                               ArrayList falseAlarmSamples
                                        eachElement: testRelation

       at the end, numGold-numMatching should = size missedSamples
                   numTest-numMatching should = size falseAlarmSamples
       
       processRelations needs to 
       1. go through every gold Relation and increment numGold
       2. go through every test Relation and increment numTest
       3. for each match that is found increment numMatching and add to matchingSamples
       4. at the end, go thorugh unmatched goldRelations and add to missedSamples
       5.                           and add unmatched testRelations to falseAlarmSamples

       
    */

/* basic problem - to sort, need to put keys into order,
   to do this with treemap, need to include numGold in key.
   that is ugly.  instead could convert objects of entire map
   into a list, and sort that.  

   But actually want to keep it as a map for purposes of breaking it up into subgroups
   really should make the key into a class with symbols for the components
   but then would need to ignore parts for different ways of sorting it.
   
   So maybe could make relationkey have the usual symbols and then also have
   a slot for numGold.  Then firstTreeMap would just ignore the numGold slot
   and define some order on the rest of it - agh, but that woudl be a pain,
   because would have to get the string for parentLabel etc. anyway.
   
   for now, just make a new key with oldkey+numgold, and stick it into a treemap.
   fix this for later.  refresh myself on the zillion different sorting
   possibilities.
*/

/* 

   HashMap relationInfoMap: 
       key = Relation.keyStr
       value = RelationInfo
       
       
   TreeMap sortedRelationMap:
       key = SortedRelationsKey(key from relationInfoMap, numGold)
       value= RelationInfo 

       
*/
import danbikel.lisp.*;
import danbikel.util.*;
import danbikel.util.AbstractMapToPrimitive.*;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.util.HashMap;
import java.text.DecimalFormat;

class SortedRelationsKey implements Comparable {
    String keyStr;
    int numGold;

    public SortedRelationsKey(String keyStr, int numGold) {
	this.keyStr=keyStr;
	this.numGold=numGold;
    }
    
    public int compareTo(Object o) {
	SortedRelationsKey otherKey=(SortedRelationsKey)o;
	int otherNumGold=otherKey.numGold;
	String otherKeyStr=otherKey.keyStr;
	if (numGold<otherNumGold) {
	    return 1;
	} else if (numGold>otherNumGold) {
	    return -1;
	} else {
	    return otherKeyStr.compareTo(keyStr);
	}
    }
}
    

class Relation {
    int sentNum;
    boolean isGold;
    int modIdx;
    Symbol modWordWord;
    int headWordIdx;
    Symbol headWordWord;
    Symbol parentLabel;
    Symbol parentLabelNoAugs;
    Symbol headLabel;
    Symbol headLabelNoAugs;
    boolean headLabelIsPreterminal;
    Symbol modLabel;
    Symbol modLabelOnlyDefaultArg;
    boolean modLabelIsPreterminal;
    boolean direction;
    boolean matched=false;
    String keyStr=null;
    
    Symbol keyParentLabel=null;
    Symbol keyHeadLabel=null;
    Symbol keyModLabel=null;
    Word prevPunc=null;
    Word prevConj=null;

    static final String relDelimStr=" ";    
    
    Relation(int sentNum, boolean isGold,
	     int modIdx, Symbol modWordWord,
	     int headWordIdx, Symbol headWordWord,
	     Symbol parentLabel, Symbol parentLabelNoAugs,
	     Symbol headLabel, Symbol headLabelNoAugs,  boolean headLabelIsPreterminal,
	     Symbol modLabel, Symbol modLabelOnlyDefaultArg, boolean modLabelIsPreterminal,
	     boolean direction,
	     Word prevConj,
	     Word prevPunc) {
	this.sentNum=sentNum;
	this.isGold=isGold;
	this.modIdx=modIdx;
	this.modWordWord=modWordWord;
	this.headWordIdx=headWordIdx;
	this.headWordWord=headWordWord;
	this.parentLabel=parentLabel;
	this.parentLabelNoAugs=parentLabelNoAugs;
	this.headLabel=headLabel;
	this.headLabelNoAugs=headLabelNoAugs;
	this.headLabelIsPreterminal=headLabelIsPreterminal;
	this.modLabel=modLabel;
	this.modLabelOnlyDefaultArg=modLabelOnlyDefaultArg;
	this.modLabelIsPreterminal=modLabelIsPreterminal;
	this.direction=direction;
	this.prevConj=prevConj;
	this.prevPunc=prevPunc;
	matched=false;
    }
    void setKey(String keyStr) {
	this.keyStr=keyStr;
    }
    void makeKey() {
	keyParentLabel=parentLabelNoAugs;
	keyHeadLabel=(headLabelIsPreterminal ? AnalyzeDependencies.TAGSym :
		      headLabelNoAugs);
	keyModLabel=(modLabelIsPreterminal ? AnalyzeDependencies.TAGSym : 
		     modLabelOnlyDefaultArg);
	keyStr="";
	if (prevConj!=null) {
	    keyStr="CONJ"+relDelimStr;
	}

	keyStr=keyParentLabel+relDelimStr+
	    keyHeadLabel+relDelimStr+
	    keyModLabel+relDelimStr+
	    (direction==Constants.LEFT ? "L" : "R");
	if (prevConj!=null) {
	    keyStr+=relDelimStr+"CONJ";
	}
	if (prevPunc!=null) {
	    keyStr+=relDelimStr+"PUNC";
	}	

    }

    boolean equals(Relation other) {
	return 
	    this.keyStr.equals(other.keyStr) &&
	    this.modIdx==other.modIdx &&
	    this.headWordIdx==other.headWordIdx;
    }
	    
    /*
    public String toString() {
	return   
	    "sentNum="+sentNum+":"+isGold+
	    parentLabel+"(" +parentLabelNoAugs+") "+
	    headLabel+"("+headLabelNoAugs+","+headLabelIsPreterminal+") "+
	    modLabel+ "("+modLabelOnlyDefaultArg+","+modLabelIsPreterminal+") "+
	    (direction==Constants.LEFT ? "L " : "R ") + 
	    modIdx + "(" +modWordWord+") "+
	    headWordIdx+"("+headWordWord+") "+
	    "  key="+key;
    }
    public String toString() {
	return   
	    "("+sentNum+","+(isGold?"GOLD":"TEST")+") "+
	    parentLabel+" " + headLabel+" " + modLabel+ 
	    (direction==Constants.LEFT ? "L " : "R ") + 
	    modIdx + "(" +modWordWord+") "+
	    headWordIdx+"("+headWordWord+") "+
	    "  key="+key;
    }
    */
    public String toString() {
	return   
	    "("+sentNum+","+(isGold?"GOLD":"TEST")+") "+
	    (prevConj!=null ? "CONJ " : "") +
	    (prevPunc!=null ? "PUNC " : "") +
	    keyParentLabel+relDelimStr+ 
	    keyHeadLabel+"("+headWordIdx+":"+headWordWord+")"+relDelimStr+
	    keyModLabel+"("+modIdx+":"+modWordWord+")"+	relDelimStr+
	    (direction==Constants.LEFT ? "L " : "R ");
    }
}


class RelationInfo {
    double recall=0.0;
    double precision=0.0;
    int numGold;
    double perTotal;
    double perCumTotal;
    int numTest;
    int numMatch;
    
    ArrayList matchSamples;
    ArrayList missSamples;
    ArrayList falseAlarmSamples;

    public RelationInfo() {
	numGold=0;
	numTest=0;
	numMatch=0;
	matchSamples=new ArrayList();
	missSamples=new ArrayList();
	falseAlarmSamples=new ArrayList();
    }
    public String toString() {
	return 
	    "numGold="+numGold+
	    " numTest="+numTest+
	    " numMatch="+ numMatch+
	    " #matchSamples="+ matchSamples.size()+
	    " #missSamples="+ missSamples.size()+
	    " #falseAlarmSamples="+falseAlarmSamples.size();
    }
}

public class AnalyzeDependencies implements Serializable {
  // constants
  //static boolean secretFlag;
    static DecimalFormat percentage=new DecimalFormat("##0.00");

    static final String TAGStr="TAG";
    static final Symbol TAGSym=Symbol.add("TAG");
    static final String indentStr="   ";

    static Symbol defaultArgAugmentation=Language.training().defaultArgAugmentation();

    BufferedWriter outResults=null;
    BufferedWriter outResultsDebug=null;
    String goldFilename=null;
    String testFilename=null;
    String resultsFilename=null;

    boolean includeAMarking=false;
    
    HashMap relationInfoMap=null;

    void debugWriteln(String s) {
	try {
	    outResultsDebug.write(s+"\n");
	}
	catch (IOException e) {
	    System.err.println("io exception"+ e);
	    e.printStackTrace();
	    System.exit(-1);
	}
    }
    void resultsWriteln(String s) {
	try {
	    outResults.write(s+"\n");
	}
	catch (IOException e) {
	    System.err.println("io exception"+ e);
	    e.printStackTrace();
	    System.exit(-1);
	}
    }

    

  private final static Symbol collinsTop = Symbol.add("TOP");

  private final static boolean shareCounts =
    Boolean.valueOf(Settings.get(Settings.trainerShareCounts)).booleanValue();

  private static final boolean addGapInfo =
    Boolean.valueOf(Settings.get(Settings.addGapInfo)).booleanValue();


  private final static String className = Trainer.class.getName();

  /**
   * The class from which an instance will be constructed in
   * {@link #main(String[])}.  This data member may be re-assigned in
   * a subclass' <code>main</code> method before invocation of this
   * class' <code>main</code> method, so that all trainer method
   * invocations done by this class' <code>main</code> method will be on
   * an instance of the subclass.
   */
  protected static Class trainerClass = Trainer.class;

  // constants for default classname prefixes for ProbabilityStructure
  // subclasses
  private final static String packagePrefix =
    Settings.get(Settings.modelStructurePackage) + ".";
  private final static String lexPriorModelStructureClassnamePrefix =
    packagePrefix + "LexPriorModelStructure";
  private final static String nonterminalPriorModelStructureClassnamePrefix =
    packagePrefix + "NonterminalPriorModelStructure";
  private final static String topNonterminalModelStructureClassnamePrefix =
    packagePrefix + "TopNonterminalModelStructure";
  private final static String topLexModelStructureClassnamePrefix =
    packagePrefix + "TopLexModelStructure";
  private final static String headModelStructureClassnamePrefix =
    packagePrefix + "HeadModelStructure";
  private final static String gapModelStructureClassnamePrefix =
    packagePrefix + "GapModelStructure";
  private final static String leftSubcatModelStructureClassnamePrefix =
    packagePrefix + "LeftSubcatModelStructure";
  private final static String rightSubcatModelStructureClassnamePrefix =
    packagePrefix + "RightSubcatModelStructure";
  private final static String modNonterminalModelStructureClassnamePrefix =
    packagePrefix + "ModNonterminalModelStructure";
  private final static String modWordModelStructureClassnamePrefix =
    packagePrefix + "ModWordModelStructure";

  // types of events gathered by the trainer (provided as public constants
  // so that users can interpret human-readable trainer output file

  /** The label for nonterminal generation events.  This symbol has the
      print-name <tt>&quot;nonterminal&quot;</tt>. */
  public final static Symbol nonterminalEventSym = Symbol.add("nonterminal");
  /** The label for head nonterminal generation events.  This symbol has the
      print-name <tt>&quot;head&quot;</tt>. */
  public final static Symbol headEventSym = Symbol.add("head");
  /** The label for modifier nonterminal generation events.  This symbol has
      the print-name <tt>&quot;mod&quot;</tt>. */
  public final static Symbol modEventSym = Symbol.add("mod");
  /** The label for gap events.  This symbol has the
      print-name <tt>&quot;gap&quot;</tt>. */
  public final static Symbol gapEventSym = Symbol.add("gap");
  /** The label for word to part-of-speech mappings.  This symbol has the
      print-name <tt>&quot;pos&quot;</tt>. */
  public final static Symbol posMapSym = Symbol.add("pos");
  /** The label for vocabulary counts.  This symbol has the
      print-name <tt>&quot;vocab&quot;</tt>. */
  public final static Symbol vocabSym = Symbol.add("vocab");
  /** The label for word feature (unknown vocabulary) counts.  This
      symbol has the print-name <tt>&quot;word-feature&quot;</tt>. */
  public final static Symbol wordFeatureSym = Symbol.add("word-feature");
  /**
   * The label for the set of pruned preterminals.  This symbol has the
   * print-name <tt>&quot;pruned-preterm&quot;</tt>.
   *
   * @see Training#prune(Sexp)
   */
  public final static Symbol prunedPretermSym = Symbol.add("pruned-preterm");
  /**
   * The label for the set of pruned punctuation preterminals.  This
   * symbol has the print-name <tt>&quot;pruned-preterm&quot;</tt>.
   *
   * @see Training#raisePunctuation(Sexp)
   * @see Training#getPrunedPunctuation()
   */
  public final static Symbol prunedPuncSym = Symbol.add("pruned-punc");

  // integer types for mapping the above symbols to ints for a switch statement
  private final static int nonterminalEventType = 1;
  private final static int headEventType = 2;
  private final static int modEventType = 3;
  private final static int gapEventType = 4;
  private final static int posMapType = 5;
  private final static int vocabType = 6;
  private final static int wordFeatureType = 7;
  private final static int prunedPretermType = 8;
  private final static int prunedPuncType = 9;

  private final static Object[][] eventsToTypesArr = {
    {nonterminalEventSym, new Integer(nonterminalEventType)},
    {headEventSym, new Integer(headEventType)},
    {modEventSym, new Integer(modEventType)},
    {gapEventSym, new Integer(gapEventType)},
    {posMapSym, new Integer(posMapType)},
    {vocabSym, new Integer(vocabType)},
    {wordFeatureSym, new Integer(wordFeatureType)},
    {prunedPretermSym, new Integer(prunedPretermType)},
    {prunedPuncSym, new Integer(prunedPuncType)},
  };

  private final static MapToPrimitive eventsToTypes = new HashMapInt();
  static {
    for (int i = 0; i < eventsToTypesArr.length; i++) {
      Object key = eventsToTypesArr[i][0];
      int value = ((Integer)eventsToTypesArr[i][1]).intValue();
      eventsToTypes.put(key, value);
    }
  }

  // data members

  // settings
  protected int unknownWordThreshold;
  protected double countThreshold;
  protected double derivedCountThreshold;
  protected int reportingInterval;
  protected int numPrevMods;
  protected int numPrevWords;
  protected boolean keepAllWords;
  protected boolean keepLowFreqTags;
  protected boolean downcaseWords;

  // data storage for training
  protected CountsTable nonterminals = new CountsTableImpl();
  protected CountsTable priorEvents = new CountsTableImpl();
  protected CountsTable headEvents = new CountsTableImpl();
  protected CountsTable modifierEvents = new CountsTableImpl();
  protected CountsTable gapEvents = new CountsTableImpl();
  protected CountsTable vocabCounter = new CountsTableImpl();
  protected CountsTable wordFeatureCounter = new CountsTableImpl();
  protected Set vocab = new HashSet();
  protected Map posMap = new HashMap();
  protected Map headToParentMap = new HashMap();
  protected Map leftSubcatMap = new HashMap();
  protected Map rightSubcatMap = new HashMap();
  protected Map modNonterminalMap = new HashMap();
  protected Map simpleModNonterminalMap = new HashMap();
  protected Set prunedPreterms;
  protected Set prunedPunctuation;
  // temporary map for canonicalizing subcat objects for
  // leftSubcatMap and rightSubcatMap data members
  transient protected Map canonicalSubcatMap;
  transient protected Subcat emptySubcat = Subcats.get();
  protected HashSet wordFeatureTypes = new HashSet();
  protected HashSet unknownWords = new HashSet();
  protected ModelCollection modelCollection;

  // Model objects
  protected Model lexPriorModel;
  protected Model nonterminalPriorModel;
  protected Model topNonterminalModel;
  protected Model topLexModel;
  protected Model headModel;
  protected Model gapModel;
  protected Model leftSubcatModel;
  protected Model rightSubcatModel;
  protected Model modNonterminalModel;
  protected Model modWordModel;

  // handle onto static WordFeatures object from Language object
  transient protected WordFeatures wordFeatures = Language.wordFeatures;

  // handles onto some data from Training for more efficient and more readable
  // code
  protected Symbol startSym = Language.training.startSym();
  protected Symbol stopSym = Language.training.stopSym();
  protected Symbol topSym = Language.training.topSym();
  protected Word startWord = Language.training.startWord();
  protected Word stopWord = Language.training.stopWord();
  protected Symbol gapAugmentation = Language.training.gapAugmentation();
  protected Symbol traceTag = Language.training.traceTag();

  // various filters used by deriveCounts, deriveSubcatMaps and
  // deriveModNonterminalMap
  protected Filter allPass = new AllPass();
  protected Filter nonTop = new Filter() {
    public boolean pass(Object obj) {
      return ((TrainerEvent)obj).parent() != topSym;
    }
  };
  protected Filter nonPreterm = new Filter() {
    public boolean pass(Object obj) {
      TrainerEvent event = (TrainerEvent)obj;
      return event.parent() != event.headWord().tag();
    }
  };
  protected Filter nonTopNonPreterm = new Filter() {
    public boolean pass(Object obj) {
      TrainerEvent event = (TrainerEvent)obj;
      return (event.parent() != topSym &&
              event.parent() != event.headWord().tag());
    }
  };
  protected Filter topOnly = new Filter() {
      public boolean pass(Object obj) {
	return ((TrainerEvent)obj).parent() == topSym;
      }
    };
  protected Filter nonStop = new Filter() {
    public boolean pass(Object obj) {
      if (obj instanceof ModifierEvent) {
	ModifierEvent modEvent = (ModifierEvent)obj;
	return modEvent.modifier() != stopSym;
      }
      else {
	return true;
      }
    }
  };
  protected Filter nonStopAndNonTop = new Filter() {
      public boolean pass(Object obj) {
	if (obj instanceof ModifierEvent) {
	  ModifierEvent modEvent = (ModifierEvent)obj;
	  return modEvent.modifier() != stopSym && modEvent.parent() != topSym;
	}
	else {
	  return true;
	}
      }
  };

  /**
   * Class to represent a MapToPrimitive.Entry object for use by the
   * {@link #getEventIterator} method.
   */
  public static class EventEntry extends AbstractMapToPrimitive.Entry {
    /**
     * The {@link TrainerEvent} object contained by this map entry.
     */
    protected TrainerEvent event;
    /**
     * The observed count of the {@link TrainerEvent} object contained by
     * this map entry.
     */
    protected double count;

    /**
     * Constructs a new <code>EventEntry</code> instance with the specified
     * {@link TrainerEvent} and count.
     *
     * @param event the {@link TrainerEvent} of this map entry
     * @param count the observed count of the {@link TrainerEvent} of this map
     * entry
     */
    public EventEntry(TrainerEvent event, double count) {
      this.event = event;
      this.count = count;
    }

    public Object getKey() {
      return event;
    }
    public double getDoubleValue(int idx) {
      return count;
    }

    /**
     * Throws an {@link UnsupportedOperationException}.
     * @return an {@link UnsupportedOperationException}.
     */
    public Object getValue() {
      throw new UnsupportedOperationException();
    }
    /**
     * Throws an {@link UnsupportedOperationException}.
     * @return an {@link UnsupportedOperationException}.
     */
    public Object setValue(Object value) {
      throw new UnsupportedOperationException();
    }
    /**
     * Throws an {@link UnsupportedOperationException}.
     * @return an {@link UnsupportedOperationException}.
     */
    public boolean replaceKey(Object newKey) {
      throw new UnsupportedOperationException();
    }
  }

  // constructor

  /**
   * Constructs a new training object, which uses values from {@link Settings}
   * for its settings.  This class is not thread-safe, and there will typically
   * be one instance of a <code>Trainer</code> object per process, constructed
   * via the {@link #main} method of this class.
   *
   * @see Settings#unknownWordThreshold
   * @see Settings#countThreshold
   * @see Settings#derivedCountThreshold
   * @see Settings#trainerReportingInterval
   * @see Settings#numPrevMods
   */
  public AnalyzeDependencies() {
    unknownWordThreshold =
      Integer.parseInt(Settings.get(Settings.unknownWordThreshold));
    countThreshold =
      Double.parseDouble(Settings.get(Settings.countThreshold));
    derivedCountThreshold =
      Double.parseDouble(Settings.get(Settings.derivedCountThreshold));
    reportingInterval =
      Integer.parseInt(Settings.get(Settings.trainerReportingInterval));
    numPrevMods =
      Integer.parseInt(Settings.get(Settings.numPrevMods));
    numPrevWords =
      Integer.parseInt(Settings.get(Settings.numPrevWords));

    String keepAllWordsStr = Settings.get(Settings.keepAllWords);
    keepAllWords = Boolean.valueOf(keepAllWordsStr).booleanValue();
    String keepLowFreqTagsStr = Settings.get(Settings.keepLowFreqTags);
    keepLowFreqTags = Boolean.valueOf(keepLowFreqTagsStr).booleanValue();
    String downcaseWordsStr = Settings.get(Settings.downcaseWords);
    downcaseWords = Boolean.valueOf(downcaseWordsStr).booleanValue();

    modelCollection = newModelCollection();
  }

  protected ModelCollection newModelCollection() {
    return new ModelCollection();
  }

  /**
   * Records observations from the training trees contained in the
   * specified S-expression tokenizer.  The observations are either mappings
   * stored in <code>Map</code> objects or items to be counted, stored in
   * <code>CountsTable</code> objects.  All the trees obtained from
   * <code>tok</code> are first preprocessed using
   * <code>Training.preProcess(Sexp)</code>.
   *
   * @param tok the S-expression tokenizer from which to obtain training
   * parse trees
   * @param auto indicates whether to automatically determine whether to
   * strip off outer parens of training parse trees before preprocessing;
   * if the value of this argument is <code>false</code>, then the value
   * of <code>stripOuterParens</code> is used
   * @param stripOuterParens indicates whether an outer layer of parentheses
   * should be stripped off of trees obtained from <code>tok</code> before
   * preprocessing and training (only used if the <code>auto</code> argument
   * is <code>false</code>)
   *
   * @see CountsTable
   * @see Training#preProcess(Sexp)
   */
    public void process(SexpTokenizer goldTok,SexpTokenizer testTok,
			boolean auto, boolean stripOuterParens)
	throws IOException {

	Sexp goldTreeSexp=null;
	Sexp testTreeSexp=null;

	int sentNum = 0, intervalCounter = 0;
	int numSents = 0;
	ArrayList goldRelations=null;
	ArrayList testRelations=null;
	System.err.println("Phase 0: reading trees and finding heads");

	relationInfoMap=new HashMap();
	for ( ; (goldTreeSexp = Sexp.read(goldTok)) != null; sentNum++, intervalCounter++) {
	    debugWriteln("sentNum="+sentNum);

	    if (intervalCounter == reportingInterval) {
		System.err.println(className + ": processed " + sentNum + " sentence" +
				   (sentNum > 1 ? "s" : ""));
		intervalCounter = 0;
	    }
	    goldTreeSexp=adjustForParens(goldTreeSexp);
	    testTreeSexp=Sexp.read(testTok);
	    numSents++;

	    if (testTreeSexp==null) {
		System.err.println("missing test parse for gold parse");
		System.exit(-1);
	    }
	    if (treeIsNull(testTreeSexp)) {
		debugWriteln("test is null");
	    } else {
		testTreeSexp=adjustForParens(testTreeSexp);
		Language.training.preProcess(goldTreeSexp);
		Language.training.preProcess(testTreeSexp);
		debugWriteln("goldTree=\n"+
				   danbikel.parser.util.Util.prettyPrint(goldTreeSexp));
		
		debugWriteln("testTree=\n"+
				   danbikel.parser.util.Util.prettyPrint(testTreeSexp));
		
		HeadTreeNode2 goldHeadTree = new HeadTreeNode2(goldTreeSexp);
		goldRelations=new ArrayList();
		collectRelations(sentNum,true,goldHeadTree,goldRelations);

		HeadTreeNode2 testHeadTree = new HeadTreeNode2(testTreeSexp);
		testRelations=new ArrayList();
		collectRelations(sentNum,false,testHeadTree,testRelations);

		processRelations(relationInfoMap,
				 sentNum, goldRelations, testRelations);
		
	    }
	}
	debugWriteln("number of Relations="+relationInfoMap.size());
	TreeMap sortedRelationMap=sortRelationScores(relationInfoMap);
	debugWriteln("number of sorted Relations="+sortedRelationMap.size());
	calcRelationScores(sortedRelationMap);
	showRelationScores(sortedRelationMap);
	

	
	prunedPreterms = Language.training.getPrunedPreterms();
	prunedPunctuation = Language.training.getPrunedPunctuation();
    }

    TreeMap sortRelationScores(HashMap relationInfoMap) {
	debugWriteln("sorting relationInfomap into TreeMap");
	TreeMap sortedMap=new TreeMap();
	Iterator keys=relationInfoMap.keySet().iterator();
	while (keys.hasNext()) {
	    String keyStr=(String)keys.next();
	    debugWriteln("keyStr="+keyStr);
	    RelationInfo relInfo=(RelationInfo)relationInfoMap.get(keyStr);
	    sortedMap.put(new SortedRelationsKey(keyStr,relInfo.numGold),relInfo);
	}
	return sortedMap;
   }
    void showRelationScores(HashMap relationInfoMap) {
	Iterator keys=relationInfoMap.keySet().iterator();
	while (keys.hasNext()) {
	    String key=(String)keys.next();
	    debugWriteln("key="+key);
	    RelationInfo relInfo=(RelationInfo)relationInfoMap.get(key);
	    debugWriteln(indentStr+"relInfo="+relInfo);

	    int numMatchSamples=relInfo.matchSamples.size();
	    int numMissSamples=relInfo.missSamples.size();
	    int numFalseAlarmSamples=relInfo.falseAlarmSamples.size();
	    if (numMatchSamples!=relInfo.numMatch) {
		debugWriteln("WARNING: numMatchSamples="+numMatchSamples+
				   " numMatch="+relInfo.numMatch);
	    }
	    if (relInfo.numGold!=
		relInfo.numMatch+numMissSamples) {
		debugWriteln("WARNING: numGold="+relInfo.numGold+
				   " numMatch="+relInfo.numMatch +
				   " numMiss="+numMissSamples);
	    }
	    if (relInfo.numTest!=
		relInfo.numMatch+numFalseAlarmSamples) {
		debugWriteln("WARNING: numTest="+relInfo.numGold+
				   " numMatch="+relInfo.numMatch+
				   " numMiss="+numFalseAlarmSamples);
	    }

	    for (int i=0; i<relInfo.matchSamples.size(); i++) {
		Relation[] matchPair=(Relation [])relInfo.matchSamples.get(i);
		debugWriteln(indentStr+"match "+i+") GOLD " + matchPair[0]);
		debugWriteln(indentStr+"match "+i+") TEST " + matchPair[1]);
	    }

	    for (int i=0; i<relInfo.missSamples.size(); i++) {
		debugWriteln(indentStr+"miss "+i+") "+relInfo.missSamples.get(i));
	    }
	    for (int i=0; i<relInfo.falseAlarmSamples.size(); i++) {
		debugWriteln(indentStr+"falseAlarm "+i+") "+relInfo.falseAlarmSamples.get(i));
	    }
	}
    }

    void calcRelationScores(TreeMap sortedRelationMap) {
	int numGoldTotal=0;
	int numGoldSoFar=0;
	Iterator keys=sortedRelationMap.keySet().iterator();
	while (keys.hasNext()) {
	    SortedRelationsKey sortedRelationsKey=
		(SortedRelationsKey)keys.next();
	    numGoldTotal+=sortedRelationsKey.numGold;
	}
	debugWriteln("total number gold relations = " + numGoldTotal);
	keys=sortedRelationMap.keySet().iterator();
	while (keys.hasNext()) {
	    SortedRelationsKey sortedRelationsKey=
		(SortedRelationsKey)keys.next();
	    RelationInfo relInfo=(RelationInfo)sortedRelationMap.get(sortedRelationsKey);
	    relInfo.perTotal=100.0*relInfo.numGold/numGoldTotal;
	    numGoldSoFar+=relInfo.numGold;
	    relInfo.perCumTotal=100.0*numGoldSoFar/numGoldTotal;
	    

	    int numMatchSamples=relInfo.matchSamples.size();
	    int numMissSamples=relInfo.missSamples.size();
	    int numFalseAlarmSamples=relInfo.falseAlarmSamples.size();
	    if (numMatchSamples!=relInfo.numMatch) {
		debugWriteln("WARNING: key="+sortedRelationsKey.keyStr+
				   " numMatchSamples="+numMatchSamples+
				   " numMatch="+relInfo.numMatch);
	    }
	    if (relInfo.numGold!=
		relInfo.numMatch+numMissSamples) {
		debugWriteln("WARNING: key="+sortedRelationsKey.keyStr+
				   " numGold="+relInfo.numGold+
				   " numMatch="+relInfo.numMatch +
				   " numMiss="+numMissSamples);
	    }
	    if (relInfo.numTest!=
		relInfo.numMatch+numFalseAlarmSamples) {
		debugWriteln("WARNING: key="+sortedRelationsKey.keyStr+
				   " numTest="+relInfo.numGold+
				   " numMatch="+relInfo.numMatch+
				   " numMiss="+numFalseAlarmSamples);
	    }

	    relInfo.recall= 
		relInfo.numGold==0 ?0.0 :
		100.00 * relInfo.numMatch / relInfo.numGold;
	    relInfo.precision = 
		relInfo.numTest==0 ? 0.0 :
		100.00 * relInfo.numMatch/relInfo.numTest;
	}
    }
	
			   
	    
    void showRelationScores(TreeMap sortedRelationMap) {

	debugWriteln("Rank   CP      P     Count          Relation           R        P");
	resultsWriteln("Rank   CP      P     Count          Relation           R        P");

	int count=0;
	Iterator keys=sortedRelationMap.keySet().iterator();

	while (keys.hasNext()) {
	    count++;
	    SortedRelationsKey sortedRelationsKey=
		(SortedRelationsKey)keys.next();
	    RelationInfo relInfo=(RelationInfo)sortedRelationMap.get(sortedRelationsKey);
	    debugWriteln(rightJustify(3,count)+
			 rightJustify(8,
				      percentage.format(relInfo.perCumTotal))+
			 rightJustify(8,percentage.format(relInfo.perTotal))+
			 rightJustify(8,relInfo.numGold)+
			 rightJustify(25,sortedRelationsKey.keyStr)+
			 rightJustify(8,percentage.format(relInfo.recall))+
			 rightJustify(8,percentage.format(relInfo.precision)));

	    resultsWriteln(rightJustify(3,count)+
			 rightJustify(8,
				      percentage.format(relInfo.perCumTotal))+
			 rightJustify(8,percentage.format(relInfo.perTotal))+
			 rightJustify(8,relInfo.numGold)+
			 rightJustify(25,sortedRelationsKey.keyStr)+
			 rightJustify(8,percentage.format(relInfo.recall))+
			 rightJustify(8,percentage.format(relInfo.precision)));


	    /*
	    debugWriteln("key="+sortedRelationsKey.keyStr +
			       " numGold="+sortedRelationsKey.numGold);

	    debugWriteln(indentStr+"relInfo="+relInfo);
	    */

	    debugWriteln("#matchSamples="+relInfo.matchSamples.size()+
			       " #missSamples="+relInfo.missSamples.size()+
			       " #falseAlarms="+relInfo.falseAlarmSamples.size());

	    for (int i=0; i<relInfo.matchSamples.size(); i++) {
		Relation[] matchPair=(Relation [])relInfo.matchSamples.get(i);
		debugWriteln(indentStr+"match "+i+") GOLD " + matchPair[0]);
		debugWriteln(indentStr+"match "+i+") TEST " + matchPair[1]);
	    }

	    for (int i=0; i<relInfo.missSamples.size(); i++) {
		debugWriteln(indentStr+"miss "+i+") "+relInfo.missSamples.get(i));
	    }
	    for (int i=0; i<relInfo.falseAlarmSamples.size(); i++) {
		debugWriteln(indentStr+"falseAlarm "+i+") "+relInfo.falseAlarmSamples.get(i));
	    }
	}
    }

    void processRelations(HashMap relationInfoMap, int sentNum,
			  ArrayList goldRelations, ArrayList testRelations) {

	for (int i=0; i<goldRelations.size(); i++) {
	    Relation goldRel=(Relation)goldRelations.get(i);
	    goldRel.makeKey();
	    
	}
	for (int i=0; i<testRelations.size(); i++) {
	    Relation testRel=(Relation)testRelations.get(i);
	    testRel.makeKey();
	}

	debugWriteln("gold Relations=");
	for (int i=0; i<goldRelations.size(); i++) {
	    debugWriteln(i+")"+goldRelations.get(i));
	    Relation rel=(Relation)goldRelations.get(i);
	    RelationInfo relationInfo=(RelationInfo) relationInfoMap.get(rel.keyStr);
	    if (relationInfo==null) {
		relationInfo=new RelationInfo();
		relationInfoMap.put(rel.keyStr,relationInfo);
	    }
	    relationInfo.numGold++;
	}

	debugWriteln("test Relations=");
	for (int i=0; i<testRelations.size(); i++) {
	    debugWriteln(i+")"+testRelations.get(i));
	    Relation rel=(Relation)testRelations.get(i);
	    RelationInfo relationInfo=(RelationInfo) relationInfoMap.get(rel.keyStr);
	    if (relationInfo==null) {
		relationInfo=new RelationInfo();
		relationInfoMap.put(rel.keyStr,relationInfo);
	    }
	    relationInfo.numTest++;
	}
	int count=0;
	for (int i=0; i<goldRelations.size(); i++) {
	    Relation goldRel=(Relation)goldRelations.get(i);
	    for (int j=0; j<testRelations.size(); j++) {
		Relation testRel=(Relation)testRelations.get(j);
		if (!goldRel.matched &&
		    goldRel.equals(testRel)) {
		    debugWriteln(++count +") match goldRel " + goldRel);
		    debugWriteln(  count +")  with testRel " + testRel);
		    goldRel.matched=true;
		    testRel.matched=true;
		    RelationInfo relationInfo=(RelationInfo) relationInfoMap.get(goldRel.keyStr);		    
		    relationInfo.numMatch++;
		    relationInfo.matchSamples.add(new Relation[] {goldRel,testRel});
		}
	    }
	}
	count=0;
	for (int i=0; i<goldRelations.size(); i++) {
	    Relation goldRel=(Relation)goldRelations.get(i);
	    if (!goldRel.matched) {
		    RelationInfo relationInfo=(RelationInfo) relationInfoMap.get(goldRel.keyStr);
		    relationInfo.missSamples.add(goldRel);
		    debugWriteln(++count +") miss goldRel " + goldRel);
	    }
	}
	count=0;
	for (int i=0; i<testRelations.size(); i++) {
	    Relation testRel=(Relation)testRelations.get(i);
	    if (!testRel.matched) {
		    RelationInfo relationInfo=(RelationInfo) relationInfoMap.get(testRel.keyStr);
		    relationInfo.falseAlarmSamples.add(testRel);
		    debugWriteln(++count +") false alarm testRel " + testRel);
	    }
	}
    }
	
    /*
    String makeKey(Relation rel) {
	String parentLabel=rel.parentLabelNoAugs.toString();
	String headLabel=(rel.headLabelIsPreterminal ? TAGStr : rel.headLabelNoAugs.toString());
	String modLabel=(rel.modLabelIsPreterminal ? TAGStr : rel.modLabelOnlyDefaultArg.toString());
			  
	return 
	    parentLabel+relDelimStr+
	    headLabel+relDelimStr+
	    modLabel+relDelimStr+
	    translateDirection(rel.direction);
    }
    */
	    
    // assume could be real null, not just (( stuff
    private static boolean treeIsNull(Sexp tree) {
	if (tree.isSymbol() &&
	    tree.symbol().toString().equals("null"))
	    return true;
	else 
	    return false;
    }


    /*    private static boolean treeIsNull(Sexp tree) {
	if (tree.isList()) {
	    return (!tree.list().get(0).isSymbol());
	} 
	return false;
    }
    */

    private static Sexp adjustForParens(Sexp tree) {
	boolean stripOuterParens = (tree.list().length() == 1 &&
				    tree.list().get(0).isList());
	if (stripOuterParens)
	    tree = tree.list().get(0);
	
	return tree;
    }
	

	
    public void collectRelations(int sentNum,
				 boolean isGold,
				 HeadTreeNode2 tree,ArrayList relationsList) {


	if (tree.preMods().size()>0) {
	    collectModifiers(sentNum,isGold,
			     tree,tree.preMods(),Constants.LEFT,relationsList);
	}
	if (tree.postMods().size()>0) {
	    collectModifiers(sentNum,isGold,
			     tree,tree.postMods(),Constants.RIGHT,relationsList);
	}
	if (!tree.isPreterminal()) {
	    collectRelations(sentNum,isGold,
			     tree.headChild(),
			     relationsList);
	}
	Iterator mods=tree.preMods().iterator();
	while (mods.hasNext()) {
	    collectRelations(sentNum,isGold,
			     (HeadTreeNode2)mods.next(),
			     relationsList);
	}
	mods=tree.postMods().iterator();
	while (mods.hasNext()) {
	    collectRelations(sentNum,isGold,
			     (HeadTreeNode2)mods.next(),
			     relationsList);
	}

    }
    public void collectModifiers(int sentNum,
				 boolean isGold,
				 HeadTreeNode2 tree, List modifiers, boolean side,
				 ArrayList relationsList) {
	Symbol parentLabel=tree.label();
	Symbol parentLabelNoAugs=Language.treebank().stripAugmentation(parentLabel);
	Symbol headLabel=tree.headChild().label();
	Symbol headLabelNoAugs=Language.treebank().stripAugmentation(headLabel);
	boolean headLabelIsPreterminal = tree.headChild().isPreterminal();
	int headWordIdx=tree.headWordIdx();
	Symbol headWordWord=tree.headWord().word();

	/*
	pass in info on whether it's a pretimerinal or not
	conversion of -A and -SBJ to -A?
	*/

	/* if base NP, just skip punctuation
	   if not, keep track of previous punc and conj
	*/
	
	Word prevPunc = null, prevConj = null;
	Iterator mods = modifiers.iterator();	
	while (mods.hasNext()) {
	    HeadTreeNode2 mod = (HeadTreeNode2)mods.next();	
	    int modIdx=mod.headWordIdx();
	    Symbol modLabel=mod.label();
	    Symbol modLabelOnlyDefaultArg=
		includeAMarking ? stripAllButDefaultArg(modLabel,new Nonterminal()) :
		Language.treebank.stripAugmentation(modLabel);
	    Symbol modWordWord=mod.headWord().word();
	   
	    boolean isBaseNP=Language.treebank.isBaseNP(parentLabelNoAugs);
	    boolean isConj = Language.treebank.isConjunction(modLabel);
	    boolean isPunc= Language.treebank.isPunctuation(modLabel);
	    if ((isConj || isPunc) &&
		(prevConj!=null || prevPunc!=null)) {
		debugWriteln("warning: more than two conj/punc in a row " + mod);
	    } 

	    if ( (isBaseNP && !isPunc)  ||
		 (!isBaseNP && !isConj && !isPunc)) {
		
		/*
		  debugWriteln(parentLabel+" " +
		  headLabel+" " +
		  modLabel+ " " +
		  (side==Constants.LEFT ? "L " : "R ") + 
		  modIdx + " " +
		  headWordIdx);
		*/
		
		Relation rel=new Relation(sentNum,
					  isGold,
					  modIdx,
					  modWordWord,
					  headWordIdx,
					  headWordWord,
					  parentLabel,
					  parentLabelNoAugs,
					  headLabel,
					  headLabelNoAugs,
					  headLabelIsPreterminal,
					  modLabel,
					  modLabelOnlyDefaultArg,
					  mod.isPreterminal(),
					  side,
					  prevConj,
					  prevPunc);
		relationsList.add(rel);
	    }
	    if (!isBaseNP) {
		prevConj=isConj? mod.headWord() : null;
		prevPunc=isPunc? mod.headWord() : null;
	    }
	}
    }



	    
    Symbol stripAllButDefaultArg(Symbol label, Nonterminal nonterminal) {
	boolean isArgument = Language.training.isArgumentFast(label);
	Symbol strippedLabel=
	    Language.treebank.stripAugmentation(label);
	Nonterminal parsedLabel =
	    Language.treebank.parseNonterminal(strippedLabel, nonterminal);
	if (isArgument) {
	    Language.treebank.addAugmentation(parsedLabel,defaultArgAugmentation);	
	}
	return parsedLabel.toSymbol();
	
    }


    public void process(String goldFilename,String testFilename,
			String resultsFilename,
			String resultsDebugFilename,
			boolean includeAMarking) {
	System.out.println("goldFilename="+goldFilename);
	System.out.println("testFilename="+testFilename);
	System.out.println("resultsFilename="+resultsFilename);
	System.out.println("resultsDebugFilename="+resultsDebugFilename);
	System.out.println("includeAMarking="+includeAMarking);


	this.goldFilename=goldFilename;
	this.testFilename=testFilename;
	this.includeAMarking=includeAMarking;
	String encoding = Language.encoding();
	int defaultFileBufsize=81920;
	
	try {
	    outResults = new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
			FileOutputStream(resultsFilename)));
	    outResultsDebug = new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
			FileOutputStream(resultsDebugFilename)));

	    debugWriteln("goldFilename="+goldFilename);
	    debugWriteln("testFilename="+testFilename);
	debugWriteln("resultsFilename="+resultsFilename);
	debugWriteln("resultsDebugFilename="+resultsDebugFilename);	    

	    process(new SexpTokenizer(goldFilename, encoding, defaultFileBufsize),
		    new SexpTokenizer(testFilename, encoding, defaultFileBufsize),
		    true,false);

	    outResults.close();
	    outResultsDebug.close();
	}
	catch (UnsupportedEncodingException e) {
	    System.err.println(e);
	}
	catch (FileNotFoundException e) {
	    System.err.println(e);
	}
	catch (IOException e) {
	    System.err.println(e);
	}
    }
	

  private static void usage() {
      System.err.println("bad usage");
      System.exit(1);
  }
  /**
   * Takes arguments according to the usage as specified in {@link #usageMsg}.
   */
  public static void main(String[] args) {
	String goldFilename=null;
	String testFilename=null;
	String resultsFilename=null;
	String resultsDebugFilename=null;
	boolean debug=false;
	boolean includeAMarking=false;

	for (int i=0; i<args.length; i++) {
	    if (args[i].charAt(0)=='-') {
		if (args[i].equals("-g")) {
		    if (i+1==args.length)
			usage();
		    goldFilename=args[++i];
		} else if (args[i].equals("-t")) {
		    if (i+1==args.length)
			usage();
		    testFilename=args[++i];		    
		} else if (args[i].equals("-a")) {
		    includeAMarking=true;
		}
		
	    }
	}
	String extraFilename=".dep";
	if (resultsFilename==null) {
	    //	    resultsFilename=testFilename+".pd.results";	    
	    resultsFilename=testFilename+extraFilename+".results";	    
	}
	if (resultsDebugFilename==null) {
	    //	    resultsFilename=testFilename+".pd.results";	    
	    resultsDebugFilename=resultsFilename+".debug";
	}

	AnalyzeDependencies analyzeDep=new AnalyzeDependencies();
	analyzeDep.process(goldFilename,testFilename,
			   resultsFilename,
			   resultsDebugFilename,
			   includeAMarking);

  }

    static final String blanks="                                   ";
    static String rightJustify(int len, int num) {
	return rightJustify(len,num+"");
    }

    static String rightJustify(int len, String t) {
	if (t.length()>len) {
	    return t;
	} else {
	    return (blanks.substring(0,len-t.length())+t);
	}
    }
    static String makePer(int num, int den) {
	double t  =
	    (den==0 ? 0.0 : (1.00*num/den));
	return (rightJustify(6,percentage.format(t)));
    }


    static String leftJustify(int len, String t) {
	if (t.length()>len) {
	    return t;
	} else {
	    return (t+blanks.substring(0,len-t.length()));
	}
    }
    static String leftJustify(int len, int num) {
	return leftJustify(len,num+"");
    }



}
