package danbikel.parser;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

public class LookFor {

    private static Training training = Language.training();
    private static Treebank treebank = Language.treebank();

static final Symbol VP = Symbol.add("VP");
    /*
    static final String parent1="NP";
    static final String parent2="VP";
    static final String modifier="NP-TMP";
    */

    static final String parent1="VP";
    static final String parent2="NP";
    static final String modifier="NP-TMP";

  protected void process(Sexp tree) {
      if (treebank.isPreterminal(tree)) 
	  return;
      
      SexpList treeList = tree.list();
      Symbol currLabel = treeList.symbolAt(0);
      boolean hasModifier=hasModifier(treeList);
      if (hasModifier) {
	boolean isParent1=(treebank.getCanonical(treeList.first().symbol()).toString().equals(parent1));
	boolean isParent2=(treebank.getCanonical(treeList.first().symbol()).toString().equals(parent2));
	if (isParent1) {
	    System.out.println(parent1+">>"+modifier);
	    System.out.println(danbikel.parser.util.Util.prettyPrint(tree));
	} else if (isParent2) {
	    System.out.println(parent2+">>"+modifier);
	    System.out.println(danbikel.parser.util.Util.prettyPrint(tree));	    
	}
      }
      
      for (int i = 1; i < treeList.length(); i++) {
	  process(treeList.get(i));
      }
  }
    
    boolean hasModifier(SexpList treeList) {
      for (int i = 1; i < treeList.length(); i++) {
	    Sexp child=treeList.get(i);	  
	    if (child.list().symbolAt(0).toString().equals(modifier)) {
		return true;
	    }
      }
      return false;
    }

	

    static final String blanks="                                   ";    
	static String leftJustify(int len, String t) {
	    if (t.length()>len) {
		return t;
	    } else {
		return (t+blanks.substring(0,len-t.length()));
	    }
	}
	static String leftJustify(int len, int num) {
	    return leftJustify(len,num+"");
	}    
    
    protected String getNewTags(SexpList treeList) {
	String ret="";
	String key="";
	for (int i = 1; i < treeList.length(); i++) {
	    Sexp child=treeList.get(i);
	    if (!(treebank.isPreterminal(child)))
		return null;
	    if (ret.equals("")) {
		ret=getNewTag(child.list().symbolAt(0).toString());
		key=getOldTagKey(child.list().symbolAt(0).toString());
	    } else {
		ret+="&"+getNewTag(child.list().symbolAt(0).toString());		
		key+="&"+getOldTagKey(child.list().symbolAt(0).toString());
	    }
	}
	return (key+"&&"+ret);
	    
    }
    
    
    
    protected String getNewTag(String tag) {
	int index=tag.indexOf("[");
	String newTag=tag.substring(index+1,tag.length()-1);
	return newTag;
    }
    protected String getOldTagKey(String tag) {
	int index=tag.indexOf("[");
	String oldTag=tag.substring(0,index);
	String[] parts=oldTag.split("\\+");
	String key="";
	for (int i=0; i<parts.length;i++) {
	    if ( (!(parts[i].startsWith("CASE"))) &&
		 (!(parts[i].startsWith("NSUFF")))) {
		if (key.equals("")) {
		    key=parts[i];
		} else {
		    key+="+"+parts[i];
		}
	    }
	}
	return key;
    }

    public void process(String filename) {

	try {
	    //	    Training3 training = (Training3)Language.training();
	    SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
						  Constants.defaultFileBufsize);
	    Sexp curr = null;
	    while ((curr = Sexp.read(tok)) != null) {
		//		System.out.println("curr="+curr);
		process(curr);
	    }
	    
	}
	catch (UnsupportedEncodingException uee) {
	    System.err.println(uee);
	}
	catch (FileNotFoundException fnfe) {
	    System.err.println(fnfe);
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    
	}
    }

	

  public static void main(String[] args) {
    String filename = null;

    filename = args[0];
    LookFor lookFor=new LookFor();
    lookFor.process(filename);
  }
}
