package danbikel.parser.arabic;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;
// (PUNC [ PUNC nogloss [DEFAULT] [ -LRB-)
/*
		(gold pos tag PV+PVSUFF_SUBJ:3FS 
		 vocShrunk taDArab+tu;ta;ti;at 
                 posShrunk PV+PVSUFF_SUBJ:[1S][2MS][2FS][3FS] 
                 gloss clash/fight@+@it/they/she@[verb] 
                 lemma [taDArab_1] 
                 vocalized taDArab+at 
                 unvocalized tDArbt
		 ) 
make sure running with substiutecolosedclassitem
strip hyphens from vocalized
problem: not stripping parens from posShrunk, for (null) case

1. (gold postag, unvocalized) - as before, w/reduction   06
2. (gold postag, vocalized) - w/reduction                05
3. (gold postag, shrunk) - w/reduction                   01
4. (shrunk postag, shrunk) - w/o reduction               21
5. (shrunk postag, shrunkstemmed)                        27
6. (shrunk postag, shrunkstemmed2)                       28
7. (shrunk postag, lemma)                                24

4. (gold postag, shrunk w/o AF Case) - w/reduction

and also run 1,2,3 w/o any reduction
4. (shrunk postag, unvocalized) - w/o reduction
5. (shrunk postag, vocalized) - w/o reduction

7. (shrunk postag w/o case, shurnk w/o AF Case) - w/o reduction

*/

class ConvertTreebank {

    static final int goldPosIndex=0;
    static final int vocShrunkIndex=1;
    static final int posShrunkIndex=2;
    static final int glossIndex=3;
    static final int lemmaIndex=4;
    static final int vocIndex=5;
    static final int unvocIndex=6;
    static final int vocShrunkStemmed1Index=7; // have to create
    static final int vocShrunkStemmed2Index=8; // no DET either


    static int posUseIndex=-1;
    static int wordUseIndex=-1;

    static String defaultExtension="";
    static String[] comparativesElements={"kl","Ay","bED",">y","jmyE","bDEp",">fdH"};
    static Set comparatives=new HashSet();
    static {
	for (int i=0; i<comparativesElements.length; i++) {
	    comparatives.add(Symbol.add(comparativesElements[i]));
	}
    }



  protected Treebank treebank = Language.treebank();

    public ConvertTreebank(int posUseIndex,int wordUseIndex) {
	this.posUseIndex=posUseIndex;
	this.wordUseIndex=wordUseIndex;
    }


    String trimHyphen(String tmp) {
	if (tmp.equals("-"))
	    return tmp;
	if (tmp.equals("-NONE-")) 
	    return tmp;
	if (tmp.equals("-LRB-")) 
	    return tmp;
	if (tmp.equals("-RRB-")) 
	    return tmp;

	
	    //	System.out.println("trimHyphen"+tmp);
	String tmps=tmp;
	if (tmp.substring(0,1).equals("-")) 
	    tmp=tmp.substring(1);
	if (tmp.substring(tmp.length()-1,tmp.length()).equals("-"))
	    tmp=tmp.substring(0,tmp.length()-1);
	//	if (!tmp.equals(tmps))
	    //	    System.out.println("trimmed  " + tmp+ " to " + tmps);
	return tmp;
	
    }
    String[] corePartsComplete={"DET","NOUN","NOUN_PROP","ADJ",
				"PV_PASS","ADV","DET","IV","IV_PASS",
				"NEG_PART","NUM","PREP","PUNC","CONJ",
				"SUB_CONJ","REL_PRON","ABBREV",
				"VERB_PART","PART","REL_ADV","EXCEPT_PART",
				"FOCUS_PART","INTERJ","RC_PART","FUT_PART",
				"EMPH_PART","INTERROG_PART","PV","POSS_PRON",
				"DEM","CV"};

    String[] corePartsPrefix={"PVSUFF_DO","IVSUFF_DO","CVSUFF_DO","POSS_PRON","DEM_PRON","PRON"};

    boolean isCorePart(String[] posShrunkParts, int index) {
	// special handling to not include DET if pat of DET+NOUN,DET+ADJ, etc.
	String part=posShrunkParts[index];
	if (wordUseIndex==vocShrunkStemmed2Index &&
	    part.equals("DET")) {
	    if (index+1<posShrunkParts.length) {
		String next=posShrunkParts[index+1];
		if (next.equals("NOUN") ||
		    next.equals("NOUN_PROP") ||
		    next.equals("ADJ") ||
		    next.equals("NUM")) {
		    return false;
		}
	    }
	}
		    
	for (int i=0; i<corePartsComplete.length; i++) {
	    if (part.equals(corePartsComplete[i])) {
		return true;
	    }
	}
	for (int i=0; i<corePartsPrefix.length; i++) {
	    if (part.startsWith(corePartsPrefix[i])) {
		return true;
	    }
	}
	return false;
    }

    Symbol getLemma(SexpList treeList) {
    	Symbol word=treeList.get(lemmaIndex).symbol();
	if (word.toString().equals("[clitics]") ||
	    word.toString().equals("[DEFAULT]")) {
	    word=getVoc(treeList);
	}
	return word;
    }
	    
    Symbol getVocShrunk(SexpList treeList) {
	Symbol word=treeList.get(vocShrunkIndex).symbol();
	if (word.toString().equals("null")) {
	    word=treeList.get(unvocIndex).symbol();
	    Symbol goldPos=treeList.get(goldPosIndex).symbol();
	    if (!goldPos.toString().equals("NO_FUNC")) {
		System.err.println("WARNING: getVocShrunk reset word from null to  " + word);
	    }	
	    // even if stem==true, can't do it if no word
	    return word;  
	}
	if (wordUseIndex!=vocShrunkStemmed1Index &&
	    wordUseIndex!=vocShrunkStemmed2Index) {
	    return word;

	}
	String vocShrunkStr=word.toString();
	String posShrunkStr=treeList.get(posShrunkIndex).toString();
	String[] vocShrunkParts=vocShrunkStr.split("\\+");
	String[] posShrunkParts=posShrunkStr.split("\\+");
	if (vocShrunkParts.length!=posShrunkParts.length) {
	    System.err.println("shrunk parts not equal " + treeList);
	}
	String vocShrunkStemmed="";
	for (int i=0; i<posShrunkParts.length; i++) {
	    if (isCorePart(posShrunkParts,i)) {
		if (vocShrunkStemmed.equals("")) {
		    vocShrunkStemmed=vocShrunkParts[i];
		} else {
		    vocShrunkStemmed+="+"+vocShrunkParts[i];
		}
	    }
	}
	if (vocShrunkStemmed.equals("")) {
	    vocShrunkStemmed=treeList.get(unvocIndex).symbol().toString();
	    System.err.println("reset word from null to " + word);
	    System.err.println("WARNING: stemmed " + treeList + " stem is empty vocShrunkIndex="+treeList.get(vocShrunkIndex) +
			       "rest to " + vocShrunkStemmed);
	}
		
	return Symbol.get(vocShrunkStemmed);

    }

    Symbol getVoc(SexpList treeList) {
	Symbol word=treeList.get(vocIndex).symbol();
	word= Symbol.get(trimHyphen(word.toString()));
	if (word.toString().equals("novocalized")) {
	    word=treeList.get(unvocIndex).symbol();
	    Symbol goldPos=treeList.get(goldPosIndex).symbol();
	    if (!goldPos.toString().equals("NO_FUNC")) {
		System.err.println("WARNING: getVoc unvocalized word " + treeList);
	    }	
	}
	return word;
    }

    protected Symbol getWordToUse(SexpList treeList) {
	Symbol word=null;

	if (wordUseIndex==vocShrunkIndex) {
	    word=getVocShrunk(treeList);
	} else if (wordUseIndex==vocIndex) {
	    word=getVoc(treeList);
	} else if (wordUseIndex==vocShrunkStemmed1Index) {
	    word=getVocShrunk(treeList);
	    //	    System.out.println("stemmed " + treeList + " = " + word);
	} else if (wordUseIndex==vocShrunkStemmed2Index) {
	    word=getVocShrunk(treeList);
	} else if (wordUseIndex==lemmaIndex) {
	    word=getLemma(treeList);
	} else {
	    word=treeList.get(wordUseIndex).symbol();
	}

	return word;
    }

    protected Symbol getPosToUse(SexpList treeList) {
	Symbol pos=treeList.get(posUseIndex).symbol();	
	if (posUseIndex==posShrunkIndex && pos.toString().equals("null")) {
	    pos=treeList.get(goldPosIndex).symbol();
	    if (!pos.toString().equals("NO_FUNC")) {
		System.err.println("WARNING: replacing posShrunk null with  " + pos);
	    }
	}
	return pos;
    }
	
  protected void process(Sexp tree) {
      if (treebank.isPreterminal(tree)) {
	    SexpList treeList = tree.list();

	    Symbol word=getWordToUse(treeList);
	    Symbol pos=getPosToUse(treeList);

	    for (int i=0; i<7; i++) {
		treeList.remove(0);
	    }
	    treeList.add(word);
	    treeList.add(0,pos);

	    /*
	    Symbol tag=tree.list().get(fullPosIndex).symbol();
	    Symbol word=tree.list().get(1).symbol();
	    if (comparatives.contains(word)) {
		//		System.out.println("found comparative at " + tree);
		String tagStr=tag.toString();
		int index=tagStr.indexOf("+");
		String newTagStr="CMP";
		if (index>-1) {
		    newTagStr+=tagStr.substring(index);
		}
		treeList.set(0,Symbol.add(newTagStr));
	    }
	    */
      }
      else if (tree.isList()) {
	    SexpList treeList = tree.list();	  
	  for (int i = 1; i < treeList.length(); i++) {
	      process(treeList.get(i));
	  }
      }
  }

    public void process(String filename)
	throws FileNotFoundException,IOException {
	System.err.println("reading file"+filename);
	int index=filename.lastIndexOf("/");
	
	try {
	    //	    Training3 training = (Training3)Language.training();
	    SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
						  Constants.defaultFileBufsize);
	    Sexp curr = null;
	    while ((curr = Sexp.read(tok)) != null) {
		//		System.out.println("curr="+curr);
		process(curr);
		System.out.println(curr);
	    
	    }
	}
	catch (UnsupportedEncodingException uee) {
	    System.err.println(uee);
	}
    }


	
    public void process(List files) 
	throws FileNotFoundException,IOException {

	for (Iterator fileIter = files.iterator(); fileIter.hasNext(); ) {
	    String baseFileName=(String)fileIter.next();
	    //	    String inFileName=baseFileName+defaultExtension;
	    process(baseFileName);
	}
	/*
	System.out.println("sorting results");
	sortResults();
	System.out.println("showing results");
	showResults();
	*/
    }


    public static void main(String[] args) {
	String filename = null;
	ArrayList dir=new ArrayList();
	int posUseIndex=Integer.parseInt(args[0]);
	int wordUseIndex=Integer.parseInt(args[1]);
	System.err.print ("using pos="+posUseIndex+" word="+wordUseIndex);
	dir.add(new File(args[2]));
	//	System.out.println("added"+args[0]+"to dir");
	List files=danbikel.parser.util.SKUtil.expandDirectories(dir,defaultExtension,false,false);
	try {
	    ConvertTreebank convertTreebank=new ConvertTreebank(posUseIndex,wordUseIndex);
	    convertTreebank.process(files);
	}
	catch (FileNotFoundException fnfe) {
	System.err.println(fnfe);
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    
	}
    }

}
