package danbikel.parser.arabic;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

class NPBInfo {
    String fullTags=null;
    String reducedTags=null;
    String words=null;
    String fileName=null;
    int sentNum=0;

}

class TreeMapKey {
    int numEntries;
    String key;
    
    public TreeMapKey(int numEntries, String key) {
	this.numEntries=numEntries;
	this.key=key;
    }
}
class Two {
    String key;
    ArrayList entries;
    
    public Two(String key,ArrayList entries) {
	this.key=key;
	this.entries=entries;
    }
}


public class ExamineNPs extends danbikel.parser.arabic.Training3 {

    /* 
       if true, then preterminals look like (unreducedtag[reduced tag] token)

    */
    boolean bothTags=false;
    static String defaultExtension=".parsed";
    //    String defaultExtension=".tree";

    
    int count=0;
    String currentFilename="";
    int currentSentNum=0;

    java.util.HashMap map=null;
    Comparator c= new Comparator() {
	    public int compare(Object obj1,Object obj2) {
		TreeMapKey o1=(TreeMapKey)obj1;
		TreeMapKey o2=(TreeMapKey)obj2;
		if (o1.numEntries<o2.numEntries) {
		    return 1;
		} else if (o1.numEntries>o2.numEntries) {
		    return -1;
		} else {
		    return (o1.key).compareTo(o2.key);
		}
	    }
	};
    TreeMap sortedEntries=new TreeMap(c);

    public void sortResults() {
	int count2=0;
	Iterator keys=map.keySet().iterator();
	while (keys.hasNext()) {
	    String key=(String)keys.next();
	    ArrayList entries=(ArrayList)map.get(key);
	    if (entries==null) {
		System.out.println("no entry  for key=" +key);
	    }
	    //	    System.out.println("adding in entry size="+entries.size()+" key="+key);
	    count2+=entries.size();
	    sortedEntries.put(new TreeMapKey(entries.size(),key),entries);
	}
	System.out.println("shoudl be "  +count + " was " + count2);
    }

	    
    void showResults() {
	Iterator keys=sortedEntries.keySet().iterator();
	while (keys.hasNext()) {
	    TreeMapKey key=(TreeMapKey)keys.next();
	    ArrayList entries=(ArrayList)sortedEntries.get(key);
	    System.out.println(leftJustify(30,key.key) + rightJustify(6,entries.size()));
	}
	System.out.println("-----------------------------------------------------");
	keys=sortedEntries.keySet().iterator();
	while (keys.hasNext()) {
	    TreeMapKey key=(TreeMapKey)keys.next();
	    System.out.println("-----------------------------------------------------");
	    ArrayList entries=(ArrayList)sortedEntries.get(key);
	    System.out.println(leftJustify(30,key.key) + rightJustify(6,entries.size()));
	    
	    int maxLenWords=0;
	    int maxLenFullTags=0;
	    int max=(entries.size()>100) ? 100 : entries.size();
	    if (key.key.startsWith("NN DT") ||
		key.key.startsWith("NN NN")) {
		max=entries.size();
	    }
	    for (int i=0; i<max; i++) {
		NPBInfo npbInfo=(NPBInfo)(entries.get(i));
		if (npbInfo.words.length()>maxLenWords)
		    maxLenWords=npbInfo.words.length();
		if (npbInfo.fullTags.length()>maxLenFullTags)
		    maxLenFullTags=npbInfo.fullTags.length();
	    }

	    for (int i=0; i<max; i++) {
		NPBInfo npbInfo=(NPBInfo)(entries.get(i));
		System.out.println(leftJustify(4,i+") ")+ 
				   leftJustify(maxLenWords+2,npbInfo.words)+
				   "    "+
				   //				   leftJustify(20,npbInfo.reducedTags)+
				   //				   "    "+
				   leftJustify(maxLenFullTags+2,npbInfo.fullTags)+
				   leftJustify(20,npbInfo.fileName+"/"+npbInfo.sentNum));
	    }
	}
    }
    
      
    public ExamineNPs(boolean bothTags) throws FileNotFoundException, IOException {
	super();
	System.err.println("in ExamineNPs");
	map=new java.util.HashMap();
	sortedEntries=new TreeMap(c);
	this.bothTags=bothTags;
    }

  protected void process(Sexp tree) {
      if (treebank.isPreterminal(tree)) {
	  /*
	    SexpList treeList = tree.list();
	    Symbol currLabel = treeList.symbolAt(0);
	    treeList.set(0, treebank.getCanonical(currLabel));
	  */
	  return;
      }
      else if (tree.isList()) {
	  SexpList treeList = tree.list();
	  Symbol currLabel = treeList.symbolAt(0);
	  boolean isNP=treebank.isNP(treeList.first().symbol());
	  if (isNP) {
	      NPBInfo npbInfo=getTags(treeList);
	      if (npbInfo!=null) {
		  count++;
		  npbInfo.fileName=currentFilename;
		  npbInfo.sentNum=currentSentNum;
		  //		  System.out.println(leftJustify(40,allNewTags)+"   " + tree);
		  ArrayList entries=(ArrayList)map.get(npbInfo.reducedTags);
		  if(entries==null) {
		      map.put(npbInfo.reducedTags,new ArrayList());
		      entries=(ArrayList)map.get(npbInfo.reducedTags);
		      //		      System.out.println("just put in null entry for key="+key);
		  }
		  entries.add(npbInfo);
	      } else {
		  //		  System.out.println("null      "+tree);
	      }
	  }
	  for (int i = 1; i < treeList.length(); i++) {
	      process(treeList.get(i));
	  }
      }
  }
    static final String blanks="                                                                                                               ";    
    static String leftJustify(int len, String t) {
	if (t.length()>len) {
	    return t;
	} else {
	    return (t+blanks.substring(0,len-t.length()));
	}
    }
    static String leftJustify(int len, int num) {
	    return leftJustify(len,num+"");
    }    
    
    static String rightJustify(int len, int num) {
	return rightJustify(len,num+"");
    }
    
    static String rightJustify(int len, String t) {
	if (t.length()>len) {
	    return t;
	} else {
	    return (blanks.substring(0,len-t.length())+t);
	}
    }
    
    protected NPBInfo getTags(SexpList treeList) {
	NPBInfo npbInfo=new NPBInfo();
	npbInfo.fullTags="";
	npbInfo.reducedTags="";
	npbInfo.words="";
	for (int i = 1; i < treeList.length(); i++) {
	    Sexp child=treeList.get(i);
	    if (!(treebank.isPreterminal(child)))
		return null;
	    
	    if (npbInfo.reducedTags.equals("")) {
		npbInfo.reducedTags=getNewTag(child.list().symbolAt(0).toString());
		npbInfo.fullTags=getOldTagKey(child.list().symbolAt(0).toString());
		npbInfo.words=child.list().symbolAt(1).toString();
	    } else {
		npbInfo.reducedTags+=" "+getNewTag(child.list().symbolAt(0).toString());		
		npbInfo.fullTags+=" "+getOldTagKey(child.list().symbolAt(0).toString());
		npbInfo.words+=" " + child.list().symbolAt(1).toString();
	    }
	}
	return npbInfo;
	    
    }
	
    
    protected String getNewTag(String tag) {
	if (!bothTags) 
	    return tag;

	int index=tag.indexOf("[");
	String newTag=tag.substring(index+1,tag.length()-1);
	return newTag;
    }
    protected String getOldTagKey(String tag) {
	if (!bothTags) 
	    return "";
	int index=tag.indexOf("[");
	String oldTag=tag.substring(0,index);
	String[] parts=oldTag.split("\\+");
	String key="";
	for (int i=0; i<parts.length;i++) {
	    //	    if ( (!(parts[i].startsWith("CASE"))) &&
	    //		 (!(parts[i].startsWith("NSUFF")))) {
	    if (true) {
		if (key.equals("")) {
		    key=parts[i];
		} else {
		    key+="+"+parts[i];
		}
	    }
	}
	return key;
    }

    public void process(String filename)
	throws FileNotFoundException,IOException {
	System.err.println("reading file"+filename);
	int index=filename.lastIndexOf("/");
	currentFilename=filename.substring(index+1);
	currentSentNum=0;
	
	try {
	    //	    Training3 training = (Training3)Language.training();
	    SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
						  Constants.defaultFileBufsize);
	    Sexp curr = null;
	    while ((curr = Sexp.read(tok)) != null) {
		//		System.out.println("curr="+curr);
		process(curr);
		currentSentNum++;
	    
	    }
	    
	    
	}
	catch (UnsupportedEncodingException uee) {
	    System.err.println(uee);
	}
    }
    
	
    public void process(List files) 
	throws FileNotFoundException,IOException {

	for (Iterator fileIter = files.iterator(); fileIter.hasNext(); ) {
	    String baseFileName=(String)fileIter.next();
	    //	    String inFileName=baseFileName+defaultExtension;
	    process(baseFileName);
	}
	System.out.println("sorting results");
	sortResults();
	System.out.println("showing results");
	showResults();
    }


    public static void main(String[] args) {
	System.out.println("in ExamineNPs main");
	String filename = null;
	boolean bothTags= (args[0].equals("true")) ? true : false;
	ArrayList dir=new ArrayList();
	dir.add(new File(args[1]));
	System.out.println("added"+args[1]+"to dir");
	List files=danbikel.parser.util.SKUtil.expandDirectories(dir,defaultExtension,false,false);
	try {
	    ExamineNPs examineNPs=new ExamineNPs(bothTags);
	    examineNPs.process(files);
	}
	catch (FileNotFoundException fnfe) {
	System.err.println(fnfe);
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    
	}
    }
    
}
