/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
    package danbikel.parser.arabic;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

/**
 * Provides methods for language-specific processing of training parse trees.
 * Even though this subclass of {@link danbikel.parser.Training} is
 * in the default English language package, its primary purpose is simply
 * to fill in the {@link #argContexts}, {@link #semTagArgStopSet} and
 * {@link #nodesToPrune} data members using a metadata resource.  If this
 * capability is desired in another language package, this class may be
 * subclassed.
 * <p>
 * This class also re-defined the method
 * {@link danbikel.parser.lang.AbstractTraining#hasPossessiveChild(Sexp)}.
 */
public class Training3 extends danbikel.parser.arabic.Training2 {

    /* ASK DAN */
    /* why isn't nontemrinal in Training protected?
     */


  public Training3() throws FileNotFoundException, IOException {
      super();
      System.err.println("in arabic Training 3");
  }

    /* I must have added this only because
       when I was doing the stuff with the new training/test
       setup in which I preprocess the tags i didn't want
       it to go through this each time.
       But this is inapprorpiate for files form ldc for 
       parsing which need to go through the tag transformation

    */
    /*
  public SexpList preProcessTest(SexpList sentence,
				 SexpList originalWords, SexpList tags) {
    if (tags == null)
      return new SexpList(1).add(sentence);
    else
      return new SexpList(2).add(sentence).add(tags);
  }
    */
  public Sexp preProcess(Sexp tree) {
    transformTags(tree);
    prune(tree);
    addBaseNPs(tree);
    //repairBaseNPs(tree);
    //addGapInformation(tree);
    //relabelSubjectlessSentences(tree);
    removeNullElements(tree);
    raisePunctuation(tree);
    identifyArguments(tree);
    //    System.out.println("strip Indicies before" + tree);
    stripIndices(tree);
    //    System.out.println("strip Indicies after " + tree);
    return tree;
  }



  private final static String[] usageMsg = {
    "bad usage"
  };

  private static void usage() {
    for (int i = 0; i < usageMsg.length; i++) {
      System.err.println(usageMsg[i]);
    }
    System.exit(1);
  }

  /** Test driver for this class. */
  public static void main(String[] args) {
    String filename = null;
    boolean transformTags = false;
    boolean prune = false;
    boolean addBaseNPs = false;
    boolean removeNullElements = false;
    boolean raisePunc = false;
    boolean idArgs = false;
    boolean stripAug = false;
    boolean combineTransformations = false;

    for (int i = 0; i < args.length; i++) {
      // for each arg that begins with a '-', examine each of its letters
      // and set booleans accordingly, except for "-combine" flag
      if (args[i].charAt(0) == '-') {
	if (args[i].equals("-combine"))
	  combineTransformations = true;
	else {
	  for (int charIdx = 1; charIdx < args[i].length(); charIdx++) {
	    char curr = args[i].charAt(charIdx);
	    switch (curr) {
	    case 't':
	      transformTags = true;
	      break;
	    case 'p':
	      prune = true;
	      break;
	    case 'b':
	      addBaseNPs = true;
	      break;
	    case 'n':
	      removeNullElements = true;
	      break;
	    case 'r':
	      raisePunc = true;
	      break;
	    case 'i':
	      idArgs = true;
	      break;
	    case 'a':
	      stripAug = true;
	      break;
	    default:
	      System.err.println("illegal flag: " + curr);
	      usage();
	    }
	  }
	}
      }
      else
	filename = args[i];
    }

    if (filename == null) {
      usage();
    }

    //    System.out.println("transformTags="+transformTags);
    Training3 training = (Training3)Language.training();
    //training.printMetadata();

    try {
      SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
					    Constants.defaultFileBufsize);
      Sexp curr = null;
      while ((curr = Sexp.read(tok)) != null) {
	if (transformTags) {
	    curr = training.transformTags(curr);
	    // curr = training.keepTransformedTags(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (prune) {
	  curr = training.prune(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (addBaseNPs) {
	  curr =training.addBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (removeNullElements) {
	  curr = training.removeNullElements(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (raisePunc) {
	  curr = training.raisePunctuation(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (idArgs) {
	  curr =training.identifyArguments(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (stripAug) {
	  curr = training.stripIndices(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (combineTransformations)
	  System.out.println(curr);
      }
    }
    catch (UnsupportedEncodingException uee) {
      System.err.println(uee);
    }
    catch (FileNotFoundException fnfe) {
      System.err.println(fnfe);
    }
    catch (IOException ioe) {
      System.err.println(ioe);
    }
  }

}
