/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
package danbikel.parser.arabic;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

/**
 * Provides methods for language-specific processing of training parse trees.
 * Even though this subclass of {@link danbikel.parser.Training} is
 * in the default English language package, its primary purpose is simply
 * to fill in the {@link #argContexts}, {@link #semTagArgStopSet} and
 * {@link #nodesToPrune} data members using a metadata resource.  If this
 * capability is desired in another language package, this class may be
 * subclassed.
 * <p>
 * This class also re-defined the method
 * {@link danbikel.parser.lang.AbstractTraining#hasPossessiveChild(Sexp)}.
 */
public class Training7 extends danbikel.parser.arabic.Training6 {
    
    protected danbikel.parser.arabic.Treebank3 treebank = 
	(danbikel.parser.arabic.Treebank3)Language.treebank();    
    
    public Training7() throws FileNotFoundException, IOException {
	super();
	System.err.println("in arabic Training 7");
    }
    
    public Sexp preProcess(Sexp tree) {
	//      System.out.println("before transformTags " + tree);
	transformTags(tree);
	//      System.out.println("after transformTags " + tree);
	prune(tree);
	System.out.println("before addNPNumbers\n    " + danbikel.parser.util.Util.prettyPrint(tree));
	addNPNumbers(tree);
	System.out.println("after addNPNumbers  \n    " + danbikel.parser.util.Util.prettyPrint(tree));
	// addBaseNPs(tree);
	//repairBaseNPs(tree);
	//addGapInformation(tree);
	//relabelSubjectlessSentences(tree);
	removeNullElements(tree);
	raisePunctuation(tree);
	identifyArguments(tree);
	stripAugmentations(tree);
	System.out.println("after strip  \n    " + danbikel.parser.util.Util.prettyPrint(tree));
	return tree;
    }

  public void postProcess(Sexp tree) {
    //unrepairBaseNPs(tree);
      //    super.postProcess(tree);
      //      System.out.println("before remove basenps = " + tree);
      // removeOnlyChildBaseNPs(tree);
      //       System.out.println("after remove basenps = " + tree);
    //    canonicalizeNonterminals(tree);
    stripArgAugAndCanonicalizeBases(tree);
  }

}
