package danbikel.parser.arabic;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

class UnifyATB {

    static String withoutVowelPath="/nlp/home/beatrice/arabic/corpus/ATB/ANNAHAR/release/ATB3-v2.0/data/penntree/without-vowel";
    static String withVowelPath="/nlp/home/beatrice/arabic/corpus/ATB/ANNAHAR/release/ATB3-v2.0/data/penntree/with-vowel";
    static String combinedPath="/nlp/home/beatrice/arabic/corpus/ATB/ANNAHAR/release/ATB3-v2.0/data/combined/";
    static String posPath="/nlp/home/beatrice/arabic/corpus/ATB/ANNAHAR/release/ATB3-v2.0/data/pos/after-treebank";


    static String defaultExtension=".tree";
    String posName=null;

    String[] errorStrings={
	"no solution, but no NO_MATCH comment", //0
	"no solution, with NO_MATCH comment", // 1 don't show
	"has NO_MATCH comment, but has solution", //2
	"has solution, but no gloss", //3
	"no lemma,pos,vocalized in solution", //4
	"no lemma but with pos,vocalized in solution, with NO_MATCH", //5
	"no lemma but with pos,vocalized in solution, without NO_MATCH", //6
	"different POS tag, tag is NO_FUNC",//7  don't show
	"different POS tag" //8
    };
    int numErrorTypes=errorStrings.length;
    int[] errors=new int[numErrorTypes];

  protected Treebank treebank = Language.treebank();

    class PosInfo {
	String lineCase;
	String lineWord;
	String lineLemma;
	String lineLemmaStripped;
	String lineGloss;
	String lineGlossStripped;
	String linePos;
	String lineVocalized;
	String lineVocalizedStripped;
	String linePosShrunk;
	String linePosShrunkStripped;
	String lineVocShrunk;
	String lineVocShrunkStripped;

	boolean defaultTakenAsSolution=false;
	String inputString;
	String lookUpWord;
	String comment;
	String index;
	String solution;
	String gloss;

	String normalizedWordStr;
	String normalizedPosStr;
	
    }
    

    int lineInPosFile=0;
    int numPosEntries=0;

    String changeParensSpaces(String str) {
	str=str.replaceAll("\\(","\\[");
	str=str.replaceAll("\\)","\\]");
	str=str.replaceAll(" ","@");	
	return str;
    }
	
    void checkForSpecialParens(PosInfo posInfo, String exc, String wordStr) {
        if (wordStr.equals(exc)) {
	    posInfo.lineVocalizedStripped=exc;
	    posInfo.lineVocShrunkStripped=exc;
	}
    }


    void changeParens(PosInfo posInfo, String wordStr) {
	String oldVocalized=posInfo.lineVocalized;
	String oldLemma=posInfo.lineLemma;
	String oldGlossShrunk=posInfo.lineGloss;
	checkForSpecialParens(posInfo,"-RRB-",wordStr);
	checkForSpecialParens(posInfo,"-LRB-",wordStr);
	checkForSpecialParens(posInfo,"-LCB-",wordStr);
	checkForSpecialParens(posInfo,"-RCB-",wordStr);


	posInfo.lineVocalizedStripped=changeParensSpaces(posInfo.lineVocalizedStripped);
	posInfo.lineLemmaStripped=changeParensSpaces(posInfo.lineLemmaStripped);
	posInfo.lineGlossStripped=changeParensSpaces(posInfo.lineGlossStripped);
	posInfo.linePosShrunkStripped=changeParensSpaces(posInfo.linePosShrunkStripped);
	posInfo.lineVocShrunkStripped=changeParensSpaces(posInfo.lineVocShrunkStripped);
	
	/*

	if (!posInfo.lineVocalized.equals(oldVocalized) ||
	    !posInfo.lineLemma.equals(oldLemma) ||
	    !posInfo.lineGloss.equals(oldGlossShrunk)) {
	    System.out.println("changed something here " + 
			       posInfo.lineVocalized.equals(oldVocalized) + " " +
			       posInfo.lineLemma.equals(oldLemma) + " " +
			       posInfo.lineGloss.equals(oldGlossShrunk));
	    System.out.println(posName);
	    System.out.println("inputString="+posInfo.inputString);
	    System.out.println("lookUpword="+posInfo.lookUpWord);
	    System.out.println("comment="+posInfo.comment);
	    System.out.println("index="+posInfo.index);
	    System.out.println("solution="+posInfo.solution);	
	    System.out.println("gloss="+posInfo.gloss);
	    System.out.println("lemma="+posInfo.lineLemma);
	    System.out.println("glossShrunk="+posInfo.lineGloss);
	}			       
	*/
    }

	
    void verifyMatch(PosInfo posInfo, String wordToFind) {
	String posInfoWord=posInfo.lineWord.substring(6);
	if (posInfoWord.equals("(")) {
	    posInfoWord="-LRB-";
	} else if (posInfoWord.equals(")")) {
	    posInfoWord="-RRB-";
	}
	
	char[] chars=new char[posInfoWord.length()];
	if (!posInfoWord.equals(wordToFind)) {
	    posInfoWord.getChars(0,posInfoWord.length(),chars,0);
	    
	    if (chars.length==2 &&
		chars[0]==216 && chars[1]==140) {
		if (wordToFind.equals(",")) {
		    posInfoWord=",";
		} else {
		    //			System.out.println("special case 1 but word="+word);
		}
	    } else if (chars.length==2 &&
		       chars[0]==216 && chars[1]==159) {
		//		    System.out.println("found special case");
		if (wordToFind.equals("?")) {
		    posInfoWord="?";
		} else {
		    //			System.out.println("special case 2 but word="+word);
		}
	    }
	    
	}

	if (!posInfoWord.equals(wordToFind)) {
	    if (!posInfo.linePos.equals("pos: PUNC")) {
		System.out.println("mismatch word "+wordToFind+" found "+posInfoWord);
		System.out.println("lineInPosFile="+lineInPosFile);
		System.out.println("case="+posInfo.lineCase);
		System.out.println("word="+posInfo.lineWord);
		System.out.println("lemma="+posInfo.lineLemma);
		System.out.println("gloss="+posInfo.lineGloss);
		System.out.println("pos="+posInfo.linePos);
		System.out.println("voc="+posInfo.lineVocalized);
		System.out.println("posShrunk="+posInfo.linePosShrunk);
		System.out.println("vocShrunk="+posInfo.lineVocShrunk);
	    }


//		for (int jj=0; jj<chars.length; jj++) {
//		    System.out.print(" ["+jj+"]="+chars[jj] + " "  + (short)(chars[jj]));
//		}
	} else {
	    //	    System.out.println("match word"+word+"found"+posInfoWord+"blah");
	}
    }


    PosInfo findPosInfo(String tag, String word, BufferedReader posReader) {
	//	System.out.println("findPosInfo");
	PosInfo posInfo=new PosInfo();
	try {
	    String line=posReader.readLine();
	    lineInPosFile++;
	    while (line.trim().equals("") ||
		   line.startsWith("#")) {
		//		System.out.println("just skipped line " + line);
		line=posReader.readLine();
		lineInPosFile++;
	    }
	    posInfo.lineCase=line;
	    posInfo.lineWord=posReader.readLine();
	    posInfo.lineLemma=posReader.readLine();
	    posInfo.lineGloss=posReader.readLine();
	    posInfo.linePos=posReader.readLine();
	    posInfo.lineVocalized=posReader.readLine();
	    posInfo.linePosShrunk=posReader.readLine();
	    posInfo.lineVocShrunk=posReader.readLine();
	    //	    posReader.readLine(); // posGeneralized
	    //	    posReader.readLine(); // vocGeneralized



	    if  (!posInfo.lineVocShrunk.startsWith("vocShrunk")) {
		System.out.println("unexpected line not vocShrunk" + posInfo.lineVocShrunk);
		System.out.println("case="+posInfo.lineCase);
		System.out.println("word="+posInfo.lineWord);
		System.out.println("lemma="+posInfo.lineLemma);
		System.out.println("gloss="+posInfo.lineGloss);
		System.out.println("pos="+posInfo.linePos);
		System.out.println("voc="+posInfo.lineVocalized);
		System.out.println("posShrunk="+posInfo.linePosShrunk);
		System.out.println("vocShrunk="+posInfo.lineVocShrunk);
		
		System.exit(-1);
	    }
	    posInfo.lineLemmaStripped=posInfo.lineLemma.substring(7).trim();
	    posInfo.lineGlossStripped=posInfo.lineGloss.substring(7).trim();
	    posInfo.lineVocalizedStripped=posInfo.lineVocalized.substring(11).trim();
	    posInfo.linePosShrunkStripped=posInfo.linePosShrunk.substring(11).trim();
	    posInfo.lineVocShrunkStripped=posInfo.lineVocShrunk.substring(11).trim();

	    line=posReader.readLine();
	    while (line!=null && !line.trim().equals("")) {
		//		System.out.println("skipping other stuff line " + line);
		line=posReader.readLine();
		lineInPosFile++;
	    }


	    
	}

	catch (IOException e) {
	    System.out.println("Exception "+e );
	    e.printStackTrace();
	}
	return posInfo;

    }

		
    void printWarning(int num, PosInfo posInfo) {
	if (num!=1 && num!=7) {
	    System.out.println("\n"+"Case "+num+":"+ errorStrings[num]);
	    System.out.println(posName);
	    System.out.println("inputString="+posInfo.inputString);
	    System.out.println("lookUpword="+posInfo.lookUpWord);
	    System.out.println("comment="+posInfo.comment);
	    System.out.println("index="+posInfo.index);
	    System.out.println("solution="+posInfo.solution);	
	    System.out.println("gloss="+posInfo.gloss);
	}
	errors[num]++;
    }
	


  protected void process(Sexp tree,BufferedReader posReader) {

      if (treebank.isPreterminal(tree)) {
	  SexpList treeList = tree.list();
	  Symbol currLabel = treeList.symbolAt(0);
	  Symbol tag=tree.list().get(0).symbol();
	  String tagStr=tag.toString();
	  Symbol word=tree.list().get(1).symbol();
	  String wordStr=word.toString();

	  //	  System.out.println("tree="+tree);
	  if (!tag.toString().equals("-NONE-")) {
	      PosInfo posInfo=findPosInfo(tag.toString(),wordStr,posReader);
	      
	      numPosEntries++;
	      if (!posInfo.linePos.substring(5).equals(tagStr)) {
		  if (tagStr.equals("NO_FUNC")) {
		      printWarning(7,posInfo);
		  } else {
		      printWarning(8,posInfo);
		      System.out.println("lemma="+posInfo.lineLemma);
		      System.out.println("posFromPosFile= "+ posInfo.linePos);
		      System.out.println("vocalized=  "+posInfo.lineVocalized);
		      System.out.println("tagStr="+tagStr);
		  }
	      } 


	      /*
	      if (posInfo.solution==null) {
		  System.out.println("no solution found!");
	      }
	      */
	      if (wordStr.equals("_") && 
		  tagStr.equals("PRON_1S") &&
		  posInfo.lineWord.equals("blah")) {
		  System.out.println("adjusting for ("+tagStr+" " +wordStr+")"+ " and blah at " + lineInPosFile + 
				     " in " + posName);
		  posInfo.lineWord=wordStr;
	      }
	      if ((wordStr.equals("_")  || wordStr.equals("____l")) &&
		  (tagStr.equals("NO_FUNC") || tagStr.equals("PUNC") || tagStr.equals("PREP")) &&
		  posInfo.lineWord.equals("blah2")) {
		  System.out.println("adjusting for ("+tag.toString()+" " +wordStr+")"+ " and blah2 at " + lineInPosFile + 
				     " in " + posName);
		  posInfo.lineWord=wordStr;
	      }
	      
	      verifyMatch(posInfo,wordStr);
	      changeParens(posInfo,wordStr);
	      treeList.add(1,Symbol.add(posInfo.lineVocalizedStripped));
	      //	      treeList.add(1,Symbol.add(posInfo.posFromPosFile));
	      treeList.add(1,Symbol.add(posInfo.lineLemmaStripped));
	      treeList.add(1,Symbol.add(posInfo.lineGlossStripped));
	      treeList.add(1,Symbol.add(posInfo.linePosShrunkStripped));
	      treeList.add(1,Symbol.add(posInfo.lineVocShrunkStripped));
	      //	      treeList.add(1,Symbol.add(posInfo.normalizedWordStr));
	      //	      treeList.add(1,Symbol.add(posInfo.normalizedPosStr));
	      /*
		(PV+PVSUFF_SUBJ:3FS taDArab+tu;ta;ti;at PV+PVSUFF_SUBJ:[1S][2MS][2FS][3FS] clash/fight@+@it/they/she@[verb] [taDArab_1] taDArab+at tDArbt) 

		(gold pos tag PV+PVSUFF_SUBJ:3FS 
		 vocShrunk taDArab+tu;ta;ti;at 
                 posShrunk PV+PVSUFF_SUBJ:[1S][2MS][2FS][3FS] 
                 gloss clash/fight@+@it/they/she@[verb] 
                 lemma [taDArab_1] 
                 vocalized taDArab+at 
                 unvocalized tDArbt
		 ) 

                 (PUNC " PUNC nogloss [DEFAULT] " ")

		 (NUM 650 NUM nogloss [DEFAULT] 650 650) 
                 (NOUN+CASE_INDEF_ACC junodiy~+AF NOUN+CASE_INDEF_ACC soldier@+@[acc.indef.] [junodiy~_1] junodiy~+AF jndyAF) 
		 (DET+NOUN+CASE_DEF_GEN Al+qaDA' DET+NOUN the@+@extermination/annihilation@+@[def.gen.] [qaDA'_2] -Al+qaDA'+i AlqDA'))
	      */
	  }

      }
      else if (tree.isList()) {
	  SexpList treeList = tree.list();
	  for (int i = 1; i < treeList.length(); i++) {
	      process(treeList.get(i),posReader);
	  }
      }
  }

    public void process(String withoutVowelName, String posName, String combinedName)
	throws FileNotFoundException,IOException {
	//	System.out.println("reading withoutVowel"+withoutVowelName);
	//	System.out.println("writing to "+combinedName);
	BufferedReader posReader=new BufferedReader(new FileReader(posName));

	BufferedWriter combinedWriter= new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
				       FileOutputStream(combinedName)));
	lineInPosFile=0;
	try {
	    //	    Training3 training = (Training3)Language.training();
	    SexpTokenizer tok = new SexpTokenizer(withoutVowelName, Language.encoding(),
						  Constants.defaultFileBufsize);
	    Sexp curr = null;
	    while ((curr = Sexp.read(tok)) != null) {
		//System.out.println("curr="+curr);
		process(curr,posReader);
		combinedWriter.write(curr+"\n");
		//		System.out.println(curr);
	    
	    }
	}
	catch (UnsupportedEncodingException uee) {
	    System.err.println(uee);
	}
	posReader.close();
	combinedWriter.close();
    }


	
    public void process(List files) 
	throws FileNotFoundException,IOException {

	int numFiles=0;
	for (Iterator fileIter = files.iterator(); fileIter.hasNext(); ) {
	    numFiles++;
	    if (numFiles%50==0) 
	    System.err.println(numFiles);
	    String baseFileName=(String)fileIter.next();
	    String withoutVowelName=baseFileName+defaultExtension;
	    System.out.println("withoutVowelName="+withoutVowelName);
	    posName=withoutVowelName.replaceAll(withoutVowelPath,posPath).replaceAll("\\.tree","\\.txt");
	    String combinedName=withoutVowelName.replaceAll(withoutVowelPath,combinedPath);
	    
	    System.out.println("posName="+posName);
	    System.out.println("combinedname="+combinedName);
	    process(withoutVowelName,posName,combinedName);
	}
	System.out.println("\n numPosEntries="+numPosEntries);
	for (int i=0; i<numErrorTypes; i++) {
	    System.out.println(i+")"+errorStrings[i]+" " +errors[i]);
	}
	/*
	System.out.println("sorting results");
	sortResults();
	System.out.println("showing results");
	showResults();
	*/
    }


    public static void main(String[] args) {
	String filename = null;
	ArrayList dir=new ArrayList();
	//	dir.add(new File(args[0]));
	dir.add(new File(withoutVowelPath));
	//	System.out.println("added"+args[0]+"to dir");
	List files=danbikel.parser.util.SKUtil.expandDirectories(dir,defaultExtension,true,false);
	try {
	    UnifyATB unifyATB=new UnifyATB();
	    unifyATB.process(files);
	}
	catch (FileNotFoundException fnfe) {
	System.err.println(fnfe);
	fnfe.printStackTrace();
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    ioe.printStackTrace();
	    
	}
    }
}
