package danbikel.parser.examine;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

class ContextInfo {
    String filename=null;
    int sentNum=0;
    int childNum=-1;
    Symbol parent=null;
    boolean parentIsNP=false;
    boolean siblingsAllPreterminals=false;
    String[] fullTags=null;
    String[] reducedTags=null;
    String[] words=null;

    String[] glosses=null;
    String[] lemmas=null;
    String[] vocalized=null;
}
    
public abstract class ExamineCore {

    int count=0;
    String currentFilename="";
    int currentSentNum=0;



    boolean bothTags=false;
    static String defaultExtension=".tree";

    public ExamineCore(boolean bothTags) {
	this.bothTags=bothTags;
    }	


    protected void  getContextInfo(SexpList treeList,ContextInfo contextInfo) {
	//	System.out.println("getting contextInfo for " + treeList);
	int treeListLen=treeList.length();

	contextInfo.siblingsAllPreterminals=true;
	contextInfo.reducedTags=new String[treeListLen-1];
	contextInfo.fullTags=(bothTags ? new String[treeListLen] : null);
	contextInfo.words=new String[treeListLen-1];
	contextInfo.siblingsAllPreterminals=true;

	contextInfo.parent=treeList.symbolAt(0);
	contextInfo.parentIsNP=Language.treebank().isNP(treeList.first().symbol());
	contextInfo.filename=currentFilename;
	contextInfo.sentNum=currentSentNum;

	for (int i = 1; i < treeList.length(); i++) {
	    Sexp child=treeList.get(i);
	    if (!(Language.treebank().isPreterminal(child))) {
		contextInfo.siblingsAllPreterminals=false;
		contextInfo.reducedTags[i-1]=child.list().symbolAt(0).toString();
		if (bothTags) 
		    contextInfo.fullTags[i-1]=contextInfo.reducedTags[i-1];
		contextInfo.words[i-1]="(nt)";
	    } else {
		contextInfo.reducedTags[i-1]=getNewTag(child.list().symbolAt(0).toString());
		if (bothTags) 
		    contextInfo.fullTags[i-1]=getOldTagKey(child.list().symbolAt(0).toString());
		contextInfo.words[i-1]=child.list().symbolAt(1).toString();
	    }
	}
    }
	
    
    protected String getNewTag(String tag) {
	if (!bothTags) 
	    return tag;

	int index=tag.indexOf("[");
	String newTag=tag.substring(index+1,tag.length()-1);
	return newTag;
    }
    protected String getOldTagKey(String tag) {
	if (!bothTags) 
	    return "";
	int index=tag.indexOf("[");
	String oldTag=tag.substring(0,index);
	String[] parts=oldTag.split("\\+");
	String key="";
	for (int i=0; i<parts.length;i++) {
	    //	    if ( (!(parts[i].startsWith("CASE"))) &&
	    //		 (!(parts[i].startsWith("NSUFF")))) {
	    if (true) {
		if (key.equals("")) {
		    key=parts[i];
		} else {
		    key+="+"+parts[i];
		}
	    }
	}
	return key;
    }


		
    public void showEntries(String name, ArrayList entries) {
	if (entries.size()==0)
	    return;
	System.out.println();
	int maxWordsLen=-1;
	int maxReducedTagsLen=-1;
	int maxFullTagsLen=-1;

	for (int i=0; i<entries.size(); i++) {
	    ContextInfo entry=(ContextInfo)entries.get(i);
	    String wordsStr="";
	    String reducedTagsStr="";
	    String fullTagsStr="";
	    for (int j=0; j<entry.words.length; j++) {
		wordsStr+=entry.words[j]+" ";
		reducedTagsStr+=entry.reducedTags[j]+" ";
		if (bothTags)
		    fullTagsStr+=entry.fullTags[j]+" ";
	    }
	    if (wordsStr.length()>maxWordsLen) 
		maxWordsLen=wordsStr.length();
	    if (reducedTagsStr.length()>maxReducedTagsLen)
		maxReducedTagsLen=reducedTagsStr.length();
	    if (fullTagsStr.length()>maxFullTagsLen)
		maxFullTagsLen=fullTagsStr.length();
	}
	maxWordsLen=(maxWordsLen>20 ? 20 : maxWordsLen);
	maxReducedTagsLen=(maxReducedTagsLen>20 ? 20 : maxReducedTagsLen);
	maxFullTagsLen=(maxFullTagsLen>60 ? 60 : maxFullTagsLen);

	System.out.println(name);
	for (int i=0; i<entries.size(); i++) {
	    ContextInfo entry=(ContextInfo)entries.get(i);
	    String wordsStr="";
	    String reducedTagsStr="";
	    String fullTagsStr="";
	    for (int j=0; j<entry.words.length; j++) {
		wordsStr+=entry.words[j]+" ";
		reducedTagsStr+=entry.reducedTags[j]+" ";
		if (bothTags)
		    fullTagsStr+=entry.fullTags[j]+" ";
	    }
	    System.out.println(danbikel.parser.util.SKUtil.leftJustify(4,i+") ")+ 
			       danbikel.parser.util.SKUtil.leftJustify(maxWordsLen,wordsStr)+
			       danbikel.parser.util.SKUtil.leftJustify(maxReducedTagsLen,reducedTagsStr)+
			       (bothTags? danbikel.parser.util.SKUtil.leftJustify(maxFullTagsLen,fullTagsStr) : "") + 
			       danbikel.parser.util.SKUtil.leftJustify(20,entry.filename+"/"+entry.sentNum));
	}
    }



    public void process(String filename)
	throws FileNotFoundException,IOException {
	System.err.println("reading file"+filename);
	int index=filename.lastIndexOf("/");
	currentFilename=filename.substring(index+1);
	currentSentNum=0;
	
	try {
	    //	    Training3 training = (Training3)Language.training();
	    SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
						  Constants.defaultFileBufsize);
	    Sexp curr = null;
	    while ((curr = Sexp.read(tok)) != null) {
		//		System.out.println("curr="+curr);
		process(curr);
		currentSentNum++;
	    
	    }
	    
	    
	}
	catch (UnsupportedEncodingException uee) {
	    System.err.println(uee);
	}
    }
    
    abstract protected void process(Sexp tree);
    abstract protected void showResults();
	
    public void process(List files) 
	throws FileNotFoundException,IOException {

	for (Iterator fileIter = files.iterator(); fileIter.hasNext(); ) {
	    String baseFileName=(String)fileIter.next();
	    //	    String inFileName=baseFileName+defaultExtension;
	    process(baseFileName);
	}
	showResults();
    }


    public static void main(String[] args) {
	boolean bothTags= (args[0].equals("true")) ? true : false;
	String listOfWords=args[1];
	ArrayList dir=new ArrayList();
	dir.add(new File(args[2]));
	
	//	System.out.println("added"+args[1]+"to dir");
	List files=danbikel.parser.util.SKUtil.expandDirectories(dir,defaultExtension,false,false);
	try {
	    ExamineContext examineContext=new ExamineContext(bothTags,listOfWords);
	    examineContext.process(files);
	}
	catch (FileNotFoundException fnfe) {
	System.err.println(fnfe);
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    
	}
    }
}
