
package danbikel.parser.historical;

import java.util.*;
import java.io.*;
import danbikel.lisp.*;
import danbikel.parser.*;


public class PrepareData extends danbikel.parser.lang.AbstractTraining2 {

    private final static String className = Training.class.getName();
    static final Symbol nullElementPreterminal = Symbol.add("-NONE-");

    /*
      doing this offline rather than as part of training/testing because
      this makes it easier to check for errors, do a training/test split,
      etc.
    */
    
   public PrepareData() throws FileNotFoundException, IOException {
	String language = Settings.get(Settings.language);
	System.out.println("language="+language);
	String metadataResource = Settings.get(metadataPropertyPrefix + language);
	System.out.println("metadataResource="+metadataResource);
	Class cl=Language.training().getClass();
	System.out.println("cl="+cl);
	InputStream is = Settings.getFileOrResourceAsStream(cl,
							    metadataResource);
	System.out.println("got is");
	int bufSize = Constants.defaultFileBufsize;
	SexpTokenizer metadataTok =
	    new SexpTokenizer(is, Language.encoding(), bufSize);
	readMetadata(metadataTok);
    }
	

      
    protected String checkTree(Sexp tree) {
	
	if (!tree.isList()) {
	    return "case1 not a list";
	}       

	if (tree.list().length()!=2 ||
	    !tree.list().get(0).isList() ||
	    !tree.list().get(1).isList()) {
	    return "case2 not two lists";
	}

	tree=removeID(tree);
	if (tree.list().get(0).isSymbol() &&
	    isMetaSymbol(tree.list().get(0).symbol())) {
	    return "case3 tree is a "+tree.list().get(0).symbol();
	}

	tree=regularizeNullElements(tree);
	tree=removeCODE(tree);

	String report=isValidTreeInternalStringReport(tree);
	if (report!=null) {
	    return report;
	}
	if (isAllNodesToPrune(tree)) {
	    System.out.println("allnodespruned true");
	    return "all nodes pruned";
	}
	return null;
    }
    
    protected Sexp removeID(Sexp tree) {
	
	Sexp sentence=tree.list().get(0);
	Sexp id=tree.list().get(1);
	tree.list().remove(1);
	tree.list().remove(0);
	/*
	  System.out.println("new tree="+tree);
	  System.out.println("id="+id);
	  System.out.println("sentence="+sentence);
	*/
	SexpList sentenceList=sentence.list();
	for (int i=0; i<sentenceList.length(); i++) {
	    tree.list().add(sentenceList.list().get(i));
	}
	//    System.out.println("new tree3="+tree);
	return tree;
    }
    
    protected Sexp removeCODE(Sexp tree) {
	if (Language.treebank().isPreterminal(tree))
	    return tree;
	if (tree.isList()) {
	    SexpList treeList = tree.list();
	    for (int i = 1; i < treeList.length(); i++) {
		if (treeList.get(i).isSymbol())  {
		    // if this is a bad child, skip it
		    System.out.println("removeCode - what is this?"+ tree);
		    continue;
		}
		Symbol childLabel = treeList.getChildLabel(i);
		if (isMetaSymbol(childLabel)) {
		    treeList.remove(i--);
		}
		else
		    removeCODE(treeList.get(i));
	    }
	}
	return tree;
    }

    protected Sexp regularizeNullElements(Sexp tree) {
	if (tree.isList()) {
	    SexpList treeList = tree.list();
	    int treeListLen = treeList.length();
	    Symbol nullElementTag =nullElementPreterminal;
	    //		danbikel.parser.ppceme.Treebank.nullElementPreterminal;
	    

	    if (treeList.isAllSymbols() && treeListLen == 3 &&
		treeList.get(1).toString().charAt(0) == '*') {
		System.out.println("regularize special case " + tree);
		treeList.remove(1);
	    }
	    else {
		for (int i = 1; i < treeListLen; i++) {
		    Sexp child = treeList.get(i);
		    if (treeList.get(0) != nullElementTag &&
			child.isSymbol() && child.symbol().toString().charAt(0) == '*') {
			SexpList newChild =
			    new SexpList(2).add(nullElementTag).add(child);
			treeList.set(i, newChild);
		    } else if (treeList.get(0) != nullElementTag &&
			       child.isSymbol() 
			       && 
			       child.symbol().toString().equals("0")) {
			SexpList newChild =
			    new SexpList(2).add(nullElementTag).add(child);
			treeList.set(i, newChild);
			String label=treeList.get(0).symbol().toString();
			if (!(label.equals("C")) &&
			    !(label.startsWith("W"))) {
			    System.out.println("warning : changed 0 to null for "+tree);
			}
		    }
		    else if (child.isList())
			regularizeNullElements(child);
		}
	    }
	}
	return tree;
    }
  

    
    String isValidTreeInternalStringReport(Sexp tree) {
	if (tree.isSymbol()) {
	    return tree+" is  Symbol";
	}
	if (treebank.isPreterminal(tree)) {
	    return null;
	}
	if (tree.isList()) {
	    SexpList treeList = tree.list();
	    if (treeList.first().isSymbol() == false) {
		return treeList.first()+" is not symbol";
	    }
	    int treeListLen = treeList.length();
	    if (treeListLen == 1) {
		return treeList + " has length 1";
	    }
	    for (int i = 1; i < treeListLen; i++) {
		String b=isValidTreeInternalStringReport(treeList.get(i));
		if (b!=null) {
		    return b;
		}
	    }
	}
	return null;
    }


    /* for each sexp, 
       1. strip outerparen
       2. call training.skip - returns:
          "tree is a " if the tree is rooted by a CODE or META
	  "tree is not a list" if the tree is not a list
	  "tree is not a list of two lists" 
	  "tree is rooted by"
           calls removeID(tree) 
           calls regularizeNullElements(tree) - make null elements into -NONE-
	   calls removeCODE - takes out CODE and META elements
	   
          "all nodes pruned"
	  "is symbol", 
          "is not symbol"
	  "has length 1"
    */
          
    static BufferedWriter[] skipFiles=new BufferedWriter[7];
    static BufferedWriter train=null;
    static BufferedWriter dev=null;
    static BufferedWriter test=null;

    static int devIndex=-1;
    static int testIndex=-1;

    public static void init(String outputDir)
	throws IOException {
	System.out.println("outputDir="+outputDir);

	for (int i=0; i<6; i++) {
	    skipFiles[i]=new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
			FileOutputStream(new File(outputDir+"/skipped"+i+".txt"))));
	}
	train = new 
	    BufferedWriter(new 
		OutputStreamWriter(new 
		    FileOutputStream(new File(outputDir+"/train.txt"))));
	dev = new 
	    BufferedWriter(new 
		OutputStreamWriter(new 
		    FileOutputStream(new File(outputDir+"/dev.txt"))));
	test = new 
	    BufferedWriter(new 
		OutputStreamWriter(new 
		    FileOutputStream(new File(outputDir+"/test.txt"))));
    }

    public static void close() 
	throws IOException {
	for (int i=0; i<6; i++) {
	    skipFiles[i].close();
	}
	train.close();
	dev.close();
	test.close();
    }

    public static int checkSkip(String skipStr) {
	if (skipStr==null) {
	    return 0;
	}
	if (skipStr.startsWith("case1")) {
	    return 1;
	} else if (skipStr.startsWith("case2")) {
	    return 2;
	} else if (skipStr.startsWith("case3")) {
	    return 3;
	} else if (skipStr.startsWith("all nodes pruned")) {
	    return 4;
	}  else {
	    return 5;
	}	
    }

    public  void process(String filename, String outputDir) 
	throws IOException {
 

	printMetadata();

	init(outputDir);
	
	int sentNum=0;
	int realSentNum=0;

	SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
					      Constants.defaultFileBufsize);
	Sexp tree = null;
	for ( ; (tree = Sexp.read(tok)) != null; sentNum++) {
	    Sexp treeCopy=tree.deepCopy();

	    String skipStr = checkTree(tree);	    
	    
	    
	    //	    String skipStr = ((danbikel.parser.lang.SKAbstractTraining)training).preskip(tree);	    
	    
	    int skipNum=checkSkip(skipStr);
	    if (sentNum%1000==0) 
		System.out.println("sentNum="+sentNum);
	    if (skipNum>2) {
		skipFiles[skipNum].write(skipStr+"\n");
	    }
	    skipFiles[skipNum].write("tree # " + (sentNum+1) + " before skip = "+treeCopy+"\n");
	    skipFiles[skipNum].write("tree # " + (sentNum+1) + " after skip = "+tree+"\n");		
	    if (skipNum==0) {
		realSentNum++;
		skipFiles[0].write("realSentNum = " + realSentNum+"\n");
		if (realSentNum%10 ==devIndex) {
		    //		if (realSentNum%10 ==8) {
		    //		    dev.write("realSentNum = " + realSentNum+"\n");
		    dev.write(tree+"\n");
		} else if (realSentNum%10 ==testIndex) {
		    //		} else if (realSentNum%10 ==9) {
		    //		    test.write("realSentNum = " + realSentNum+"\n");
		    test.write(tree+"\n");		    
		} else {
		    //		    train.write("realSentNum = " + realSentNum+"\n");
		    train.write(tree+"\n");
		}
	    }
	    if (skipNum==6) {
		skipFiles[6].write("skip reason = " + skipStr+"\n");
	    }
	}
	close();
    }

    public static void main(String[] args) {
	String filename = args[0];
	String outputDir=args[1];
	devIndex=Integer.parseInt(args[2]);
	testIndex=Integer.parseInt(args[3]);
	System.out.println("filename="+filename);
	System.out.println("outputDir="+outputDir);


	try {
	    PrepareData prepareData=new PrepareData();
	    prepareData.process(filename,outputDir);
	}
	catch (IOException e) {
	    System.out.println("exception"+e);
	}
    }
}   
