package danbikel.parser.lang;

import danbikel.lisp.*;
import danbikel.parser.*;
import java.util.*;
import java.io.*;

public abstract class AbstractTraining2 extends AbstractTraining implements Training2 {

    private Symbol topSym=Symbol.add("TOP");
    private Nonterminal nonterminal9a = new Nonterminal();

    protected static final boolean skipPostProcessing =
	Settings.getBoolean("parser.decoder.skipPostProcessing");


    private final static String className = Training.class.getName();

    private SexpList addToRaiseCode = new SexpList();
    private SexpList raiseCode = new SexpList();

    protected static Symbol CODE = Symbol.get("CODE");
    protected final static Symbol META = Symbol.get("META");
    protected final static Symbol[] metaSymbols={CODE,META};


      private Nonterminal nonterminal3 = new Nonterminal();

    // data members
    //    private Nonterminal nonterminal = new Nonterminal();


    public Sexp stripDollarSigns(Sexp tree) {
	if (treebank.isPreterminal(tree)) {
	    Symbol word = tree.list().get(1).symbol();
	    tree.list().set(1,stripDollarSigns(word));
	    return tree;
	}
	for (int i=1; i<tree.list().length(); i++) {
	    stripDollarSigns(tree.list().get(i));
	}
	return tree;
    }
    
    public Symbol stripDollarSigns(Symbol word) {
	String wordStr=word.toString();
	boolean modified=false;
	if (wordStr.startsWith("$")) {
	    wordStr=wordStr.substring(1);
	    modified=true;
	}
	if (wordStr.endsWith("$")) {
	    wordStr=wordStr.substring(0,wordStr.length()-1);
	    modified=true;
	}
	return (modified? Symbol.add(wordStr) : word);
    }

    protected void stripArgAugAndCanonicalizeBases(Sexp tree) {
	if (treebank.isPreterminal(tree)) {
	    /*
	      SexpList treeList = tree.list();
	      Symbol currLabel = treeList.symbolAt(0);
	      treeList.set(0, treebank.getCanonical(currLabel));
	    */
	    return;
	}
	else if (tree.isList()) {
	    SexpList treeList = tree.list();
	    Symbol currLabel = treeList.symbolAt(0);
	    treeList.set(0, stripArgAugAndCanonicalizeBase(currLabel));
	    int treeListLen = treeList.length();
	    for (int i = 1; i < treeListLen; i++)
		stripArgAugAndCanonicalizeBases(treeList.get(i));
	}
    }
    public Symbol stripArgAugAndCanonicalizeBase(Symbol label) {
	//    Symbol canonicalArgLabel = (Symbol)canonicalArgCache.get(label);
	//    if (canonicalArgLabel == null) {
	Symbol newLabel=removeDefaultArgAugmentation(label,new Nonterminal());
	Nonterminal parsedLabel =
	    treebank.parseNonterminal(newLabel);
	parsedLabel.base = treebank.getCanonical(parsedLabel.base);
	return parsedLabel.toSymbol();
    }
    //      if (argAugmentations.contains(augmentations.get(i))) {
    protected Symbol removeDefaultArgAugmentation(Symbol label,
						  Nonterminal nonterminal) {
	Nonterminal parsedLabel = treebank.parseNonterminal(label, nonterminal);
	boolean removedSomething = false;
	SexpList augmentations = parsedLabel.augmentations;
	for (int i = 0; i < augmentations.length(); i++) {
	    if (defaultArgAugmentation==augmentations.get(i)) {
		i--; // move index to delimeter that preceded this arg augmentation
		augmentations.remove(i);  // remove delimeter
		augmentations.remove(i);  // remove arg augmentation
		removedSomething = true;
		i--; // decrement index again to offset increment in for loop
	    }
	}
	return removedSomething ? parsedLabel.toSymbol() : label;
    }

    protected void raiseCodeNodes(Sexp tree) {
	raiseCode.clear();
	raiseCodeNodes(tree, raiseCode, Constants.LEFT, true);
	/*
	  for (int i = 0; i < raiseCode.length(); i++) {
	  System.out.println("WARNING : raised out " + raiseCode.get(i));
	  }
	*/
	if (raiseCode.size()>0) {
	    SexpList treeList = tree.list();
	    for (int raiseIdx = 0; raiseIdx < raiseCode.length(); raiseIdx++) {
		int i=treeList.length();
		System.out.println("raised out CODE left " + 
				   raiseIdx + ")" + 
				   raiseCode.get(raiseIdx) + " putting back in");
		treeList.add(i, raiseCode.get(raiseIdx));
	    }
	}
	raiseCode.clear();
	raiseCodeNodes(tree, raiseCode, Constants.RIGHT, true);
	/*
	  for (int i = 0; i < raiseCode.length(); i++) {
	  System.out.println("WARNING : raised out " + raiseCode.get(i));
	  }
	*/
	if (raiseCode.size()>0) {
	    SexpList treeList = tree.list();
	    for (int raiseIdx = 0; raiseIdx < raiseCode.length(); raiseIdx++) {
		System.out.println("raised out CODE right " + 
				   raiseIdx + ")" + 
				   raiseCode.get(raiseIdx) + " putting back in");
		treeList.add(1, raiseCode.get(raiseIdx));
	    }
	}
    }

    private void raiseCodeNodes(Sexp tree, SexpList raise, boolean direction, boolean atTop) {
	if (tree.isSymbol() || treebank.isPreterminal(tree))
	    return;
	if (allChildrenCodeToRaise(tree)) {
	    System.err.println(Training.class.getName() +
			       ": warning: all children are CODE to raise\n" +
			       "\t" + tree);
	    return;
	}

	// if tree is a list with at least two children (i.e., of length 3)
	if (tree.isList()) {
	    SexpList treeList = tree.list();
	    
	    boolean leftToRight = direction == Constants.LEFT;
	    int startIdx = (leftToRight ? 1 : treeList.length() - 1);
	    int endIdx = (leftToRight ? treeList.length() - 1 : 1);
	    int increment = (leftToRight ? 1 : -1);
	    for (int i = startIdx;
		 (leftToRight && i <= endIdx) || (!leftToRight && i >= endIdx);
		 i += increment) {
		// we're occasionally removing items from tree, so recalculate these
		startIdx = (leftToRight ? 1 : treeList.length() - 1);
		endIdx = (leftToRight ? treeList.length() - 1 : 1);
		Sexp currChild = treeList.get(i);
		
		raiseCodeNodes(currChild,  raise, direction, false);
		
		// if it's the last child that we're visiting
		if (i == endIdx) {
		    // while there's ending punctuation, remove it and add it
		    // to the raise queue (in queue order, hence the call to
		    // reverse, since we're grabbing nodes from the outside in)
		    addToRaiseCode.clear();
		    while (treeList.length() > 1 &&
			   isCodeToRaise(treeList.get(endIdx))) {
			/*
			  if (!atTop) {
			  addToRaiseCode.add(treeList.remove(endIdx));
			  endIdx = (leftToRight ? treeList.length() - 1 : 1);
			  } else {
			  System.out.println("WARNING : not raising out CODE "+treeList.get(endIdx));
			  }
			*/
			addToRaiseCode.add(treeList.remove(endIdx));
			endIdx = (leftToRight ? treeList.length() - 1 : 1);
		    }
		    // set i to the new last element index (not strictly necessary)
		    if (raise.addAll(addToRaiseCode.reverse()))
			i = endIdx;
		}
		// if it's not the last child we're visiting at this level and
		// there are raise requests enqueued, we can oblige by
		// inserting the punctuation just after the current node
		else if (raise.length() != 0) {
		    // since items naturally shift rightward when we use the add method,
		    // to add "after" current node in a right-to-left traversal requires
		    // keeping add index the same (i.e., an increment of zero)
		    int insertIncrement = (leftToRight ? increment : 0);
		    i += insertIncrement;
		    for (int raiseIdx = 0; raiseIdx < raise.length(); raiseIdx++) {
			treeList.add(i, raise.get(raiseIdx));
			if (leftToRight)
			    i += insertIncrement;
		    }
		    endIdx = (leftToRight ? treeList.length() - 1 : 1);
		    i -= insertIncrement; // offset increment for enclosing for loop
		    raise.clear();
		}
	    }
	}
    }
    
    private boolean allChildrenCodeToRaise(Sexp tree) {
	int treeLen = tree.list().length();
	for (int i = 1; i < treeLen; i++)
	    if (isCodeToRaise(tree.list().get(i)) == false)
		return false;
	return true;
    }
    
    protected boolean isMetaSymbol(Symbol label) {
	for (int i = 0; i < metaSymbols.length; i++) {
	    if (label == metaSymbols[i]) {
		return true;
	    }
	}
	return false;
    }
    
    public boolean isCodeToRaise(Sexp preterm) {
	return (treebank.isPreterminal(preterm) &&
		isMetaSymbol(preterm.list().first().symbol()));
    }

  public Sexp stripIndices(Sexp tree) {
    if (treebank.isPreterminal(tree))
      return tree;
    if (tree.isList()) {
      Symbol label = tree.list().first().symbol();
      tree.list().set(0, treebank.stripIndex(label, nonterminal3));
      int treeListLen = tree.list().length();
      for (int i = 1; i < treeListLen; i++) 
	stripIndices(tree.list().get(i));
    }
    return tree;
  }

    public Sexp parentAnnotation(Sexp tree) {
	parentAnnotation(tree,topSym);
	return tree;
    }

    public void parentAnnotation(Sexp tree, Symbol grandParentLabel) {
	if (treebank.isPreterminal(tree))
	    return;
	SexpList treeList = tree.list();
	int treeListLen = treeList.length();	
	Sexp parent=treeList.get(0);
	Symbol strippedParentLabel=treebank.stripAugmentation(parent.symbol());
	for (int j=1; j<treeListLen; j++) {
	    parentAnnotation(treeList.get(j),strippedParentLabel);
	}
	Nonterminal parsedParent =
	    treebank.parseNonterminal(parent.symbol(), nonterminal9a);
	parsedParent.base=Symbol.add(parsedParent.base.toString()+"^"+grandParentLabel);
	treeList.set(0,parsedParent.toSymbol());
    }

    public void undoParentAnnotation(Sexp tree) {
	if (treebank.isPreterminal(tree))
	    return;
	SexpList treeList = tree.list();
	int treeListLen = treeList.length();	
	Sexp parent=treeList.get(0);
	Nonterminal parsedParent =
	    treebank.parseNonterminal(parent.symbol(), nonterminal9a);
	String parentBaseStr=parsedParent.base.toString();
	int index=parentBaseStr.indexOf("^");
	if (index==-1) 
	    System.out.println("warning: no ^ in "+tree);
	parsedParent.base=Symbol.add(parentBaseStr.substring(0,index));
	treeList.set(0,parsedParent.toSymbol());
	for (int j=1; j<treeListLen; j++) {
	    undoParentAnnotation(treeList.get(j));
	}

    }


}
