package danbikel.parser.parsedebug;

import java.io.*;
import java.util.*;
import java.util.regex.*;
import java.text.DecimalFormat;

import danbikel.parser.util.*;
import danbikel.parser.*;
import danbikel.lisp.*;
import danbikel.util.*;

class CrossingSummary {
    int sentNum;
    double recall;
    double precision;
    int numCrossing;

    public CrossingSummary(int sentNum, double recall,
			   double precision, int numCrossing) {
	this.sentNum=sentNum;
	this.recall=recall;
	this.precision=precision;
	this.numCrossing=numCrossing;
    }
}

class LabelPR implements Comparable {
    String label;
    double numMatching;
    double numGold;
    double numTest;
    double perTotal;
    static DecimalFormat myFormatter = new DecimalFormat("###.##");

    ArrayList bracketsMatching=new ArrayList();
    ArrayList bracketsMiss=new ArrayList();
    ArrayList bracketsFalseAlarm=new ArrayList();

    public int compareTo(Object o) {
	double otherNumGold=((LabelPR)o).numGold;
	String otherLabel=((LabelPR)o).label;
	if (numGold<otherNumGold) {
	    return 1;
	} else if  (numGold>otherNumGold) {
	    return -1;
	} else {
	    return label.compareTo(otherLabel);
	}
    }
}

class CrossingBracketInfo {
    ParseTreeNode goldNode;
    ParseTreeNode testNode;
    
    public CrossingBracketInfo(ParseTreeNode goldNode,
			       ParseTreeNode testNode) {
	this.goldNode=goldNode;
	this.testNode=testNode;
    }
	
}

class Terminal {
    ParseTreeNode node;
    Symbol text;
    Symbol label;
    int result;
    int index;
  
    public Terminal(ParseTreeNode node,Symbol text, Symbol label, int index) {
	this.node=node;
	this.text=text;
	this.label=label;
	this.index=index;
	result=ParseDebug.UNDEFINED;
    }
  
    public String toString() {
	return ("terminal "+
		"["+index+"]"+
		"("+label+","+text+")");
    }
}
class Bracket2 {
    int sentNum;
    int goldOrTest;
    Bracket bracket;
    
    public Bracket2(Bracket bracket, int sentNum, int goldOrTest) {
	this.sentNum=sentNum;
	this.goldOrTest=goldOrTest;
	this.bracket=bracket;
    }

    public String toString() {
	return (sentNum+":"+
		(goldOrTest==ParseDebug.GOLD_TREE ? "G" : "T") + 
		"["+bracket.leftIdx+","+bracket.rightIdx+"]"+
		"("+bracket.label+")");
    }

}


class Bracket {
    int leftIdx;
    int rightIdx;
    Symbol label;
    /* no indices
       no default Arg marker (-A)
       in case using test output nonstripped
    */


    Symbol labelWithFuncTags;

    Symbol strippedLabel;
    Symbol normalizedLabel;
    int result; 
    ParseTreeNode node;

    Nonterminal nonterminal=null;

    public Bracket(ParseTreeNode node,Symbol label, 
		   int leftIdx, int rightIdx) {
	this.node=node;
	this.label=label;
	this.leftIdx=leftIdx;
	this.rightIdx=rightIdx;
	result=ParseDebug.UNDEFINED;
    }
    public String toString() {
	return ("bracket "+
		"["+leftIdx+","+rightIdx+"]"+
		"("+label+")");
    }
}


class Equiv {
    Symbol s1;
    Symbol s2;
    
    public Equiv(Symbol s1,Symbol s2) {
	this.s1=s1;
	this.s2=s2;
    }
}


class SentenceInfo {
    int numBracketsAfterDeletion=0;
    protected CountsTable numNormalizedLabels=null;

    /* Each element is of type Terminal.
       terminalsAll includes all terminals,
       terminals does not include deleted terminals
    */
    ArrayList terminalsAll=null;
    ArrayList terminals=null;
    /* Each element if of type Bracket
     */

    ArrayList bracketsAll=null;
    ArrayList brackets=null;

    Sexp originalTree=null;
    ParseTreeNode parseTreeNode=null;

    public SentenceInfo(Sexp originalTree,
			ParseTreeNode parseTreeNode) {
	this.originalTree=originalTree;
	this.parseTreeNode=parseTreeNode;
	numBracketsAfterDeletion=0;
	bracketsAll=new ArrayList();
	terminalsAll=new ArrayList();
	brackets=new ArrayList();
	terminals=new ArrayList();
	numNormalizedLabels=new CountsTableImpl();
	
    }
}

class EvalInfo {
    int numCrossing=0;
    int numMatch=0;
    int numCorrectTag=0;
    int numTerminalsExceptForSome=0;
    double recall=0.0;
    double precision=0.0;
    double tagAccuracy=0.0;
    ArrayList crossingBrackets=new ArrayList();
    protected CountsTable numMatchingNormalizedLabels=new CountsTableImpl();
    java.util.HashMap bracketsMatching=new java.util.HashMap();
    java.util.HashMap bracketsMiss=new java.util.HashMap();
    java.util.HashMap bracketsFalseAlarm=new java.util.HashMap();

}

class SentenceBrackets {
    int sentNum;
    ArrayList brackets=new ArrayList();

    public SentenceBrackets(int sentNum, ArrayList brackets) {
	this.sentNum=sentNum;
	this.brackets=brackets;
    }
    public String toString() {
	String ret=sentNum+"("+brackets.size()+")";
	for (int i=0; i<brackets.size(); i++) {
	    Bracket bracket=(Bracket)brackets.get(i);
	    ret+=bracket.toString()+" ";
	}
	return ret;
    }
}

class TotalInfo {
    String name;
    int numGoldBrackets;
    int numTestBrackets;
    int numMatch;
    int numSent;
    int numErrorSent;
    int numSkipSent;
    int numNullSent; // only relevent for when running in nonulls mode
    int numCompleteSent;
    int numWords;
    int numCrossing;
    int numNoCrossingSent; // no crossing sentences
    int num2LCrossingSent; // 2 or less crossing sentences 
    int numCorrectTag;

    protected CountsTable numGoldNormalizedLabels=null;
    protected CountsTable numTestNormalizedLabels=null;
    protected CountsTable numMatchingNormalizedLabels=null;
    // each entry is a LabelPR, sorted by numGold

    java.util.HashMap normalizedLabelMap=new java.util.HashMap();


    java.util.HashMap bracketsMatching=new java.util.HashMap();
    java.util.HashMap bracketsMiss=new java.util.HashMap();
    java.util.HashMap bracketsFalseAlarm=new java.util.HashMap();    

    public TotalInfo(String name) {
	this.name=name;
	numGoldNormalizedLabels=new CountsTableImpl();
	numTestNormalizedLabels=new CountsTableImpl();
	numMatchingNormalizedLabels=new CountsTableImpl();
	
    }

}
/** The gold and test trees are each stored in a ParseTreeNode structure.
    Each ParseTreeNode has the label and text, and also the indices
    based on every terminal (<code>leftIdx</code>,<code>rightIdx</code>),
    and also the indices based on every only the terminals not included
    in the list of terminals to be deleted (DELETE_LABEL in COLLINS.prm - 
    TOP,-NONE-, comma, colon, double single quotes, period).  If a terminal
    is actually a DELETE_LABEL, then the field <code>deleted</code> is
    true in the ParseTreeNode instance for that terminal.
    
    The info from the ParseTreeNode is loaded into a SentenceInfo for
    the gold and test trees. When doing so, terminalsAll and bracketsAll
    is filled up with the info that disregards the deleted labels
    while terminals and brackets  is filled up with the info that 
    takes into account of the deleted labels. The difference is that the 
    terminals nad brackets use <code>left(right)IdxDeleted</code> instead
    of <code>left(right)Idx</code> and don't include any terminals with
    deleted==true.  After that point only terminals and brackets is used.
    
    The slight convenience here is for printing out the debug info,
    since evalb only prints out the terminals that are not deleted.

    Note: The definition of DELETE_LABEL in evalb.c is used in two ways.
    First, when reading in the tree, any labels with a DeleteLabel are
    ignored and not even stored.  This is unlike what we do here, where
    such labels are included, but ignored for scoring.  But this is why
    we need to keep track of the indices, just to make sure that the output
    is the same.  Second, during massage_data, any brackets with that label
    are deleted.  But those are stored, just with a status set to deleted
    
*/


    

    public class ParseDebug {
	Symbol defaultArgAugmentation=Language.training().defaultArgAugmentation();
	protected BitSet augmentationDelimSet;
	protected Symbol allLabels=Symbol.add("all");

	static final int UNMATCH=0;
	static final int MATCH=1;
	static final int DELETE=5;
	static final int UNDEFINED=9;
	private static Treebank treebank = Language.treebank();
	//    private static Training training = Language.training();
	static DecimalFormat percentage=new DecimalFormat("##0.00");

    

    
	String goldFilename=null;
	String testFilename=null;
	String resultsFilename=null;
	String matchInfoFilename=null;
	String crossingFilename=null;
	String tagsFilename=null;
	boolean debug=false;
	boolean nonulls=false;
	boolean funcTagsCount=false;
	boolean putTagsBack=false;
	int sentNum=0;
	int errorCount=0;

	int status=0;
	int skip=1;
	TotalInfo totalInfo=new TotalInfo("-- All --");
	TotalInfo total40Info=new TotalInfo("-- len<=40 --");
	ArrayList crossingCases=new ArrayList();
	BufferedWriter outResults=null;
	BufferedWriter outCrossing=null;
	BufferedWriter outMatchInfo=null;
	BufferedWriter outTags=null;
    
	ArrayList deleteLabels=null;
	ArrayList deleteLabelsForLength=null;
	Equiv[] equivs=null;
	public static final int GOLD_TREE=0;
	public static final int TEST_TREE=1;

	public ParseDebug() {
	    augmentationDelimSet = new BitSet(Character.MAX_VALUE);
	    String augmentationDelimStr = "-=";
	    for (int i = 0; i < augmentationDelimStr.length(); i++)
		augmentationDelimSet.set(augmentationDelimStr.charAt(i));
	}

    
	public void process(String goldFilename,String testFilename,
			    String resultsFilename,
			    String matchInfoFilename,
			    String crossingFilename, 
			    String tagsFilename,
			    boolean debug,
			    boolean funcTagsCount,
			    boolean nonulls,
			    boolean putTagsBack) {
	    this.goldFilename=goldFilename;
	    this.testFilename=testFilename;
	    this.matchInfoFilename=matchInfoFilename;
	    this.tagsFilename=tagsFilename;
	    this.debug=debug;
	    this.funcTagsCount=funcTagsCount;
	    this.nonulls=nonulls;
	    this.putTagsBack=putTagsBack;
	    String encoding = "ISO-8859-1";
	    int defaultFileBufsize=81920;
	
	    initStuff();
	
	    try {
		openFiles(resultsFilename,crossingFilename);
		System.out.println("goldFilename="+goldFilename);
		System.out.println("testFilename="+testFilename);
		printHead();

		//	    SexpTokenizer goldTok=

		process(new SexpTokenizer(goldFilename, encoding, defaultFileBufsize),
			new SexpTokenizer(testFilename, encoding, defaultFileBufsize));
		closeFiles();
	    }
	    catch (UnsupportedEncodingException e) {
		System.err.println(e);
	    }
	    catch (FileNotFoundException e) {
		System.err.println(e);
	    }
	    catch (IOException e) {
		System.err.println(e);
	    }
	}
	void openFiles(String resultsFilename, String crossingFilename) 
	    throws IOException {
	    outResults = new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
			FileOutputStream(resultsFilename)));
	    outCrossing = new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
			FileOutputStream(crossingFilename)));
	    outMatchInfo = new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
			FileOutputStream(matchInfoFilename)));
	    if (putTagsBack) {
		outTags = new 
		    BufferedWriter(new 
			OutputStreamWriter(new 
			    FileOutputStream(tagsFilename)));
	    }
		
	    
	
	}
	void closeFiles() 
	    throws IOException {
	    outResults.close();
	    outCrossing.close();
	    outMatchInfo.close();
	    if (putTagsBack)
		outTags.close();
	}
	    
	
	public void process(SexpTokenizer goldTok,SexpTokenizer testTok) 
	    throws IOException {
	    Sexp goldTreeSexp=null;
	    Sexp testTreeSexp=null;

	    sentNum=1;
	    boolean stripOuterParens;


	    for (; (goldTreeSexp=Sexp.read(goldTok))!=null ; sentNum++) {

		if ((testTreeSexp=Sexp.read(testTok))==null) {
		    System.err.println("missing test parse for gold parse");
		}
		//		System.out.println("goldTreeSexp="+goldTreeSexp);
		if (treeHasNoRoot(goldTreeSexp)) {
		    //		    System.out.println("gold has no root");
		} else {
		    goldTreeSexp=adjustForParens(goldTreeSexp);
		}		    
		//		System.out.println("testTreeSexp="+testTreeSexp);
		if (treeHasNoRoot(testTreeSexp)) {
		    // System.out.println("test has no root");
		} else {
		    testTreeSexp=adjustForParens(testTreeSexp);
		    //		System.out.println("test after"+Util.prettyPrint(testTreeSexp));
		}
		evalSentence(goldTreeSexp,testTreeSexp,sentNum);

	    
		//	    Language.training().preProcess(goldTreeSexp);
		//	    HeadTreeNode headTree = new HeadTreeNode(goldTreeSexp);
		//System.out.println("headTree="+headTree);
      
	    }

	    displayTotalLine(totalInfo);
	    outResults.write("=== Summary ===\n");
	    displayTotalSummary(totalInfo);
	    displayTotalSummary(total40Info);
	    writeCrossingSummary();
	    processLabelCounters(totalInfo);
	    double totalGoldLabels=totalInfo.numGoldNormalizedLabels.count(allLabels);
	    System.out.println("totalGoldLabels="+totalGoldLabels);
	    TreeMap sortedMap=
		sortLabelMap(totalGoldLabels,totalInfo.normalizedLabelMap);

	    outResults.write("STRIPPED AND NORMALIZED LABELS \n");
	    writeLabelPRMap(sortedMap);
	    //	outResults.write("STRIPPED AND NORMALIZED LABELS \n");
	    //	writeLabelPRMap(totalInfo.normalizedLabelMap);

	}
    
    
	void processLabelCounters(TotalInfo totalInfo) 
	    throws IOException {

	    processCounter("numGoldNormalizedLabels",totalInfo.numGoldNormalizedLabels,totalInfo.normalizedLabelMap);
	    processCounter("numTestNormalizedLabels",totalInfo.numTestNormalizedLabels,totalInfo.normalizedLabelMap);
	    processCounter("numMatchingNormalizedLabels",totalInfo.numMatchingNormalizedLabels,totalInfo.normalizedLabelMap);
	    processInstances("match",totalInfo.bracketsMatching,totalInfo.normalizedLabelMap);
	    processInstances("miss",totalInfo.bracketsMiss,totalInfo.normalizedLabelMap);
	    processInstances("falsealarm",totalInfo.bracketsFalseAlarm,totalInfo.normalizedLabelMap);

	}

	TreeMap sortLabelMap(double totalGoldLabels, java.util.HashMap combinedMap) 
	    throws IOException {
	    System.out.println("in sortLabelPRMap");
	    TreeMap sortedMap=new TreeMap();
	    Iterator keys=combinedMap.keySet().iterator();
	    while (keys.hasNext()) {
		Symbol key=(Symbol)keys.next();
		System.out.println("key="+key);
		LabelPR labelPR=(LabelPR)combinedMap.get(key);
		int numGold=(int)labelPR.numGold;
		labelPR.perTotal=100.0*numGold / totalGoldLabels;
		sortedMap.put(labelPR,labelPR);
	    }
	    return sortedMap;
	}
	
	public void processInstances(String name, java.util.HashMap totalInfoMap,
				     java.util.HashMap combinedMap)
	    throws IOException {
	    Iterator keys=totalInfoMap.keySet().iterator();	
	    while (keys.hasNext()) {
		Symbol tag=(Symbol)keys.next();
		ArrayList totalInstances=(ArrayList)totalInfoMap.get(tag);
		LabelPR labelPR=(LabelPR)combinedMap.get(tag);
		if (labelPR==null) {
		    labelPR=new LabelPR();
		    labelPR.label=tag.toString();
		    //		System.out.println("addingnew to vocabInfo "+tag+" "  + vocabInfo);
		    combinedMap.put(tag,labelPR);
		}		
		if (name.equals("match")) {
		    labelPR.bracketsMatching=totalInstances;
		} else if (name.equals("miss")) {
		    labelPR.bracketsMiss=totalInstances;
		} else {
		    labelPR.bracketsFalseAlarm=totalInstances;
		}
	    }
	}

	public void processCounter(String name, CountsTable counter,java.util.HashMap combinedMap)
	    throws IOException {
	    Iterator keys = counter.keySet().iterator();
	    while (keys.hasNext()) {
		Object o = keys.next();
		Symbol tag=(Symbol)o;
		double val=counter.count(o);
		resultsWriteLine(name+" ("+tag+" " + val+")");
		LabelPR labelPR=(LabelPR)combinedMap.get(tag);
		if (labelPR==null) {
		    labelPR=new LabelPR();
		    labelPR.label=tag.toString();
		
		    //		System.out.println("addingnew to vocabInfo "+tag+" "  + vocabInfo);
		    combinedMap.put(tag,labelPR);
		}
		if (name.equals("numGoldNormalizedLabels")) {
		    labelPR.numGold=val;
		} else if (name.equals("numTestNormalizedLabels")) {
		    labelPR.numTest=val;
		} else {
		    labelPR.numMatching=val;
		}
	    }
	}

	void writeLabelPRMap(java.util.HashMap combinedMap)
	    throws IOException {

	    //	    System.out.println("in writeLabelPRMap");
	    int totalGold=0;
	    int totalTest=0;
	    int totalMatch=0;
	    double recall=0.0;
	    double precision=0.0;

	    System.out.println("number of labels =  " + combinedMap.size());

	    resultsWriteLine(rightJustify(8,"")+
			     rightJustify(8,"GOLD")+
			     rightJustify(8,"TEST")+
			     rightJustify(8,"MATCH")+
			     rightJustify(8,"RECALL")+
			     rightJustify(8,"PREC"));
	    Iterator keys=combinedMap.keySet().iterator();
	    while (keys.hasNext()) {
		Symbol key=(Symbol)keys.next();
		System.out.println("key="+key);
		LabelPR labelPR=(LabelPR)combinedMap.get(key);
		int numGold=(int)labelPR.numGold;
		int numTest=(int)labelPR.numTest;
		int numMatching=(int)labelPR.numMatching;
		if (!(labelPR.label.equals("all"))) {
		    totalGold+=numGold;
		    totalTest+=numTest;
		    totalMatch+=numMatching;
		}

		recall= 
		    numGold==0 ?0.0 :
		    100.00 * numMatching / numGold;
		precision = 
		    numTest==0 ? 0.0 :
		    100.00 * numMatching/numTest;
		resultsWriteLine(rightJustify(8,labelPR.label)+
				 rightJustify(8,numGold)+
				 rightJustify(8,percentage.format(labelPR.perTotal))+
				 rightJustify(8,numTest)+
				 rightJustify(8,numMatching)+
				 rightJustify(8,percentage.format(recall))+
				 rightJustify(8,percentage.format(precision)));
	    
	    }
	    resultsWriteLine("totalGold="+totalGold+
			     " totalTest="+totalTest+
			     " totalMatch="+totalMatch);
	}

	void writeLabelPRMap(TreeMap treeMap)
	    throws IOException {
	    //	    System.out.println("in revised writeLabelPRMap");
	    //	    System.out.println("in writeLabelPRMap TreeMap");
	    int totalGold=0;
	    int totalTest=0;
	    int totalMatch=0;
	    double recall=0.0;
	    double precision=0.0;

	    System.out.println("number of labels =  " + treeMap.size());
	    resultsWriteLine(rightJustify(8,"")+
			     rightJustify(8,"GOLD")+
			     rightJustify(8,"%GOLD")+
			     rightJustify(8,"TEST")+
			     rightJustify(8,"MATCH")+
			     rightJustify(8,"RECALL")+
			     rightJustify(8,"PREC"));
	    Iterator keys=treeMap.keySet().iterator();
	    while (keys.hasNext()) {
		LabelPR labelPR=(LabelPR)keys.next();
		int numGold=(int)labelPR.numGold;
		int numTest=(int)labelPR.numTest;
		int numMatching=(int)labelPR.numMatching;
		if (!(labelPR.label.equals("all"))) {
		    totalGold+=numGold;
		    totalTest+=numTest;
		    totalMatch+=numMatching;
		}

		recall= 
		    numGold==0 ?0.0 :
		    100.00 * numMatching / numGold;
		precision = 
		    numTest==0 ? 0.0 :
		    100.00 * numMatching/numTest;
		resultsWriteLine(rightJustify(8,labelPR.label)+
				 rightJustify(8,numGold)+
				 rightJustify(8,percentage.format(labelPR.perTotal))+
				 rightJustify(8,numTest)+
				 rightJustify(8,numMatching)+
				 rightJustify(8,percentage.format(recall))+
				 rightJustify(8,percentage.format(precision)));
		resultsWriteLine("matching:");
		for (int i=0; i<labelPR.bracketsMatching.size();i++) {
		    SentenceBrackets sentenceBrackets=(SentenceBrackets)labelPR.bracketsMatching.get(i);
		    resultsWriteLine(sentenceBrackets.toString());
		}

		resultsWriteLine("miss:");
		for (int i=0; i<labelPR.bracketsMiss.size();i++) {
		    SentenceBrackets sentenceBrackets=(SentenceBrackets)labelPR.bracketsMiss.get(i);
		    resultsWriteLine(sentenceBrackets.toString());
		}

		resultsWriteLine("falseAlarm:");
		for (int i=0; i<labelPR.bracketsFalseAlarm.size();i++) {
		    SentenceBrackets sentenceBrackets=(SentenceBrackets)labelPR.bracketsFalseAlarm.get(i);
		    resultsWriteLine(sentenceBrackets.toString());
		}
		    

	    
	    }
	    resultsWriteLine("totalGold="+totalGold+
			     " totalTest="+totalTest+
			     " totalMatch="+totalMatch);
	}

	void writeCrossingSummary() 
	    throws IOException {
	    outCrossing.write("==================================\n");
	    for (int i=0; i<crossingCases.size(); i++) {
		CrossingSummary crossingSummary=(CrossingSummary)crossingCases.get(i);
		outCrossing.write(crossingSummary.sentNum+"  "+
				  rightJustify(6,percentage.format(crossingSummary.recall))+"/" +
				  rightJustify(6,percentage.format(crossingSummary.precision))+ "  "+
				  rightJustify(2,crossingSummary.numCrossing+"\n"));
	    }
	    outCrossing.write("==================================\n");
	}


	/*
	  void calcTotalInfo(TotalInfo totInfo) {
	  if (totInfo.numGoldBrackets>0 &&
	  totInfo.numTestBrackets>0) {
	  double recall = 
	  totInfo.numGoldBrackets==0 ?
	  0.0 :
	  100.0*totInfo.numMatch / 
	  totInfo.numGoldBrackets;
      
	  double precision= 
	  totInfo.numTestBrackets==0 ?
	  0.0 :
	  100.0*totInfo.numMatch / 
	  totInfo.numTestBrackets;
	  }
      
	  double tagAccuracy=
	  totInfo.numWords==0?
	  0.0 :
	  100.0*totInfo.numCorrectTag / totInfo.numWords;
	  }
	*/
    

	void displayTotalLine(TotalInfo totInfo) 
	    throws IOException {
	    resultsWriteLine("============================================================================");
      
	    resultsWrite("                "+
			 makePer(100*totInfo.numMatch,
				 totInfo.numGoldBrackets) +" "+
			 makePer(100*totInfo.numMatch,
				 totInfo.numTestBrackets) +" "+
			 rightJustify(6,totInfo.numMatch)+" "+
			 rightJustify(5,totInfo.numGoldBrackets)+" "+
			 rightJustify(5,totInfo.numTestBrackets)+"  "+
			 rightJustify(5,totInfo.numNoCrossingSent));
	    resultsWriteLine("  "+
			     rightJustify(5,totInfo.numWords)+" "+
			     rightJustify(5,totInfo.numCorrectTag)+"   "+
			     makePer(100*totInfo.numCorrectTag,totInfo.numWords));

	}

	void resultsWriteLine(String str) 
	    throws IOException {
	    outResults.write(str+"\n");
	}
	void resultsWrite(String str) 
	    throws IOException {
	    outResults.write(str);
	}


	void displayTotalSummary(TotalInfo totInfo) 
	    throws IOException {
	    int sentn = totInfo.numSent-totInfo.numErrorSent-totInfo.numSkipSent;
	    resultsWriteLine("\n"+totInfo.name);
	    resultsWriteLine("Number of sentence        = "+
			     rightJustify(6,totInfo.numSent));
	    resultsWriteLine("Number of Error sentence  = "+
			     rightJustify(6,totInfo.numErrorSent));
	    resultsWriteLine("Number of Skip  sentence  = "+
			     rightJustify(6,totInfo.numSkipSent));
	    resultsWriteLine("Number of Valid sentence  = "+
			     rightJustify(6,sentn));
	    resultsWriteLine("Bracketing Recall         = "+
			     makePer(100*totInfo.numMatch,
				     totInfo.numGoldBrackets));
	    resultsWriteLine("Bracketing Precision      = "+
			     makePer(100*totInfo.numMatch,
				     totInfo.numTestBrackets));
	    resultsWriteLine("Complete match            = "+
			     makePer(100*totInfo.numCompleteSent,
				     sentn));
	    resultsWriteLine("Average crossing          = "+
			     makePer(totInfo.numCrossing,
				     sentn));
	    resultsWriteLine("No crossing               = "+
			     makePer(100*totInfo.numNoCrossingSent,
				     sentn));
	    resultsWriteLine("2 or less crossing        = "+
			     makePer(100*totInfo.num2LCrossingSent,
				     sentn));
	    resultsWriteLine("Tagging accuracy          = "+
			     makePer(100*totInfo.numCorrectTag,
				     totInfo.numWords));
	}


	void evalSentence(Sexp goldTreeSexp, Sexp testTreeSexp,int sentNum) 
	    throws IOException {

	    ParseTreeNode goldTree=new ParseTreeNode(goldTreeSexp);
	    ParseTreeNode testTree=new ParseTreeNode(testTreeSexp);

	    SentenceInfo goldSentenceInfo=new 
		SentenceInfo(goldTreeSexp,goldTree);
	    SentenceInfo testSentenceInfo=new 
		SentenceInfo(testTreeSexp,testTree);
	
	    getBrackets(goldTree,goldSentenceInfo);
	    getBrackets(testTree,testSentenceInfo);
	    EvalInfo evalInfo=new EvalInfo();
	    massageData(goldSentenceInfo);
	    massageData(testSentenceInfo);
	    evalInfo.numTerminalsExceptForSome=
		makeUpLength(goldSentenceInfo);
	    matchBracketsPR(goldSentenceInfo,
			    testSentenceInfo,
			    evalInfo,
			    totalInfo.numSent);
	    findBracketInstances(goldSentenceInfo,
				 testSentenceInfo,
				 evalInfo,
				 totalInfo.numSent);

	    findCrossingCases(goldSentenceInfo,
			      testSentenceInfo,
			      evalInfo,
			      totalInfo.numSent);

	    calcBracketsPR(goldSentenceInfo,
			   testSentenceInfo,
			   evalInfo,
			   totalInfo.numSent);
	    if (putTagsBack) {
		outTags.write(testTreeSexp+"\n");
	    }
	    displaySentenceResult(outResults,goldSentenceInfo,testSentenceInfo,evalInfo);
	    boolean nullSkip=nonulls && treeHasNoRoot(testTreeSexp);
	    if (treeHasNoRoot(goldTreeSexp) &&
		!nonulls) {
		System.out.println("WARNING: skipping gold tree with no root even though"+
				   " no nulls not specified"+goldTreeSexp);
		nullSkip=true;
	    }

	    updateTotalInfo(totalInfo,goldSentenceInfo,testSentenceInfo,evalInfo,nullSkip);
	    if (evalInfo.numTerminalsExceptForSome<=40) {
		updateTotalInfo(total40Info,goldSentenceInfo,testSentenceInfo,evalInfo,nullSkip);
	    }

	    writeToCrossingFile(goldSentenceInfo,
				testSentenceInfo,
				evalInfo);

	    /*
	      showBrackets(goldTerminalsNotDeleted,
	      goldBracketsNotDeleted,
	      testTerminalsNotDeleted,
	      testBracketsNotDeleted);
	    */
	    if (debug) 
		dspInfo(goldSentenceInfo.terminals,
			goldSentenceInfo.brackets,
			testSentenceInfo.terminals,
			testSentenceInfo.brackets);
	    

	    showDetailedInfo(goldSentenceInfo,testSentenceInfo,evalInfo);

	}

	void showDetailedInfo(SentenceInfo goldSentenceInfo,
			      SentenceInfo testSentenceInfo,
			      EvalInfo evalInfo) 
	    throws IOException {
	    ArrayList goldBrackets=goldSentenceInfo.brackets;
	    ArrayList testBrackets=testSentenceInfo.brackets;
	    resultsWriteLine("goldTree=\n"+
			     (treeHasNoRoot(goldSentenceInfo.originalTree) ? 
			      goldSentenceInfo.originalTree.toString() : 
			      danbikel.parser.util.Util.prettyPrint(goldSentenceInfo.originalTree)));
	    /*
	    resultsWriteLine("testTree=\n"+
			    (treeHasNoRoot(testSentenceInfo.originalTree) ? 
			     testSentenceInfo.originalTree.toString() :
			     danbikel.parser.util.Util.prettyPrint(testSentenceInfo.originalTree)));
	    */
	    if (treeHasNoRoot(testSentenceInfo.originalTree)) {
		resultsWriteLine("test is null:"+testSentenceInfo.originalTree);
	    } else {
		resultsWriteLine("testTree=\n"+
				 danbikel.parser.util.Util.prettyPrint(testSentenceInfo.originalTree));
	    }
	}

	
	void writeToCrossingFile(SentenceInfo goldSentenceInfo,
				 SentenceInfo testSentenceInfo,
				 EvalInfo evalInfo) 
	    throws IOException{
	    //	if (evalInfo.recall<60 || evalInfo.precision<50 ||
	    //	    evalInfo.numCrossing>5) {
	    crossingCases.add(new CrossingSummary(sentNum,
						  evalInfo.recall,
						  evalInfo.precision,
						  evalInfo.numCrossing));
	    outCrossing.write("==================================\n");
	    outCrossing.write("Sentence # " + sentNum+"\n");
	    outCrossing.write("Recall/Precision = " + 
			      rightJustify(6,percentage.format(evalInfo.recall))+"/" +
			      rightJustify(6,percentage.format(evalInfo.precision))+"\n");
	    outCrossing.write("# Crossing Brackets = " + evalInfo.numCrossing+"\n");
	    outCrossing.write("==================================\n");
	    outCrossing.flush();
	    outCrossing.write("GOLD TREE=\n"+
			      (treeHasNoRoot(goldSentenceInfo.originalTree) ? 
			       goldSentenceInfo.originalTree.toString() : 
			       Util.prettyPrint(goldSentenceInfo.originalTree))
			      +"\n");
	    outCrossing.flush();
	    outCrossing.write("TEST TREE=\n"+
			      (treeHasNoRoot(testSentenceInfo.originalTree) ? 
			       testSentenceInfo.originalTree.toString() : 
			       Util.prettyPrint(testSentenceInfo.originalTree))
			      +"\n");
	    for (int i=0; i<evalInfo.crossingBrackets.size(); i++) {
		CrossingBracketInfo crossing=
		    (CrossingBracketInfo)evalInfo.crossingBrackets.get(i);
	        outCrossing.write("Crossing Bracket # " + i+"\n");
		
		outCrossing.write("  Gold = "+crossing.goldNode.label+" "+
				  Util.collectLeaves(crossing.goldNode.nodeInOriginalTree)+"\n");
		outCrossing.write("  Test = "+crossing.testNode.label+" "+
				  Util.collectLeaves(crossing.testNode.nodeInOriginalTree)+"\n");
		outCrossing.flush();
	    }
	    //	}
	}
	    
	

	void dspInfo(ArrayList goldTerminals,
		     ArrayList goldBrackets,
		     ArrayList testTerminals,
		     ArrayList testBrackets) 
	    throws IOException {
	    //      DecimalFormat rightJustifiedThree=new DecimalFormat("000");
	    resultsWrite("-<1>---(wn1="+
			 rightJustify(3,goldTerminals.size())+
			 ", bn1="+
			 rightJustify(3,goldBrackets.size())+
			 ")-           ");
	    resultsWriteLine("-<2>---(wn2="+
			     rightJustify(3,testTerminals.size())+
			     ", bn2="+
			     rightJustify(3,testBrackets.size())+
			     ")-");
	    int n=(goldTerminals.size()>testTerminals.size() ?
		   goldTerminals.size() :
		   testTerminals.size());
	    for (int i=0; i<n; i++) {
		if (i<goldTerminals.size()) {
		    Terminal goldTerminal=(Terminal)goldTerminals.get(i);
		    resultsWrite(rightJustify(3,i)+" : "+
				 goldTerminal.result +
				 " : "+
				 leftJustify(6,goldTerminal.label.toString())+
				 "  "+
				 leftJustify(16,goldTerminal.text.toString())+
				 "      ");
		} else {
		    resultsWrite("                                        ");
		}
		if (i<testTerminals.size()) {
		    Terminal testTerminal=(Terminal)testTerminals.get(i);
		    resultsWriteLine(rightJustify(3,i)+" : "+
				     testTerminal.result +
				     " : "+
				     leftJustify(6,testTerminal.label.toString())+
				     "  "+
				     leftJustify(16,testTerminal.text.toString()));
		} else {
		    resultsWriteLine("");
		}
	    }
	    resultsWriteLine("");

	    n=(goldBrackets.size()>testBrackets.size() ?
	       goldBrackets.size() :
	       testBrackets.size());
	    for (int i=0; i<n; i++) {
		if (i<goldBrackets.size()) {
		    Bracket goldBracket=(Bracket)goldBrackets.get(i);
		    /*
		      System.out.println("goldBracket="+goldBracket);
		      System.out.println("goldBracket.label="+goldBracket.label);
		      System.out.println("goldBracket.strippedLabel="+goldBracket.strippedLabel);
		      System.out.println("goldBracket.nonterminal="+goldBracket.nonterminal.toString());
		    */
		    String goldLabelStr=goldBracket.label.toString();
		    if (goldBracket.strippedLabel!=null) {
			if (funcTagsCount) {
			    goldLabelStr=goldBracket.nonterminal.toString();
			} else {
			    goldLabelStr=goldBracket.strippedLabel.toString();
			}
		    }
		    resultsWrite(rightJustify(3,i)+" : "+
				 goldBracket.result +
				 " : "+
				 rightJustify(3,goldBracket.leftIdx)+
				 "  "+
				 rightJustify(3,goldBracket.rightIdx)+
				 "  "+
				 //			     leftJustify(6,goldBracket.label.toString())+
				 leftJustify(6,goldLabelStr)+
				 "      ");
		} else {
		    resultsWrite("                                ");
		}
		if (i<testBrackets.size()) {
		    Bracket testBracket=(Bracket)testBrackets.get(i);
		    /*
		      System.out.println("testBracket="+testBracket);
		      System.out.println("testBracket.label="+testBracket.label);
		      System.out.println("testBracket.strippedLabel="+testBracket.strippedLabel);
		      System.out.println("testBracket.nonterminal="+testBracket.nonterminal.toString());
		    */
		    String testLabelStr=testBracket.label.toString();
		    if (testBracket.strippedLabel!=null) {
			if (funcTagsCount) {
			    testLabelStr=testBracket.nonterminal.toString();
			} else {
			    testLabelStr=testBracket.strippedLabel.toString();
			}
		    }
		    resultsWriteLine(rightJustify(3,i)+" : "+
				     testBracket.result +
				     " : "+
				     rightJustify(3,testBracket.leftIdx)+
				     "  "+
				     rightJustify(3,testBracket.rightIdx)+
				     "  "+
				     leftJustify(6,testLabelStr));
		    //				 leftJustify(6,testBracket.label.toString()));
		} else {
		    resultsWriteLine("");
		}
	    }
	    resultsWriteLine("");
	    resultsWriteLine("========");
	
	
	}
      
    
	void showBrackets(ArrayList goldTerminalsNotDeleted,
			  ArrayList goldBracketsNotDeleted,
			  ArrayList testTerminalsNotDeleted,
			  ArrayList testBracketsNotDeleted) 
	    throws IOException {
	    resultsWriteLine("gold Terminals");
	    for (int i=0; i<goldTerminalsNotDeleted.size(); i++) {
		resultsWriteLine(i+") "+goldTerminalsNotDeleted.get(i));
	    }
	
	    resultsWriteLine("gold Brackets");
	    for (int i=0; i<goldBracketsNotDeleted.size(); i++) {
		resultsWriteLine(i+") "+goldBracketsNotDeleted.get(i));
	    }
	}
    
	/* 
	   match brackets by setting result of goldBracket and testBracket to MATCH
	   increment evalInfo.numMatch
	   add to collection of mapping normalizedLabels in numMatchingNormalizedLabels.
	   This is used later on to get the P/R/F on each bracket
	*/
	void matchBracketsPR(SentenceInfo goldSentenceInfo,
			     SentenceInfo testSentenceInfo,
			     EvalInfo evalInfo,
			     int sentenceNum)
	    throws IOException {
	    evalInfo.numMatch=0;
	    for (int i=0; i<goldSentenceInfo.brackets.size(); i++) {
		for (int j=0; j<testSentenceInfo.brackets.size(); j++) {
		    Bracket goldBracket=(Bracket)goldSentenceInfo.brackets.get(i);
		    Bracket testBracket=(Bracket)testSentenceInfo.brackets.get(j);
		    if (goldBracket.result!=DELETE &&
			testBracket.result==UNMATCH &&
			goldBracket.leftIdx==testBracket.leftIdx &&
			goldBracket.rightIdx==testBracket.rightIdx) {
			//		  if (labelComp(goldBracket.strippedLabel,testBracket.strippedLabel)) {
			if (labelComp(goldBracket,testBracket)) {
			    outMatchInfo.write("match:"+sentNum+
					       " gold:("+goldBracket.leftIdx+","+goldBracket.rightIdx+
					       "):"+goldBracket.label+
					       " test:("+testBracket.leftIdx+","+testBracket.rightIdx+
					       "):"+testBracket.label+"\n");
			    goldBracket.result=MATCH;
			    testBracket.result=MATCH;
			    evalInfo.numMatch++;
			    evalInfo.numMatchingNormalizedLabels.add(goldBracket.normalizedLabel);
			    evalInfo.numMatchingNormalizedLabels.add(allLabels);
			    if (putTagsBack) {
				Sexp goldNode=goldBracket.node.nodeInOriginalTree;
				Sexp testNode=testBracket.node.nodeInOriginalTree;
				if (goldNode.list().get(0).isSymbol()) {
				    /*
				    System.out.println("put tags back " + " gold="+goldNode+
						       " test="+testNode);
				    System.out.println("goldBracket.label="+goldBracket.label);
				    */
				    Symbol goldLabel=goldNode.list().get(0).symbol();
				    Symbol testLabel=testNode.list().get(0).symbol();
				    //				    System.out.println("goldLabel="+goldLabel+" testLabel="+testLabel);
				    if (goldLabel!=testLabel) {
					testNode.list().set(0,goldLabel);
					// System.out.println("reset testLabel to " + goldLabel);
				    }
				}
							   
			    }


				
			    /* need to do by copy of symbol rather than adding of augmentation sbecause
			       of indices and gapping.  well, actually don't need to but it's easier
			    */
			    /*
			    System.out.println("matching bracket : add augmentations from " + 
					       goldBracket.label + 
					       " to " + testBracket.label);
			    */
			    break;
			} else {
			    outMatchInfo.write("mismatch:"+sentNum+
					       " gold:("+goldBracket.leftIdx+","+goldBracket.rightIdx+
					       "):"+goldBracket.label+
					       " test:("+testBracket.leftIdx+","+testBracket.rightIdx+
					       "):"+testBracket.label+"\n");
			}

		    }
		}
	    }

	}
    
	void findBracketInstances(SentenceInfo goldSentenceInfo,
				  SentenceInfo testSentenceInfo,
				  EvalInfo evalInfo,
				  int sentenceNum) {
	    for (int i=0; i<goldSentenceInfo.brackets.size(); i++) {
		Bracket goldBracket=(Bracket)goldSentenceInfo.brackets.get(i);
		if (goldBracket.result!=DELETE) {
		    if (goldBracket.result==MATCH) {
			addToHashMap(goldBracket,
				     evalInfo.bracketsMatching);
		    } else {
			addToHashMap(goldBracket,
				     evalInfo.bracketsMiss);
		    }
		}
	    }


	    for (int i=0; i<testSentenceInfo.brackets.size(); i++) {
		Bracket testBracket=(Bracket)testSentenceInfo.brackets.get(i);
		if (testBracket.result!=DELETE &&
		    testBracket.result!=MATCH) {
		    addToHashMap(testBracket,
				 evalInfo.bracketsFalseAlarm);
		}
	    }
	}
    
	/* count up # of crossing cases and add eac one to evalInfo.crossingBrackets
	 */
	void findCrossingCases(SentenceInfo goldSentenceInfo,
			       SentenceInfo testSentenceInfo,
			       EvalInfo evalInfo,
			       int sentenceNum)
	    throws IOException {
	    evalInfo.numCrossing=0;
	    for (int j=0; j<testSentenceInfo.brackets.size(); j++) {
		for (int i=0; i<goldSentenceInfo.brackets.size(); i++) {
		    Bracket goldBracket=(Bracket)goldSentenceInfo.brackets.get(i);
		    Bracket testBracket=(Bracket)testSentenceInfo.brackets.get(j);
		    if ((goldBracket.leftIdx<testBracket.leftIdx &&
			 goldBracket.rightIdx>testBracket.leftIdx &&
			 goldBracket.rightIdx<testBracket.rightIdx) ||
			(goldBracket.leftIdx>testBracket.leftIdx &&
			 goldBracket.leftIdx<testBracket.rightIdx &&
			 goldBracket.rightIdx>testBracket.rightIdx)) {
			evalInfo.numCrossing++;
			evalInfo.crossingBrackets.add(new 
			    CrossingBracketInfo(goldBracket.node,
						testBracket.node));
			/*
			  outCrossing.write("CROSSING\n");
			  outCrossing.write("gold=\n");
			  outCrossing.write(goldBracket.node.nodeInOriginalTree.toString()+"\n");
			  outCrossing.write("test=\n");
			  outCrossing.write(testBracket.node.nodeInOriginalTree.toString()+"\n");
			*/
			break;
		    }
		}
	    }
	}
    
	void calcBracketsPR(SentenceInfo goldSentenceInfo,
			    SentenceInfo testSentenceInfo,
			    EvalInfo evalInfo,
			    int sentenceNum)
	    throws IOException {

	    evalInfo.numCorrectTag=0;
	    for (int i=0; i<goldSentenceInfo.terminals.size(); i++) {
		Terminal goldTerminal=(Terminal)goldSentenceInfo.terminals.get(i);
		Terminal testTerminal=(Terminal)testSentenceInfo.terminals.get(i);
		if (labelComp(goldTerminal.label,testTerminal.label)) {
		    goldTerminal.result=MATCH;
		    testTerminal.result=MATCH;
		    evalInfo.numCorrectTag++;
		} else {
		    goldTerminal.result=UNMATCH;
		    testTerminal.result=UNMATCH;
		}
	    }
	    evalInfo.recall= 
		goldSentenceInfo.numBracketsAfterDeletion==0 ?
		0.0 :
		100.00 * evalInfo.numMatch / 
		goldSentenceInfo.numBracketsAfterDeletion;

	    evalInfo.precision = 
		testSentenceInfo.numBracketsAfterDeletion==0 ?
		0.0 :
		100.00 * evalInfo.numMatch / 
		testSentenceInfo.numBracketsAfterDeletion;

	    evalInfo.tagAccuracy=
		goldSentenceInfo.terminals.size()==0 ?
		0.0 :
		100.0*evalInfo.numCorrectTag / goldSentenceInfo.terminals.size();

	}
    
            
	void printHead() 
	    throws IOException {
	    resultsWriteLine("  Sent.                        Matched  Bracket   Cross        Correct Tag");
	    resultsWriteLine(" ID  Len.  Stat. Recal  Prec.  Bracket gold test Bracket Words  Tags Accracy");
	    resultsWriteLine("============================================================================");

	}

    
	void displaySentenceResult(BufferedWriter out,
				   SentenceInfo goldSentenceInfo,
				   SentenceInfo testSentenceInfo,
				   EvalInfo evalInfo) 
	    throws IOException {
	
	    out.write(rightJustify(4,sentNum)+"  "+
		      rightJustify(3,evalInfo.numTerminalsExceptForSome)+"    "+
		      status+"  ");
	
	    out.write(rightJustify(6,percentage.format(evalInfo.recall))+" "+
		      rightJustify(6,percentage.format(evalInfo.precision))+"   "+
		      rightJustify(3,evalInfo.numMatch)+"    "+
		      rightJustify(3,goldSentenceInfo.numBracketsAfterDeletion)+"  "+
		      rightJustify(3,testSentenceInfo.numBracketsAfterDeletion)+"    "+
		      rightJustify(3,evalInfo.numCrossing));
	    out.write("   "+
		      rightJustify(4,goldSentenceInfo.terminals.size())+"  "+
		      rightJustify(4,evalInfo.numCorrectTag)+"   "+
		      rightJustify(6,percentage.format(evalInfo.tagAccuracy))+"\n");
		       
	}

	/* 
	   goldSentenceInfo and testInfo have the info on the number of brackets
	   evalInfo has the info on how many matches
       
	   numBracketsAfterDeletion is the size of 
	   sentenceInfo.brackets not including ones 
	   with result==DELETE.  That is set in massageData
	   for brackets with leftIdx==rightIdx, or a bracket in
	   deleteLabels. Actually the latter should never happen
	   since such brackets are not even included in bracket
	   begin with.

	   To keep track of individual nonterminal results:
	   make up a new class with (numGoldBrackets,numTestBrackets,
	   numMatching) and make a hash map with key=nonterminal
	   label. While at it, might as well do it for all
	   brackets a well, so everything is contained in evalInfo

	   Have same maps in totalInfo and modify updattotal accordingly
	*/



	void updateTotalInfo(TotalInfo totalInfo,
			     SentenceInfo goldSentenceInfo,
			     SentenceInfo testSentenceInfo,
			     EvalInfo evalInfo, boolean nullSkip) {

	    totalInfo.numSent++;
	    if (status==1) {
		totalInfo.numErrorSent++;
	    } else if (status==2) {
		totalInfo.numSkipSent++;
	    } else {
		if (nullSkip) {
		    totalInfo.numNullSent++;
		    return;
		}
		totalInfo.numGoldBrackets+=goldSentenceInfo.numBracketsAfterDeletion;
		totalInfo.numTestBrackets+=testSentenceInfo.numBracketsAfterDeletion;
		totalInfo.numMatch+=evalInfo.numMatch;
		if ((goldSentenceInfo.numBracketsAfterDeletion==
		     testSentenceInfo.numBracketsAfterDeletion) 
		    &&
		    (testSentenceInfo.numBracketsAfterDeletion==
		     evalInfo.numMatch)) {
		    totalInfo.numCompleteSent++;
		}
		totalInfo.numWords+=goldSentenceInfo.terminals.size();
		totalInfo.numCrossing+=evalInfo.numCrossing;
		if (evalInfo.numCrossing==0) {
		    totalInfo.numNoCrossingSent++;
		}
		if (evalInfo.numCrossing<=2) {
		    totalInfo.num2LCrossingSent++;
		}
		totalInfo.numCorrectTag+=evalInfo.numCorrectTag;
		totalInfo.numGoldNormalizedLabels.addAll(goldSentenceInfo.numNormalizedLabels);
		totalInfo.numTestNormalizedLabels.addAll(testSentenceInfo.numNormalizedLabels);	    
		totalInfo.numMatchingNormalizedLabels.addAll(evalInfo.numMatchingNormalizedLabels);
		//System.out.println("updating total for sentence " + totalInfo.numSent);
		//		System.out.println("matching:");
		addToHashMap(totalInfo.numSent,evalInfo.bracketsMatching,totalInfo.bracketsMatching);
		//		System.out.println("miss:");
		addToHashMap(totalInfo.numSent,evalInfo.bracketsMiss,totalInfo.bracketsMiss);
		//		System.out.println("false alarm:");
		addToHashMap(totalInfo.numSent,evalInfo.bracketsFalseAlarm,totalInfo.bracketsFalseAlarm);
	    }
	}

    
			  
	int makeUpLength(SentenceInfo sentenceInfo) {
	    int len=0;
	    for (int i=0; i<sentenceInfo.terminalsAll.size(); i++) {
		Terminal terminal=(Terminal)sentenceInfo.terminalsAll.get(i);
		if (!deleteLabelsForLength.contains(terminal.label)) {
		    len++;
		}
	    }
	    return len;
	}
	/* Some Brackets are marked as deleted in ParseTreeNode.
	   These are the ones for which the label is in deleteLabels.
	   In these cases,  ParseTreeNode.deleted=true and
	   rightIdxDeleted=-1.

	   Only brackets that with ParseTreeNode.deleted==false
	   are added to sentenceInfo.brackets.  massageDAta goes
	   through these brackets, not bracketsAll, so such deleted
	   brackes are not counted at all.  This is sort of annoying,
	   but it's done to be consistent with the output of evalb.
       
	   However, it's further possible that a bracket covers 
	   the null string (leftIdx==rightIdx), in which case its
	   result is set to DELETE, and is otherwise initialized to
	   UNMATCH.

	   For some reason this code is also checking deleteLabels
	   here.  This should never happens, since any bracket
	   which is in this should never have been added to the
	   bracket array.

	*/

	/* can't depend on correct stripIndex being used,
	   so copy stripIndex from Treebank2 into this, 
	   and make up own code to call parseNonterminal
	   and then go through augmentations and delete
	   label and default arg augmentation
	*/

	void massageData(SentenceInfo sentenceInfo) {
    
	    for (int i=0; i<sentenceInfo.brackets.size(); i++) {
		Bracket bracket=(Bracket)sentenceInfo.brackets.get(i);
		bracket.result=UNMATCH;
		if (bracket.leftIdx==bracket.rightIdx) {
		    bracket.result=DELETE;

		} else {
		    bracket.strippedLabel=stripAugmentation(bracket.label);
		    bracket.normalizedLabel=normalizeLabel(bracket.strippedLabel);
		    bracket.nonterminal=new Nonterminal();
		    treebank.parseNonterminal(bracket.label,bracket.nonterminal);
		    stripIndex(bracket.nonterminal);
		    stripDefaultArg(bracket.nonterminal);
	      
	      
		    for (int j=0; j<deleteLabels.size(); j++) {
			if (labelComp(bracket.strippedLabel,(Symbol)deleteLabels.get(j))) {
			    System.out.println("Bracket " + bracket + " has deleted label " + 
					       bracket.label);
			    bracket.result=DELETE;
			}
		    }
		}

	    }
	    sentenceInfo.numBracketsAfterDeletion=0;
	    for (int i=0; i<sentenceInfo.brackets.size(); i++) {
		Bracket bracket=(Bracket)sentenceInfo.brackets.get(i);
		if (bracket.result!=DELETE) {
		    sentenceInfo.numBracketsAfterDeletion++;
		}
	    }

	    /* update number of gold and sentence brackets for each
	       nonterminal type */

	    for (int i=0; i<sentenceInfo.brackets.size(); i++) {
		Bracket bracket=(Bracket)sentenceInfo.brackets.get(i);
		if (bracket.result!=DELETE) {
		    sentenceInfo.numNormalizedLabels.add(bracket.normalizedLabel);
		    sentenceInfo.numNormalizedLabels.add(allLabels);
		}
	    }
	}
	/*
	  It is possible that a treebank.java will not include some
	  delimeter, such as =.  evalb strips off everything 
	  with - or =, so we need ot do the same here, regardless
	  of what treebank.stripAugmentation is.
	*/
    
	public void stripIndex(Nonterminal nonterminal) {
	    if (nonterminal.index == -1)
		return;
	    // the last element in the augmentations list was the delimiter
	    // between the pre-index part of the nonterminal and the index
	    nonterminal.augmentations.remove(nonterminal.augmentations.size() - 1);
	    nonterminal.index = -1; // "remove" the index

	    /* check for the case of an additional index */
	
	    SexpList augmentations = nonterminal.augmentations;
	    for (int i = 0; i < augmentations.length(); i++) {
		Sexp thisAug = augmentations.get(i);
		if (!Language.treebank().isAugDelim(thisAug)) {
		    String thisAugStr=thisAug.toString();
		    if (Text.isAllDigits(thisAugStr)) {
			i--; // move index to delimeter that preceded this arg augmentation
			augmentations.remove(i);  // remove delimeter
			augmentations.remove(i);  // remove arg augmentation
			i--; // decrement index again to offset increment in for loop
		    }
		}
	    }
	}
	public void stripDefaultArg(Nonterminal nonterminal) {
	    SexpList augmentations = nonterminal.augmentations;
	    for (int i = 0; i < augmentations.length(); i++) {
		if (defaultArgAugmentation==augmentations.get(i)) {
		    i--; // move index to delimeter that preceded this arg augmentation
		    augmentations.remove(i);  // remove delimeter
		    augmentations.remove(i);  // remove arg augmentation
		    i--; // decrement index again to offset increment in for loop
		}
	    }
	}
	


	public Symbol stripAugmentation(Symbol label) {
	    String labelStr = label.toString();
	    int labelStrLen = labelStr.length();
	    int augmentationIdx = -1;

	    for (int i = 0; i < labelStrLen; i++) {
		if (augmentationDelimSet.get(labelStr.charAt(i))) {
		    augmentationIdx = i;
		    break;
		}
	    }
	    return ((augmentationIdx == -1) ? label :
		    Symbol.add(labelStr.substring(0, augmentationIdx)));
	}    
    
	/* regardless of how Treebank is setup, strip all indices
	   (including case of two indices), and default Arg
	   delimiter

	   What we really want to know is:
	   What is being stripped off of each label by training?
	   We want to duplicate that, plus in addition strip off
	   any defaulArg tag, in case we are processing a unpostprocessed
	   test file

	   So actually the easiest way to do this is take the tree
	   and run it through stripAugmentations or stripIndicies
	   However, the problem here is that we do not know whether
	   it is running using stripIndices or stripArgumentations.
	   This is why I should have perhaps overrode stripArgumentations.
	   But actually on, because subcatBag uses stripArgumentations
	   while training uses stripIndices.

	   Really need ot make a new AbstractTraining2 with my own
	   interface for anything I might need.

	   But for now, want to know:
	   1) what function tags do we care about - only the
	   ones resulting from the training process. There are really
	   three cases:

	   a. The test code recovers all function tags.
	   So strip out only defaultArgAugmentation and
	   index.  Use treebanbank.stripIndex
	   b. The test code recovers only the argument
	   synonym tags.  Can't quite use Training.stripAugmentations
	   since this works on a full tree.  But can
	   get list of tags from  Training.argNonterminals() and
	   so do it myself.  Also need to stripIndex and
	   defaultArgAugmentations
	   c. Bare label - strip index, strip all labels
	   - use stripAugmentation as currently doing.
	*/



	void getBrackets(ParseTreeNode tree, 
			 SentenceInfo sentenceInfo) {

	    //	    System.out.println("getBackets tree="+tree);

	    if (tree.isPreterminal()) {
		//		System.out.println("preterminal"+tree);
		sentenceInfo.terminalsAll.add(new Terminal(tree,
							   tree.text,
							   tree.label,
							   tree.leftIdx));
		if (!tree.deleted) {
		    sentenceInfo.terminals.add(new Terminal(tree,
							    tree.text,
							    tree.label,
							    tree.leftIdxDeleted));
		}
	    } else {
		//		System.out.println("bracket " + tree);
		sentenceInfo.bracketsAll.add(new Bracket(tree,
							 tree.label,
							 tree.leftIdx,
							 tree.rightIdx));
						     
		if (!tree.deleted) {
		    sentenceInfo.brackets.add(new Bracket(tree,
							  tree.label,
							  tree.leftIdxDeleted,
							  tree.rightIdxDeleted));
		}
		ListIterator it=tree.children.listIterator();
		while (it.hasNext()) {
		    ParseTreeNode child=(ParseTreeNode)it.next();
		    getBrackets(child,sentenceInfo);
		}
	    }
	}

	private static void usage() {
	    System.err.println("bad usage!");
	    System.exit(1);
	}
	/*
	  private static boolean treeIsNull(Sexp tree) {
	  if (tree.isSymbol() &&
	  tree.symbol().toString().equals("null"))
	  return true;
	  else 
	  return false;
	  }
	*/
	private static boolean treeHasNoRoot(Sexp tree) {
	    if (tree.isList()) {
		return (!tree.list().get(0).isSymbol());
	    } 
	    return false;
	}

	private static Sexp adjustForParens(Sexp tree) {
	    boolean stripOuterParens = (tree.list().length() == 1 &&
					tree.list().get(0).isList());
	    if (stripOuterParens)
		tree = tree.list().get(0);
	
	    return tree;
	}
	
    
	public void initStuff() {

	    deleteLabels=new ArrayList();
	    deleteLabels.add(Symbol.add("TOP"));
	    deleteLabels.add(Symbol.add("-NONE-"));
	    deleteLabels.add(Symbol.add(","));
	    deleteLabels.add(Symbol.add(":"));
	    deleteLabels.add(Symbol.add("``"));
	    deleteLabels.add(Symbol.add("''"));
	    deleteLabels.add(Symbol.add("."));
	    ParseTreeNode.setDeleteLabels(deleteLabels);

	    deleteLabelsForLength=new ArrayList();
	    deleteLabelsForLength.add(Symbol.add("-NONE-"));

	    equivs=new Equiv[3];
	    equivs[0]=new Equiv(Symbol.add("ADVP"),Symbol.add("PRT"));
	    equivs[1]=new Equiv(Symbol.add("SG"),Symbol.add("S"));
	    equivs[2]=new Equiv(Symbol.add("NPB"),Symbol.add("NP"));
	
	}
	Symbol normalizeLabel(Symbol s) {
	    for (int i=0; i<equivs.length; i++) {
		if (s==equivs[i].s1) {
		    return equivs[i].s2;
		}
	    }
	    return s;
	}

	boolean labelComp(Bracket bracket1, Bracket bracket2) {
	    if (!funcTagsCount) {
		return labelComp(bracket1.strippedLabel,bracket2.strippedLabel);
	    }
	    // first make sure bases match
	    if (!labelComp(bracket1.strippedLabel,bracket2.strippedLabel)) {
		return false;
	    }
	    // now check augmentations

	    if (!checkContainsAll(bracket1.nonterminal.augmentations,
				  bracket2.nonterminal.augmentations)) 
		return false;
	    if (!checkContainsAll(bracket2.nonterminal.augmentations,
				  bracket1.nonterminal.augmentations)) 
		return false;
	    return true;
	}	
	boolean checkContainsAll(SexpList augmentations1, SexpList augmentations2) {
	    int numaug1=augmentations1.length();
	    int numaug2=augmentations2.length();	    
	    for (int i=0; i<numaug1; i++) {
		Sexp thisAug = augmentations1.get(i);	    
		if (!augmentations2.contains(thisAug)) {
		    return false;
		}
	    }
	    return true;
	}
	
	boolean labelComp(Symbol s1, Symbol s2) {
	    if (s1==s2) {
		return true;
	    }
	    for (int i=0; i<equivs.length; i++) {
		if (((s1==equivs[i].s1) &&
		     (s2==equivs[i].s2)) 
		    ||
		    ((s1==equivs[i].s2) &&
		     (s2==equivs[i].s1))) {
		    return true;
		}
	    }
	    return false;
	}

	static final String blanks="                                   ";
	static String rightJustify(int len, int num) {
	    return rightJustify(len,num+"");
	}

	static String rightJustify(int len, String t) {
	    if (t.length()>len) {
		return t;
	    } else {
		return (blanks.substring(0,len-t.length())+t);
	    }
	}
	static String makePer(int num, int den) {
	    double t  =
		(den==0 ? 0.0 : (1.00*num/den));
	    return (rightJustify(6,percentage.format(t)));
	}


	static String leftJustify(int len, String t) {
	    if (t.length()>len) {
		return t;
	    } else {
		return (t+blanks.substring(0,len-t.length()));
	    }
	}
	static String leftJustify(int len, int num) {
	    return leftJustify(len,num+"");
	}

	public void addToHashMap(int sentNum,
				 java.util.HashMap evalInfoMap,
				 java.util.HashMap totalInfoMap) {
	    Iterator keys=evalInfoMap.keySet().iterator();	
	    while (keys.hasNext()) {
		Symbol key=(Symbol)keys.next();
		//		System.out.println("addToHashMap key="+key);
		ArrayList totalInstances=(ArrayList)totalInfoMap.get(key);
		if (totalInstances==null) {
		    totalInstances=new ArrayList();
		    totalInfoMap.put(key,totalInstances);
		}	    
		totalInstances.add(new SentenceBrackets(sentNum,(ArrayList)evalInfoMap.get(key)));
	    }
	}

	public void addToHashMap(Bracket bracket,
				 java.util.HashMap hashMap) {
	    Symbol key=bracket.normalizedLabel;
	
	    ArrayList instances=(ArrayList)hashMap.get(key);
	    if (instances==null) {
		instances=new ArrayList();
		hashMap.put(key,instances);
	    }
	    instances.add(bracket);
	}
    
	public static void main(String[] args) {

	    String goldFilename=null;
	    String testFilename=null;
	    String resultsFilename=null;
	    String matchInfoFilename=null;
	    String crossingFilename=null;
	    String tagsFilename=null;
	    boolean debug=false;
	    boolean funcTagsCount=false;
	    boolean nonulls=false;
	    boolean putTagsBack=false;
	    for (int i=0; i<args.length; i++) {
		if (args[i].charAt(0)=='-') {
		    if (args[i].equals("-g")) {
			if (i+1==args.length)
			    usage();
			goldFilename=args[++i];
		    } else if (args[i].equals("-t")) {
			if (i+1==args.length)
			    usage();
			testFilename=args[++i];		    
		    } else if (args[i].equals("-debug")) {
			debug=true;
		    } else if (args[i].equals("-nonulls")) {
			nonulls=true;
		    } else if (args[i].equals("-func")) {
			funcTagsCount=true;
		    } else if (args[i].equals("-tags")) {
			putTagsBack=true;
		    }
		}
	    }
	    String extraFilename=".pd";
	    if (funcTagsCount) 
		extraFilename=".pd.func";
	    if (nonulls) 
		extraFilename+=".nonulls";
	    
	    if (resultsFilename==null) {
		//	    resultsFilename=testFilename+".pd.results";	    
		resultsFilename=testFilename+extraFilename+".results";	    
	    }
	    if (crossingFilename==null) {
		//	    crossingFilename=testFilename+".pd.crossing";
		crossingFilename=testFilename+extraFilename+".crossing";
	    }

	    if (matchInfoFilename==null) {
		//	    	    matchInfoFilename=testFilename+".pd.matchinfo";
		matchInfoFilename=testFilename+extraFilename+".matchinfo";
	    }

	    if (tagsFilename==null) {
		//	    crossingFilename=testFilename+".pd.crossing";
		tagsFilename=testFilename+".tags";
	    }


	    System.out.println("goldFilename="+goldFilename);
	    System.out.println("testFilename="+testFilename);
	    System.out.println("resultsFilename="+resultsFilename);
	    System.out.println("crossingFilename="+crossingFilename);
	    System.out.println("matchInfoFilename="+matchInfoFilename);
	    System.out.println("tagsFilename="+tagsFilename);
	    System.out.println("debug="+debug);
	    System.out.println("nonulls="+nonulls);
	    System.out.println("funcTagsCount "+funcTagsCount);
	    ParseDebug parseDebug=new ParseDebug();
	    parseDebug.process(goldFilename,testFilename,
			       resultsFilename,
			       matchInfoFilename,
			       crossingFilename,
			       tagsFilename,
			       debug,
			       funcTagsCount,nonulls,
			       putTagsBack);
	}
    }



