/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
package danbikel.parser.ppceme;

import java.util.*;
import java.io.*;
import danbikel.lisp.*;
import danbikel.parser.*;


public class Training10 extends Training7 {

    private Symbol topSym=Symbol.add("TOP");
    private Nonterminal nonterminal9 = new Nonterminal();

    public Training10() throws FileNotFoundException, IOException {
	super();
	System.out.println("ppceme Training10 constructor");
    }
    public Sexp preProcess(Sexp tree) {
	parentAnnotation(tree);
	stripDollarSigns(tree);
	prune(tree);
	//addBaseNPs(tree);
	//repairBaseNPs(tree);
	//    addGapInformation(tree);
	//    relabelSubjectlessSentences(tree);
	removeNullElements(tree);
	raisePunctuation(tree);
	identifyArguments(tree);
	stripAugmentations(tree);
	//fixSubjectlessSentences(tree);  this method now in English lang. package
	return tree;
    }

    public void postProcess(Sexp tree) {
	//unrepairBaseNPs(tree);
	//    super.postProcess(tree);
	//removeOnlyChildBaseNPs(tree);
	//System.out.println("after remove basenps = " + tree);
	//    canonicalizeNonterminals(tree);

	//raiseCodeNodes(tree);
	//System.out.println("after raiseCodeNodes " + tree);
	System.out.println("before undoparent annotation"+tree);
	undoParentAnnotation(tree);
	System.out.println("before stripArgAugAndCanonicalizeBases" + tree);	
	stripArgAugAndCanonicalizeBases(tree);
	System.out.println("after stripArgAugAndCanonicalizeBases" + tree);
    }

    public Sexp parentAnnotation(Sexp tree) {
	parentAnnotation(tree,topSym);
	return tree;
    }

    public void parentAnnotation(Sexp tree, Symbol grandParentLabel) {
	if (treebank.isPreterminal(tree))
	    return;
	SexpList treeList = tree.list();
	int treeListLen = treeList.length();	
	Sexp parent=treeList.get(0);
	Symbol strippedParentLabel=treebank.stripAugmentation(parent.symbol());
	for (int j=1; j<treeListLen; j++) {
	    parentAnnotation(treeList.get(j),strippedParentLabel);
	}
	Nonterminal parsedParent =
	    treebank.parseNonterminal(parent.symbol(), nonterminal9);
	parsedParent.base=Symbol.add(parsedParent.base.toString()+"^"+grandParentLabel);
	treeList.set(0,parsedParent.toSymbol());
    }

    public void undoParentAnnotation(Sexp tree) {
	if (treebank.isPreterminal(tree))
	    return;
	SexpList treeList = tree.list();
	int treeListLen = treeList.length();	
	Sexp parent=treeList.get(0);
	Nonterminal parsedParent =
	    treebank.parseNonterminal(parent.symbol(), nonterminal9);
	String parentBaseStr=parsedParent.base.toString();
	int index=parentBaseStr.indexOf("^");
	if (index==-1) 
	    System.out.println("warning: no ^ in "+tree);
	parsedParent.base=Symbol.add(parentBaseStr.substring(0,index));
	treeList.set(0,parsedParent.toSymbol());
	for (int j=1; j<treeListLen; j++) {
	    undoParentAnnotation(treeList.get(j));
	}

    }

  /** Test driver for this class. */
  public static void main(String[] args) {
    String filename = null;
    boolean parentAnnotation=true;
    filename=args[0];
    if (filename == null) {
      usage();
    }
    
    Symbol test1=Symbol.add("NP^S-OB2");
    Training10 training = (Training10)Language.training();
    danbikel.parser.Treebank treebank = Language.treebank();
    System.out.println("getCanonical("+test1+")="+treebank.getCanonical(test1));
    System.out.println("isNP="+treebank.isNP(test1));
    //training.printMetadata();

    try {
      SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
					    Constants.defaultFileBufsize);
      Sexp curr = null;
      int sentNum=0;
      while ((curr = Sexp.read(tok)) != null) {
	  String skipStr = training.skip(curr);
	  if (skipStr != null) {
	      System.err.println(": skipping tree No. " +
				 (sentNum + 1) + ": " + skipStr + 
				 " tree="+curr);
	      continue;
	  }
	  curr = training.parentAnnotation(curr);
	  System.out.println(curr);
	  training.undoParentAnnotation(curr);
	  System.out.println(curr);
	  System.out.println("------");

      }
    }
    catch (UnsupportedEncodingException uee) {
      System.err.println(uee);
    }
    catch (FileNotFoundException fnfe) {
      System.err.println(fnfe);
    }
    catch (IOException ioe) {
      System.err.println(ioe);
    }
  }
}

