/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
package danbikel.parser.ppceme;

import java.util.*;
import java.io.*;
import danbikel.lisp.*;
import danbikel.parser.*;


public class Training11 extends Training9 {


    public Training11() throws FileNotFoundException, IOException {
	super();
	System.out.println("ppceme Training11 constructor");
    }

    protected static String[] augsToKeepElements={"ABS","IMP","INF","MAT","PPL","SMC","SUB"};

  protected static Set augsToKeep = new HashSet();
  static {
      for (int i = 0; i < augsToKeepElements.length; i++) {
      augsToKeep.add(Symbol.add(augsToKeepElements[i]));
      System.out.println("keeping aug"+augsToKeepElements[i]);
      }

  }
    
  protected void stripAugmentations(Symbol label, Nonterminal nonterminal,
                                    boolean parseLabel) {
    if (parseLabel)
      treebank.parseNonterminal(label, nonterminal);
    SexpList augmentations = nonterminal.augmentations;
    for (int i = 0; i < augmentations.length(); i++) {
      Sexp thisAug = augmentations.get(i);
      if (!Language.treebank().isAugDelim(thisAug) &&
          thisAug != gapAugmentation &&
          !argAugmentations.contains(thisAug) &&
          !augsToKeep.contains(thisAug)) {
        i--; // move index to delimeter that preceded this arg augmentation
        augmentations.remove(i);  // remove delimeter
        augmentations.remove(i);  // remove arg augmentation
        i--; // decrement index again to offset increment in for loop
      }
    }
    // if there's an index, make sure to strip delimeter that precedes it
    if (nonterminal.index >= 0)
      augmentations.remove(augmentations.size() - 1);
    nonterminal.index = -1; // effectively strips off index
  }
    /*

    public Sexp convertCONJP(Sexp tree) {
        if (treebank.isPreterminal(tree)) 
            return tree;
        if (tree.isList()) {
            SexpList treeList = tree.list();
            int treeListLen = treeList.length();
            for (int j=1; j<treeListLen; j++) {
                convertCONJP(treeList.get(j));
            }
            //      System.out.println("convertCONJP "+tree);
            Symbol parentLabel=treebank.stripAugmentation(treeList.symbolAt(0));
	    Nonterminal parsedParent=treebank.parseNonterminal(treeList.symbolAt(0));
            //      System.out.println("grandparentLabel="+grandparentLabel);
            if (adjunctStructure(treeList,parentLabel)) {
                //              System.out.println("is AdjunctStructure");
                for (int j=2; j<treeListLen; j++) {
                    Sexp child = treeList.get(j);
                    //                  System.out.println("child= "+child);
                    if (isCONJPXP(child,parentLabel)) {
                        //                      System.out.println("isCONJPXP");
                        SexpList childList=child.list();
                        Nonterminal parsedChild =
                            treebank.parseNonterminal(childList.symbolAt(0));
                        parsedChild.base=Symbol.add("CONJP"+parentLabel.toString());
			if (parsedChild.augmentations.length()>0) {
			    System.err.println("CONJP has augmentations"+childList.symbolAt(0));
			} else {
			    for (int augIdx = 0; augIdx < parsedParent.augmentations.length(); augIdx++) {
				parsedChild.augmentations.add(parsedParent.augmentations.get(augIdx));
			    }
			}
                        childList.set(0,parsedChild.toSymbol());
                    }
                }
            }
        }
	return tree;
    }


    public void fixCONJP(Sexp tree) {
        if (treebank.isPreterminal(tree)) 
            return;
        if (tree.isList()) {
            SexpList treeList = tree.list();
            int treeListLen = treeList.length();
            String parentString=treeList.symbolAt(0).toString();
            if (parentString.startsWith(CONJPString) &&
                parentString.length()>CONJPString.length()) {
                treeList.set(0,CONJP);
            }
            for (int j=1; j<treeListLen; j++) {
                fixCONJP(treeList.get(j));
            }
        }
    }
    
    */

  /** Test driver for this class. */
  public static void main(String[] args) {
    String filename = null;
    boolean splitPRN=true;
    boolean convertConj=false;
    boolean stripDollarSigns = false;
    boolean prune = false;
    //    boolean addBaseNPs = false;
    //boolean repairBaseNPs= false;
    boolean removeNullElements = false;
    boolean raisePunc = false;
    boolean idArgs = false;
    boolean stripAug = false;
    boolean combineTransformations = false;

    for (int i = 0; i < args.length; i++) {
      // for each arg that begins with a '-', examine each of its letters
      // and set booleans accordingly, except for "-combine" flag
      if (args[i].charAt(0) == '-') {
	if (args[i].equals("-combine"))
	  combineTransformations = true;
	else {
	  for (int charIdx = 1; charIdx < args[i].length(); charIdx++) {
	    char curr = args[i].charAt(charIdx);
	    switch (curr) {
	    case 'd':
	      convertConj = true;
	      break;
	    case 'e':
	      splitPRN= true;
	      break;
	    case 't':
	      stripDollarSigns = true;
	      break;
	    case 'p':
	      prune = true;
	      break;
	      /*
	    case 'b':
	      addBaseNPs = true;
	      break;
	    case 'c':
	      repairBaseNPs = true;
	      break;
	      */
	    case 'n':
	      removeNullElements = true;
	      break;
	    case 'r':
	      raisePunc = true;
	      break;
	    case 'i':
	      idArgs = true;
	      break;
	    case 'a':
	      stripAug = true;
	      break;
	    default:
	      System.err.println("illegal flag: " + curr);
	      usage();
	    }
	  }
	}
      }
      else
	filename = args[i];
    }

    if (filename == null) {
      usage();
    }

    Training11 training = (Training11)Language.training();
    //training.printMetadata();

    try {
      SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
					    Constants.defaultFileBufsize);
      Sexp curr = null;
      int sentNum=0;
      while ((curr = Sexp.read(tok)) != null) {
	  String skipStr = training.skip(curr);
	  if (skipStr != null) {
	      System.err.println(": skipping tree No. " +
				 (sentNum + 1) + ": " + skipStr + 
				 " tree="+curr);
	      continue;
	  }
	  System.out.println("----------------------------");
	  System.out.println(curr);
	if (splitPRN) {
	  curr = training.separateFuncTags(curr,true);
	  if (!combineTransformations)
	    System.out.println(curr);
	}

	if (convertConj) {
	  curr = training.convertCONJP(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (stripDollarSigns) {
	  curr = training.stripDollarSigns(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (prune) {
	  curr = training.prune(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	/*
	if (addBaseNPs) {
	  curr =training.addBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (repairBaseNPs) {
	  curr =training.repairBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	*/
	if (removeNullElements) {
	  curr = training.removeNullElements(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (raisePunc) {
	  curr = training.raisePunctuation(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (idArgs) {
	  curr =training.identifyArguments(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (stripAug) {
	  curr = training.stripAugmentations(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (combineTransformations)
	  System.out.println(curr);
      }
      System.out.println("----------------------------");
      sentNum++;
    }
    catch (UnsupportedEncodingException uee) {
      System.err.println(uee);
    }
    catch (FileNotFoundException fnfe) {
      System.err.println(fnfe);
    }
    catch (IOException ioe) {
      System.err.println(ioe);
    }
  }

}
