/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
package danbikel.parser.ppceme;

import java.util.*;
import java.io.*;
import danbikel.lisp.*;
import danbikel.parser.*;


public class Training8 extends Training7 {

    static final String CONJPString="CONJP";
    //  static final Symbol CONJP = Symbol.add("CONJP");
    static final Symbol CONJP = Symbol.add(CONJPString);
    private Nonterminal nonterminal5 = new Nonterminal();

    public Training8() throws FileNotFoundException, IOException {
	super();
	System.err.println("ppceme Training8 constructor");
    }
    public Sexp preProcess(Sexp tree) {
	convertCONJP(tree);
        //      verifyCONJP(tree);
	stripDollarSigns(tree);
	prune(tree);
	//addBaseNPs(tree);
	//repairBaseNPs(tree);
	//    addGapInformation(tree);
	//    relabelSubjectlessSentences(tree);
	removeNullElements(tree);
	raisePunctuation(tree);
	identifyArguments(tree);
	stripAugmentations(tree);
	//fixSubjectlessSentences(tree);  this method now in English lang. package
	return tree;
    }

    public void postProcess(Sexp tree) {
        System.out.println("before fixCONJP = " + tree);
        fixCONJP(tree);
        System.out.println("after fixCONJP = " + tree);
	//unrepairBaseNPs(tree);
	//    super.postProcess(tree);
	//removeOnlyChildBaseNPs(tree);
	//System.out.println("after remove basenps = " + tree);
	//    canonicalizeNonterminals(tree);

	//raiseCodeNodes(tree);
	//System.out.println("after raiseCodeNodes " + tree);
	stripArgAugAndCanonicalizeBases(tree);
	System.out.println("after stripArgAugAndCanonicalizeBases" + tree);
    }

    public boolean isCONJP(Symbol label) {
        return label==CONJP;
    }
    /*
      NP    parentLabel
      /  \
      firstChildLabel          NP   CONJP
      / \
      CONJ  NP grandchild
    */

    /*
      CONJP doesn't have to have only two children.  could also have a ALSO after CONJ.
      So need to look for first non-preterminal.

      for now ignore headless CONJP:
      (NP-PRN
      (NP (NPR Cuthbert) (NPR London))
      (, .)
      (CONJP
      (NP (NPR Richard) (NPR Sampson)))))
      later on might want to try making these into CONJP, which would
      require only looking for
      XP
      / 
      XP  ... CONJP
      and making all such cases into CONJPXP

      but this example shows that CONJP doesn't have to be the second child.

    */
                          
    
    public boolean adjunctStructure(SexpList treeList,Symbol grandparentLabel) {
        if (!treeList.get(1).isList())
            return false;
        Symbol firstChildLabel=treebank.stripAugmentation(treeList.get(1).list().first().symbol());
        return  (grandparentLabel==firstChildLabel);
    }
    public boolean isCONJPXP(Sexp tree, Symbol parentLabel) {
        /* root of tree is CONJP and has at least two children, first child is CONJ */

        //      System.out.println("isCONJPXP grandParentLAbel="+grandparentLabel+" tree="+tree);
        if (!tree.isList()) 
            return false;
        SexpList treeList=tree.list();
        //      System.out.println("is list");
        if (!isCONJP(treebank.stripAugmentation(treeList.symbolAt(0))))
            return false;
        int treeListLen=treeList.length();
        /* only check if has at least two children and first is CONJ */

        if (treeListLen<3) {
            //      System.out.println("is CONJP but treeListLen="+treeListLen+"tree="+tree);
            return false;
        }
        if (!treebank.isConjunction(treeList.get(1).list().first().symbol())) {
            //      System.out.println("is CONJP but left child is not CONJ="+"tree="+tree);
            return false;
        }
	//        return true;
	Sexp grandchildLabel=treebank.stripAugmentation(treeList.get(2).list().first().symbol());
	//      System.out.println("grandChildLabel="+grandchildLabel+" grandParentLabel="+grandparentLabel);
	/*
	if (grandchildLabel!=parentLabel) {
	    System.out.println("warning:is CONJP but grandparentLabel="+parentLabel+
			       "grandchildLabel="+grandchildLabel+"tree="+tree);
	}
	*/
	return true;

    }
        
    public void verifyCONJP(Sexp tree) {
        if (treebank.isPreterminal(tree)) 
            return;
        if (tree.isList()) {
            SexpList treeList = tree.list();
            int treeListLen = treeList.length();
            Symbol parentLabel=treebank.stripAugmentation(treeList.symbolAt(0));
            if (isCONJP(parentLabel)) {
                if (treeListLen<3) {
                    System.out.println("CONJP with <2 children"+tree);
                }
                if (!treebank.isConjunction(treeList.get(1).list().first().symbol())) {
                    System.out.println("CONJP with left child not CONJ "+tree);
                }               
            }
            for (int j=1; j<treeListLen; j++) {
                verifyCONJP(treeList.get(j));
            }
        }
    }
    public Sexp convertCONJP(Sexp tree) {
        if (treebank.isPreterminal(tree)) 
            return tree;
        if (tree.isList()) {
            SexpList treeList = tree.list();
            int treeListLen = treeList.length();
            for (int j=1; j<treeListLen; j++) {
                convertCONJP(treeList.get(j));
            }
            //      System.out.println("convertCONJP "+tree);
            Symbol parentLabel=treebank.stripAugmentation(treeList.symbolAt(0));
            //      System.out.println("grandparentLabel="+grandparentLabel);
            if (adjunctStructure(treeList,parentLabel)) {
                //              System.out.println("is AdjunctStructure");
                for (int j=2; j<treeListLen; j++) {
                    Sexp child = treeList.get(j);
                    //                  System.out.println("child= "+child);
                    if (isCONJPXP(child,parentLabel)) {
                        //                      System.out.println("isCONJPXP");
                        SexpList childList=child.list();
                        Nonterminal parsedChild =
                            treebank.parseNonterminal(childList.symbolAt(0), nonterminal5);
                        parsedChild.base=Symbol.add("CONJP"+parentLabel.toString());
                        childList.set(0,parsedChild.toSymbol());
                    }
                }
            }
        }
	return tree;
    }


    public void fixCONJP(Sexp tree) {
        if (treebank.isPreterminal(tree)) 
            return;
        if (tree.isList()) {
            SexpList treeList = tree.list();
            int treeListLen = treeList.length();
            String parentString=treeList.symbolAt(0).toString();
            if (parentString.startsWith(CONJPString) &&
                parentString.length()>CONJPString.length()) {
                treeList.set(0,CONJP);
            }
            for (int j=1; j<treeListLen; j++) {
                fixCONJP(treeList.get(j));
            }
        }
    }


  /** Test driver for this class. */
  public static void main(String[] args) {
    String filename = null;
    boolean convertConj=false;
    boolean stripDollarSigns = false;
    boolean prune = false;
    boolean addBaseNPs = false;
    boolean repairBaseNPs= false;
    boolean removeNullElements = false;
    boolean raisePunc = false;
    boolean idArgs = false;
    boolean stripAug = false;
    boolean combineTransformations = false;

    for (int i = 0; i < args.length; i++) {
      // for each arg that begins with a '-', examine each of its letters
      // and set booleans accordingly, except for "-combine" flag
      if (args[i].charAt(0) == '-') {
	if (args[i].equals("-combine"))
	  combineTransformations = true;
	else {
	  for (int charIdx = 1; charIdx < args[i].length(); charIdx++) {
	    char curr = args[i].charAt(charIdx);
	    switch (curr) {
	    case 'd':
	      convertConj = true;
	      break;
	    case 't':
	      stripDollarSigns = true;
	      break;
	    case 'p':
	      prune = true;
	      break;
	    case 'b':
	      addBaseNPs = true;
	      break;
	    case 'c':
	      repairBaseNPs = true;
	      break;
	    case 'n':
	      removeNullElements = true;
	      break;
	    case 'r':
	      raisePunc = true;
	      break;
	    case 'i':
	      idArgs = true;
	      break;
	    case 'a':
	      stripAug = true;
	      break;
	    default:
	      System.err.println("illegal flag: " + curr);
	      usage();
	    }
	  }
	}
      }
      else
	filename = args[i];
    }

    if (filename == null) {
      usage();
    }

    Training8 training = (Training8)Language.training();
    //training.printMetadata();

    try {
      SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
					    Constants.defaultFileBufsize);
      Sexp curr = null;
      int sentNum=0;
      while ((curr = Sexp.read(tok)) != null) {
	  String skipStr = training.skip(curr);
	  if (skipStr != null) {
	      System.err.println(": skipping tree No. " +
				 (sentNum + 1) + ": " + skipStr + 
				 " tree="+curr);
	      continue;
	  }
	if (stripDollarSigns) {
	  curr = training.stripDollarSigns(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (convertConj) {
	  curr = training.convertCONJP(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (prune) {
	  curr = training.prune(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (addBaseNPs) {
	  curr =training.addBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (repairBaseNPs) {
	  curr =training.repairBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (removeNullElements) {
	  curr = training.removeNullElements(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (raisePunc) {
	  curr = training.raisePunctuation(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (idArgs) {
	  curr =training.identifyArguments(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (stripAug) {
	  curr = training.stripAugmentations(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (combineTransformations)
	  System.out.println(curr);
      }
      sentNum++;
    }
    catch (UnsupportedEncodingException uee) {
      System.err.println(uee);
    }
    catch (FileNotFoundException fnfe) {
      System.err.println(fnfe);
    }
    catch (IOException ioe) {
      System.err.println(ioe);
    }
  }

}
