   package danbikel.parser.util;

import java.io.*;
import java.util.*;
import danbikel.util.*;
import danbikel.lisp.*;
import danbikel.parser.util.*;



/***
 * replace nulls with flat parse
 */

/*
  withNull file:

  mode=1:
  every line is either:
  1) tree w/o extra parens
     just leave it as is
  2) tree w/o root e.g. ((CODE <text>))
     leave it as is
  3) "null"
     replace with flat parse with dummy root

  note:
  have to check for nulls and not just result of ((word (tag)) (word (tag))
  because otherwise can't tell difference with type 2.
  Could make code work directly with result of distributed parsing
  but also want it to work with non-distributed with nulls in there,
  so assume that result of distributed parsing will have nulls

  w/o null file:
  (could also be parse input file)
  each sentence in the form 
  ((word (tag)) (word (tag)) ...)

  (for later steps when the input is already a tree could make it look for
  the difference automatically)

*/
  
  
  


public class Conversion4{
    private static final int bufSize = 81920;
    // was probably false for constraint stuff

    static boolean getFlatParseForNull=true;
    static String defaultRoot="S";
    static String modeStr="1";

  private Conversion4() {}
    /**
       if the tree in the "with nulls" file is "null", then if 
       this is set true, expect that the corresponding entry in the
       "no nulls" file is of the form ((word (POS)) (word (POS))...)
       that is - the usual input format, and get the flat parse.
       If false, then expect that it's already a tree, and so just
       get that without any change.

    

  /**
   * Reads parse trees either from standard input or a specified file,
   * converting them to sentences and printing those sentences on standard
   * output.
   * <pre>
   * usage: [-v|-help|-usage] [-tags] [filename]
   *         -v|-help|-usage: prints out this message
   *         -tags: indicates to spit out one S-expression per word, of the form
   *                 (word (tag))
   *         filename is the file to be processed (standard input is assumed if
   *                 this argument is "-" or is not present)
   * </pre>
   */

    static String noNullsExt=".parsed";
    static String withNullsExt=".null.parsed";
    static String outFileExt=".parsed.nn";

  protected static SexpList getWords(SexpList sent) {
    int size = sent.size();
    SexpList wordList = new SexpList(size);
    for (int i = 0; i < size; i++)
      wordList.add(sent.get(i).list().get(0));
    return wordList;
  }


  protected static boolean sentContainsWordsAndTags(SexpList sent) {
    int size = sent.size();
    for (int i = 0; i < size; i++) {
	//	System.out.println("checking element " + i + " = " + sent.get(i));
      if (!wordTagList(sent.get(i)))
	return false;
    }
    return true;
  }

  protected static boolean wordTagList(Sexp sexp) {
    if (sexp.isSymbol())
      return false;
    SexpList list = sexp.list();
    // this is a word-tag list is the first element is a symbol (the word)
    // and the second element is a list containing all symbols (the list
    // of possible tags)
    return (list.size() == 2 && list.get(0).isSymbol() &&
	    list.get(1).isList() && list.get(1).list().isAllSymbols());
  }

  protected static SexpList getTagListsFromTree(Sexp tree) {
    Sexp taggedSentence = Util.collectTaggedWords(tree);
    return getTagLists(taggedSentence.list());
  }

  protected static SexpList getTagLists(SexpList sent) {
    int size = sent.size();
    SexpList tagLists = new SexpList(size);
    for (int i = 0; i < size; i++)
      tagLists.add(sent.get(i).list().get(1));
    return tagLists;
  }


    public static void process(InputStream inStreamNoNulls,
			       InputStream inStreamWithNulls,
			       BufferedWriter out) 
	throws IOException {

	Sexp sentNoNulls = null;
	Sexp sentWithNulls=null;
	String enc = System.getProperty("file.encoding");
	SexpTokenizer tokNoNulls = new 
	    SexpTokenizer(inStreamNoNulls, enc, bufSize);
	SexpTokenizer tokWithNulls = new 
	    SexpTokenizer(inStreamWithNulls, enc , bufSize);
	for (int i = 1; ((sentNoNulls = Sexp.read(tokNoNulls)) != null); i++) {
	    //		  System.out.println("no nulls="+sentNoNulls.toString());
	    if ((sentWithNulls=Sexp.read(tokWithNulls))==null) {
		System.err.println("missing sentWithNulls for " + sentNoNulls);
		System.exit(-1);
	    }

	    if (modeStr.equals("1")) {
		if (SKUtil.treeIsNull(sentWithNulls)) {
		  System.out.println("tree is null");
		  System.out.println("with nulls="+sentWithNulls);
		  System.out.println("no nulls="+sentNoNulls.toString());
		  SexpList sent=sentNoNulls.list();
		  String replacementTree= 
		      ( SKUtil.treeHasRoot(sent) ?
			sent.toString() :
			getFlatParse(sent));
		
		/* 
		   this must have been for the constraint parsing stuff
		String flatParse=(treeHasNoRoot(sent) ?
				  getFlatParse(sent) : sent.toString());
		*/

		  out.write(replacementTree+"\n");
		  // hack for Ann Taylor's trees
		  /*
		  if (!flatParse.equals("(S)")) {
				    out.write(flatParse+"\n");
		    }
		  */
				//	out.write(flatParse+"\n");
		} else {
		    //		  System.out.println("no nulls="+sentNoNulls);
		    //		  System.out.println("with nulls="+sentWithNulls);
		    // out.write(sentNoNulls+"\n");
		    out.write(sentWithNulls+"\n");
		}
	    
	    }
	}
	out.close();
	inStreamNoNulls.close();
	inStreamWithNulls.close();
    }

	

    public static void process(List files) 
	throws IOException {
	for (Iterator fileIter = files.iterator(); fileIter.hasNext(); ) {
	    String baseFileName=(String)fileIter.next();
	    //	    String inFileNameWithNulls=baseFileName+".null.parsed";
	    //	    String inFileNameNoNulls=baseFileName+".parsed";
	    //	    String outFileName=baseFileName+".parsed.nn";
	    String inFileNameWithNulls=baseFileName+withNullsExt;
	    String inFileNameNoNulls=baseFileName+noNullsExt;
	    String outFileName=baseFileName+outFileExt;


	    System.out.println("inFileNoNulls="+inFileNameNoNulls);
	    System.out.println("inFileWithNulls="+inFileNameWithNulls);
	    System.out.println("outFile="+outFileName);
	    InputStream inStreamNoNulls=null;
	    InputStream inStreamWithNulls=null;
	    BufferedWriter out=null;

	    inStreamNoNulls = new FileInputStream(inFileNameNoNulls);
	    inStreamWithNulls = new FileInputStream(inFileNameWithNulls);
	    out = new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
			FileOutputStream(outFileName),
				       "ISO-8859-1"));
	    process(inStreamNoNulls,
		    inStreamWithNulls,
		    out);
	}
    }

    protected static String getFlatParse(SexpList sent) {
	System.out.println("getting flat parse for "+sent);
	String flatParse="("+defaultRoot;
	System.out.println("starting with " + flatParse);
	if (sentContainsWordsAndTags(sent)) {
	    SexpList words=getWords(sent);
	    SexpList tags=getTagLists(sent);
	    if (words.size()!=tags.size()) {
		System.out.println("unequal size");
		System.exit(-1);
	    }
	    //	    System.out.println("wordsSize="+words.size());
	    for (int i=0; i<words.size(); i++) {
		SexpList oneTagSet=tags.listAt(i);
		Symbol oneTag = oneTagSet.symbolAt(0);
		flatParse+= " ("+ oneTag + " " + words.symbolAt(i)+")";
	    }
	    flatParse+=")";
	    return flatParse;
	} else {
	    System.out.println("sent fails sentContainsWordsAndTags");
	    System.out.println("not in right format");
	    System.exit(-1);
	}
    return null;

    }


    public static void main(String[] args) {
	modeStr=args[0];
	defaultRoot=args[1];
	ArrayList dir=new ArrayList();
	dir.add(new File(args[2]));
	noNullsExt=args[3];
	withNullsExt=args[4];
	outFileExt=args[5];

	List files=SKUtil.expandDirectories(dir,withNullsExt,true,false);
	try {
	    process(files);
	}
	catch (Exception e) {
	    System.err.println(e);
	    System.exit(-1);
	}
    }
}
    
