package danbikel.parser.util;

import java.io.*;
import java.util.*;
import danbikel.util.*;
import danbikel.lisp.*;
import danbikel.parser.*;
import danbikel.parser.util.*;

public class StripArgAugCanonicalizeRemoveNulls{
    private static final int bufSize = 81920;
    
    // data members used by raisePunctuation
    private SexpList addToRaise = new SexpList();
    private SexpList raise = new SexpList();

    protected Set nodesToRaise = new HashSet();

    protected static Symbol defaultArgAugmentation=Language.training().defaultArgAugmentation();
    protected Treebank treebank = Language.treebank();

    private Nonterminal nonterminal3 = new Nonterminal();

    boolean removeNulls;
    String newExt;
    
    public StripArgAugCanonicalizeRemoveNulls(boolean removeNulls,String newExt) {
	this.removeNulls=removeNulls;
	this.newExt=newExt;
    }
	
    public Sexp stripIndices(Sexp tree) {
	if (Language.treebank().isPreterminal(tree))
	    return tree;
	if (tree.isList()) {
	    Symbol label = tree.list().first().symbol();
	    tree.list().set(0, Language.treebank().stripIndex(label, nonterminal3));
	    int treeListLen = tree.list().length();
	    for (int i = 1; i < treeListLen; i++) 
		stripIndices(tree.list().get(i));
	}
	return tree;
    }


    public Sexp removeNullElements(Sexp tree) {
	if (treebank.isPreterminal(tree))
	    return tree;
	if (tree.isList()) {
	    SexpList treeList = tree.list();
	    for (int i = 1; i < treeList.length(); i++) {
		if (treebank.isNullElementPreterminal(treeList.get(i))) {
		    treeList.remove(i--); // postdecrement i to offset for loop increment
		}
		else {
		    removeNullElements(treeList.get(i));
		    // if removing null preterminals from the current child yields
		    // a childless child, then remove the current child
		    if (treeList.get(i).isList() &&
			treeList.get(i).list().length() == 1)
			treeList.remove(i--);
		}
	    }
	}
	return tree;
    }


    
    protected void stripArgAugAndCanonicalizeBases(Sexp tree) {
	if (Language.treebank().isPreterminal(tree)) {
	    /*
	      SexpList treeList = tree.list();
	      Symbol currLabel = treeList.symbolAt(0);
	      treeList.set(0, treebank.getCanonical(currLabel));
	    */
	    return;
	}
	else if (tree.isList()) {
	    SexpList treeList = tree.list();
	    Symbol currLabel = treeList.symbolAt(0);
	    treeList.set(0, stripArgAugAndCanonicalizeBase(currLabel));
	    int treeListLen = treeList.length();
	    for (int i = 1; i < treeListLen; i++)
		stripArgAugAndCanonicalizeBases(treeList.get(i));
	}
    }
    public Symbol stripArgAugAndCanonicalizeBase(Symbol label) {
	//    Symbol canonicalArgLabel = (Symbol)canonicalArgCache.get(label);
	//    if (canonicalArgLabel == null) {
	Symbol newLabel=removeDefaultArgAugmentation(label,new Nonterminal());
	Nonterminal parsedLabel =
	    Language.treebank().parseNonterminal(newLabel);
	parsedLabel.base = Language.treebank().getCanonical(parsedLabel.base);
	return parsedLabel.toSymbol();
    }
    
    protected Symbol removeDefaultArgAugmentation(Symbol label,
						  Nonterminal nonterminal) {
	Nonterminal parsedLabel = Language.treebank().parseNonterminal(label, nonterminal);
	boolean removedSomething = false;
	SexpList augmentations = parsedLabel.augmentations;
	for (int i = 0; i < augmentations.length(); i++) {
	    if (defaultArgAugmentation==augmentations.get(i)) {
		i--; // move index to delimeter that preceded this arg augmentation
		augmentations.remove(i);  // remove delimeter
		augmentations.remove(i);  // remove arg augmentation
		removedSomething = true;
		i--; // decrement index again to offset increment in for loop
	    }
	}
	return removedSomething ? parsedLabel.toSymbol() : label;
    }
    
    public void process(InputStream inStream,
			BufferedWriter out) 
	throws IOException {
	boolean stripOuterParens;
	//	System.out.println("in process for inStream="+inStream);
	Sexp tree = null;
	String enc = System.getProperty("file.encoding");
	SexpTokenizer tok = new 
	    SexpTokenizer(inStream, enc, bufSize);
	for (int i = 1; ((tree = Sexp.read(tok)) != null); i++) {
	    //	    System.out.println("tree="+tree);
	    stripOuterParens = (tree.list().length() == 1 &&
				tree.list().get(0).isList());	
	    if (stripOuterParens)
		tree = tree.list().get(0);	    
	    //	    System.out.println("tree="+tree);
	    stripArgAugAndCanonicalizeBases(tree);
	    if (removeNulls) {
		removeNullElements(tree);
		stripIndices(tree);
	    }
	    out.write(tree+"\n");
	}

	out.close();
	inStream.close();
    }


    public void process(List files) 
	throws IOException {
	for (Iterator fileIter = files.iterator(); fileIter.hasNext(); ) {
	    String inFileName=(String)fileIter.next();
	    //	    String inFileName=baseFileName+".parsed.fixed.raised.with_ecs";
	    //	    String inFileName=baseFileName+".parsed.fixed.raised.stripped";
	    //	    String inFileName=baseFileName+".parsed.fixed.raised.wrapped.with_ecs";
	    //	    String inFileName=baseFileName+".parsed.fixed.raised.wrapped.with_ecs";
	    //	    String outFileName=baseFileName+".parsed.fixed.raised.wrapped.with_ecs.stripped";
	    String outFileName=inFileName+newExt;
	    System.out.println("inFileName="+inFileName);
	    System.out.println("outFileName="+outFileName);
	    InputStream inStream=null;
	    inStream = new FileInputStream(inFileName);
	    BufferedWriter out=null;

	    out = new 
		BufferedWriter(new 
			       OutputStreamWriter(new 
						  FileOutputStream(outFileName),
						  "ISO-8859-1"));
	    process(inStream,
		    out);
	}
    }


    public static void main(String[] args) {


	String inDir=args[0];
	String inExt=args[1];
	boolean removeNulls=Boolean.valueOf(args[2]).booleanValue();
	String newExt=args[3];
	System.out.println("inDir="+inDir);
	System.out.println("inExt="+inExt);
	System.out.println("removeNulls="+removeNulls);
	System.out.println("newExt"+newExt);
	ArrayList dirs=new ArrayList();
	dirs.add(new File(inDir));
	// 	List files=expandDirectories(dir,".parsed.fixed.raised",true,false);
	// List files=SKUtil.expandDirectories(dir,".parsed.fixed.raised.wrapped.with_ecs",true,false);
	List files=SKUtil.expandDirectories(dirs,inExt,false,false);
	System.out.println("# files="+files.size());
	StripArgAugCanonicalizeRemoveNulls strip=new StripArgAugCanonicalizeRemoveNulls(removeNulls,newExt);
	try {
	    strip.process(files);
	}
	catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	    System.exit(-1);
	}
    }
}
