/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
    package danbikel.parser.util;

import danbikel.lisp.*;
import danbikel.parser.*;
import java.util.*;

/**
 * Contains basic utility functions for <code>Sexp</code> objects that
 * represent parse trees.
 * @author Dan Bikel
 */
public class Util2 {

  private Util2() {}

  /**
   * Returns a string containing the pretty-printed version of the specified
   * parse tree.
   *
   * @param tree the tree to pretty-print
   * @return a string containing the pretty-printed version of the specified
   * parse tree.
   */
  public static String prettyPrint(Sexp tree) {
    StringBuffer sb = new StringBuffer();
    prettyPrint(tree, sb, 0,false);
    return sb.toString();
  }
  private static void prettyPrint(Sexp tree, StringBuffer sb, int level, boolean donewline) {
      if (donewline) {
	  sb.append("\n");
	  for (int i = 0; i < level; i++)
	      sb.append(" ");
      }
      if (Language.treebank().isPreterminal(tree)) {
      sb.append(tree.toString());
    }
    /* put first child on same line, and then each of the rest on a new line */
    else if (tree.isList()) {
      SexpList treeList = tree.list();
      int treeListLen = treeList.length();
      // SK added - if not a tree, just print it as is
      if (!(treeList.get(0).isSymbol())) {
	  sb.append(tree.toString());
	  return;
      }
      sb.append("(").append(treeList.symbolAt(0));
      sb.append(" ");
      level+=treeList.symbolAt(0).toString().length()+2;
      for (int i = 1; i < treeListLen; i++) {
	Sexp child = treeList.get(i);
	if (i==1) {
	  prettyPrint(child, sb, level,false);
	} else {
	    prettyPrint(child, sb, level,true);
	}
      }
      sb.append(")");
    }
    else
      sb.append(tree);
  }
}
