/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
    package danbikel.parser;

import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

public class SKAnalyzeDisns extends AnalyzeDisns{

    private SKAnalyzeDisns() {}

  public static double[] getLogProbDisn(Model model, int level, Event hist,
					Set futures, double[] disn,
					Event[] futureArray,
					Transition tmpTrans) {
    double logProbs[] = disn;
    Transition trans = tmpTrans;
    trans.setHistory(hist);
    Iterator it = futures.iterator();
    for (int idx = 0; it.hasNext(); idx++) {
      trans.setFuture((Event)it.next());
      futureArray[idx]=trans.future();
      logProbs[idx] = model.estimateLogProbUsingPrecomputed(trans, level);
    }
    return logProbs;
  }


  public static void writeModelStatsSK(Model model) throws IOException {
    ProbabilityStructure structure = model.getProbStructure();
    String structureClassName = structure.getClass().getName();
    String filename = structureClassName + ".disns";
    System.err.println("Writing distribution stats to \"" + filename + "\".");
    // create file for this model
    OutputStream os = new FileOutputStream(filename);
    OutputStreamWriter osw = new OutputStreamWriter(os, Language.encoding());
    PrintWriter writer =  new PrintWriter(new BufferedWriter(osw));

    // set up temporary data structures for getFutures and getLogProbDisn
    // methods
    int disnLen = 1000;
    double[] disn = new double[disnLen];
    Event[] futureArray=new Event[disnLen];
    /*
    int disnLen =0;
    double[] disn = null;
    Event[] futureArray=null;
    */
    Transition tmpTrans = new Transition(null,null);
    Set futuresSet = new LinkedHashSet();

    // foreach back-off level
    //   foreach history at level
    //     print level hist count diversity numNonZeroSmoothedProbs entropy

    // for each level
    //  disnLen=# of futures at this level (should be same for all levels, though)
    //  entrySet is set of all histories at this level
    //  for each history at level
    //     get count, diversity of the history
    //     get array of all logProb of all futures of this history
    //     fills up disn with vals and futureArray with corresponding futures
    //     logProbDisn seems to be the same thing as disn (!?)
    //    The length of logProbDisn is a minimum of 1000, although its
    //    values may only be filled up with disnLen size elements, one for
    //    each future

    // The problem with the numNonZeroProbs is that it goes from logProbDisn.length
    // down to 0, instea do from disnLen to 0.
    
    int numLevels = structure.numLevels();
    for (int level = 0; level < numLevels; level++) {
	Set futures = getFutures(futuresSet, model, level);
	disnLen = futures.size();
	/*
	disn = new double[disnLen];
	futureArray=new Event[disnLen];
	*/
	if (disnLen > disn.length) {
	    disn = new double[disnLen];
	    futureArray=new Event[disnLen];
	}
	Event hist = null;
	Set entrySet = model.counts[level].history().entrySet();
	Iterator it = entrySet.iterator();
	int totalNumDistributions=0;
	double totalEntry=0.0;
	
	while (it.hasNext()) {
	    MapToPrimitive.Entry entry = (MapToPrimitive.Entry)it.next();
	    hist = (Event)entry.getKey();
	    double count  = entry.getDoubleValue(CountsTrio.hist);
	    double diversity = entry.getDoubleValue(CountsTrio.diversity);
	    double[] logProbDisn = getLogProbDisn(model, level, hist,
						  futures, disn, futureArray, tmpTrans);
	    int numNonZeroProbs = 0;
	    
	    
	    //	    for (int i = logProbDisn.length - 1; i >= 0; i--)
	    for (int i = disnLen - 1; i >= 0; i--)
		if (logProbDisn[i] != Constants.logOfZero) 
		    numNonZeroProbs++;
	    writer.println(level + "\t" + hist + "\t" + count + "\t" +
			   diversity + "\t" + numNonZeroProbs + "\t" +
			   entropyFromLogProbs(logProbDisn, disnLen));
	    totalNumDistributions++;
	    totalEntry+=entropyFromLogProbs(logProbDisn, disnLen);
	    //	    writer.println("  totals " + totalNumDistributions + " so far " + totalEntry);
	    
	    //      for (int i = logProbDisn.length - 1; i >= 0; i--)
	    /*
	    for (int i = disnLen - 1; i >= 0; i--)
		if (logProbDisn[i] != Constants.logOfZero)
		    writer.println("   "+i+") "+ futureArray[i]+"  " + logProbDisn[i]);
	    */
	      }
      writer.println("avg entropy level " + level + " = " + totalEntry+"/"+totalNumDistributions+"  = " + 
			 (totalEntry/totalNumDistributions));
	
    }
    writer.flush();
    writer.close();
  }

  

  /**
   * Analyzes and saves information about every distribution in every
   * {@link Model} contained in a {@link ModelCollection}.  It is important that
   * <ul>
   * <li>{@link Model#precomputeProbs} was <tt>true</tt>,
   * <li>{@link Model#deleteCountsWhenPrecomputingProbs} was <tt>false</tt> and
   * <li>{@link Model#createHistBackOffMap} was <tt>true</tt>
   * </ul>
   * when the {@link ModelCollection} to be analyzed was created.
   * <p>
   * usage: <tt>&lt;derived data file&gt;</tt><br>
   * where <tt>&lt;derived data file&gt;</tt> was produced by {@link Trainer}.
   *
   * @param args an array containing at least one element that is the name
   * of a model collection (derived data file) as produced by a {@link Trainer}
   * instance
   *
   * @see Trainer
   * @see Trainer#writeModelCollection(ObjectOutputStream,String,String)
   * @see Trainer#loadModelCollection(String)
   */
  public static void main(String[] args) {
    String mcFilename = args[0];
    // collect any additional args into a set
    Set structureNames = new LinkedHashSet();
    for (int argIdx = 1; argIdx < args.length; argIdx++)
      structureNames.add(args[argIdx]);
    try {
      ModelCollection mc = Trainer.loadModelCollection(mcFilename);
      Model mwm = mc.modWordModel();

      // output all histories
      //outputHistories(mwm);
      /*
      String modEventStr = args.length > 1 ? args[1] :
	"((foo VB) (to TO) VP-A (+START+) " +
	"((+START+ +START+)) VP TO (VP-A) false right)";

      analyzeModWordDisn(mc, modEventStr);
      */

      // for each Model in ModelCollection (using ModelCollection.modelList)
      //   for each Model within each Model (using Model.getModel)
      //     writeModelStats(Model)
      Iterator models = mc.modelList().iterator();
      while (models.hasNext()) {
	Model model = (Model)models.next();
	int numModels = model.numModels();
	for (int i = 0; i < numModels; i++) {
	  Model ithModel = model.getModel(i);
	  String structureClassName =
	    ithModel.getProbStructure().getClass().getName();
	  if (structureClassName.equals("danbikel.parser.ms.HeadModelStructure1"))
	      writeModelStatsSK(ithModel);
	  if (structureNames.contains(structureClassName)) 
	    writeKLDistStats(ithModel);
	}
      }
    }
    catch (Exception e) {
      e.printStackTrace(System.err);
    }
  }
}
