    package  danbikel.parser;

import java.util.HashMap;
import danbikel.util.*;
import danbikel.lisp.*;
import danbikel.parser.constraints.*;
import danbikel.parser.util.Util;

import java.io.*;
import java.util.*;
import java.rmi.*;

public class DecoderHistorical extends Decoder implements Serializable {





  public DecoderHistorical(int id, DecoderServerRemote server) {
      super(id,server);
  }


    /* Want to modify Decoder.preProcess such that
       1. Dollars signs are stripped from words
          done here.  Can't do it inside preProcessTest
	  because want to do it before check for unknown words
       2. Words are pruned based on pos tag
          done by overriding definition of removeWord
       3. initial and final punctuation words are removed based on 
          pos tag, not word
		

      Note: I could modify this to not override preProcess, and 
      instead just overrid isPuncRaiseWord to always return false,
      and put the the real peripheral pruning inside preProcessTest
      

    */

  protected void preProcess(SexpList sentence, SexpList tags)
  throws RemoteException {

      System.out.println("entering DecoderHistorical preProcess " + sentence);
    // preserve original sentence
    originalSentence.clear();
    originalSentence.addAll(sentence);

    // preserve original tags, if non-null
    if (tags != null)
      originalTags = new SexpList(tags);
    else
      originalTags = null;

    originalWords.clear();
    originalWords.addAll(sentence);

    for (int i = 0; i < sentence.length(); i++) {
	sentence.set(i, ((Training2)Language.training()).stripDollarSigns(sentence.symbolAt(i)));
    }

    // eliminate pruned words
    for (int i = sentence.length() - 1; i >= 0; i--) {
      Symbol word = (downcaseWords ?
		     Symbol.get(sentence.get(i).toString().toLowerCase()) :
		     sentence.symbolAt(i));
      Symbol tag = tags == null ? null : tags.listAt(i).first().symbol();
      if (Language.training().removeWord(word, tag, i, sentence,
					 tags, originalTags,
					 prunedPretermsPosSet,
					 prunedPretermsPosMap))
	removeWord(sentence, tags, i);
    }

    SexpList convertedSentence = server.convertUnknownWords(sentence);
    // we cannot just say
    //   sentence = server.convertUnknownWords(sentence);
    // because the server might return a copy of the sentence list (this is
    // guaranteed to happen if the server is remote), and we need
    // to modify the object that was passed as an arg to this function, because
    // that's what the caller of preProcess expects; the alternative would be
    // for this method to work with the (potentially) different list object
    // returned by the server and then return that list object, but then this
    // method should probably also return the tags list as well, making things
    // difficult, since we only get to return a single object
    if (convertedSentence != sentence) {
      sentence.clear();
      sentence.addAll(convertedSentence);
    }


    // downcase words
    int sentLen = sentence.length();
    if (downcaseWords) {
      for (int i = 0; i < sentLen; i++) {
	if (sentence.get(i).isList()) // skip unknown words
	  continue;
	Symbol downcasedWord =
	  Symbol.add(sentence.symbolAt(i).toString().toLowerCase());
	sentence.set(i, downcasedWord);
      }
    }


    // remove intitial and final punctuation "words"
    for (int i = 0; i < sentence.length() - 1; i++) {
      if (sentence.get(i).isList())
	break;
      if (isPuncRaiseWordTag(sentence.get(i),tags,i)) {
	removeWord(sentence, tags, i);
	i--;
      }
      else
	break;
    }
    for (int i = sentence.length() - 1; i > 0; i--) {
      if (sentence.get(i).isList())
	break;
      if (isPuncRaiseWordTag(sentence.get(i),tags,i)) {
	removeWord(sentence, tags, i);
      }
      else
	break;
    }

    // finally, perform any language-specific pre-processing of the words
    // and tags
    SexpList langSpecific =
      Language.training.preProcessTest(sentence, originalWords, tags);
    sentence = langSpecific.listAt(0);
    if (tags != null)
      tags = langSpecific.listAt(1);

   System.out.println("leaving DecoderHistorical preProcess " + sentence);
  }


  protected boolean isPuncRaiseWordTag(Sexp word,SexpList tags, int wordIdx) {
      SexpList tagSet=tags.listAt(wordIdx);
      Symbol tag = tagSet.symbolAt(0);
      return Language.treebank.isPunctuation(tag);
  }


    /*
      SK: only have this to not return null if sentence entirely
      pruned, assuming it was CODE thing.
      How does it go from 
      input ((<text> (CODE)))      
      to 
      output ((CODE <text>))
    */



}
