package danbikel.parser;

import danbikel.lisp.*;
import danbikel.util.*;
import danbikel.util.AbstractMapToPrimitive.*;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.util.HashMap;
import java.text.DecimalFormat;


public class PostProcess {

    BufferedWriter outResults=null;



    public PostProcess() {
	
    }
    
    private static boolean treeIsNull(Sexp tree) {
	if (tree.isSymbol() &&
	    tree.symbol().toString().equals("null"))
	    return true;
	else 
	    return false;
    }
    
    public void process(SexpTokenizer parsedTok) 
	throws IOException {
	int sentNum = 0, intervalCounter = 0;
	int numSents = 0;
	int reportingInterval=100;
	
	Sexp tree=null;
	
	for ( ; (tree = Sexp.read(parsedTok)) != null; sentNum++, intervalCounter++) {
	    
	    if (intervalCounter == reportingInterval) {
		System.err.println("postprocessed " + sentNum + " sentence" +
				   (sentNum > 1 ? "s" : ""));
		intervalCounter = 0;
	    }
	    
	    if (treeIsNull(tree)) {
		resultsWriteln("null");
	    } else {
		Language.training.postProcess(tree);
		resultsWriteln(tree.toString());
	    }
	}
    }


    void resultsWriteln(String s) {
	try {
	    outResults.write(s+"\n");
	}
	catch (IOException e) {
	    System.err.println("io exception"+ e);
	    e.printStackTrace();
	    System.exit(-1);
	}
    }

    public void process(String parsedFilename,String newFilename) {
	System.out.println("parsedFilename="+parsedFilename);
	System.out.println("newFilename="+newFilename);

	String encoding = Language.encoding();
	int defaultFileBufsize=81920;

	try {
	    outResults = new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
				       FileOutputStream(newFilename)));

	    process(new SexpTokenizer(parsedFilename, encoding, defaultFileBufsize));
	    outResults.close();
	}
	catch (UnsupportedEncodingException e) {
	    System.err.println(e);
	}
	catch (FileNotFoundException e) {
	    System.err.println(e);
	}
	catch (IOException e) {
	    System.err.println(e);
	}
    }




  public static void main(String[] args) {
	
	String parsedFilename=args[0];
	String newFilename=args[1];
	
	PostProcess postProcess=new PostProcess();
	postProcess.process(parsedFilename,newFilename);
  }
}
