/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
    package danbikel.parser.arabic;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

/**
 * Provides methods for language-specific processing of training parse trees.
 * Even though this subclass of {@link danbikel.parser.Training} is
 * in the default English language package, its primary purpose is simply
 * to fill in the {@link #argContexts}, {@link #semTagArgStopSet} and
 * {@link #nodesToPrune} data members using a metadata resource.  If this
 * capability is desired in another language package, this class may be
 * subclassed.
 * <p>
 * This class also re-defined the method
 * {@link danbikel.parser.lang.AbstractTraining#hasPossessiveChild(Sexp)}.
 */
public class Training2 extends danbikel.parser.arabic.Training1 {

  private Nonterminal nonterminal1b = new Nonterminal();
  private Nonterminal nonterminal2b = new Nonterminal();

    /* must be a better way to do this */
  private final static Symbol tagMapSym = Symbol.add("tag-map");

  public Training2() throws FileNotFoundException, IOException {
      super();
      System.err.println("in arabic Training 2");
  }

  protected void readMetadataHook(Symbol dataType,
				  int metadataLen, SexpList metadata) {
    if (dataType == tagMapSym) {
      for (int i = 1; i < metadataLen; i++) {
	SexpList mapping = metadata.get(i).list();
	TagMap2.add(mapping.symbolAt(0), mapping.symbolAt(1));
      }
    }
  }

  public SexpList preProcessTest(SexpList sentence,
				 SexpList originalWords, SexpList tags) {
    if (tags == null)
      return super.preProcessTest(sentence, originalWords, tags);
    SexpList processed = new SexpList(2);
    processed.add(sentence);
    int numWords = sentence.size();
    for (int i = 0; i < numWords; i++) {
      Symbol origWord = originalWords.symbolAt(i);

      SexpList currWordTags = tags.listAt(i);
      int numTags = currWordTags.size();

      for (int tagIdx = 0; tagIdx < numTags; tagIdx++) {

	Symbol origTag = currWordTags.symbolAt(tagIdx);
	Word word = danbikel.parser.Words.get(origWord, origTag);

	currWordTags.set(tagIdx, TagMap2.transformTag(word));
      }

      tags.set(i, currWordTags);
    }
    processed.add(tags);
    System.out.println("preprocesstest Training2 after tags = " + tags);
    return processed;
  }

  protected Sexp transformTags(Sexp tree) {
    if (Language.treebank().isPreterminal(tree) &&
	!Language.treebank().isNullElementPreterminal(tree)) {
      Word word = Language.treebank().makeWord(tree);
      Symbol newTag = TagMap2.transformTag(word);
      tree.list().set(0, newTag);
    }
    else if (tree.isList()) {
      SexpList treeList = tree.list();
      int treeListLen = treeList.length();
      for (int i = 1; i < treeListLen; i++)
	transformTags(treeList.get(i));
    }
    return tree;
  }


  protected Sexp keepTransformedTags(Sexp tree) {
    if (Language.treebank().isPreterminal(tree) &&
	!Language.treebank().isNullElementPreterminal(tree)) {
      Word word = Language.treebank().makeWord(tree);
      Symbol oldTag= word.tag();
      Symbol newTag = TagMap2.transformTag(word);
      //      tree.list().set(0, newTag);
      String combinedTag=oldTag.toString()+"["+newTag.toString()+"]";
      tree.list().set(0, Symbol.add(combinedTag));

    }
    else if (tree.isList()) {
      SexpList treeList = tree.list();
      int treeListLen = treeList.length();
      for (int i = 1; i < treeListLen; i++)
	keepTransformedTags(treeList.get(i));
    }
    return tree;
  }


    /* only strip default arg, not the new args like SBJ
     */
  protected void stripArgAugAndCanonicalizeBases(Sexp tree) {
      if (treebank.isPreterminal(tree)) {
	  /*
	    SexpList treeList = tree.list();
	    Symbol currLabel = treeList.symbolAt(0);
	    treeList.set(0, treebank.getCanonical(currLabel));
	  */
	  return;
      }
      else if (tree.isList()) {
	  SexpList treeList = tree.list();
	  Symbol currLabel = treeList.symbolAt(0);
	  treeList.set(0, stripArgAugAndCanonicalizeBase(currLabel));
	  int treeListLen = treeList.length();
	  for (int i = 1; i < treeListLen; i++)
	      stripArgAugAndCanonicalizeBases(treeList.get(i));
      }
  }
    public Symbol stripArgAugAndCanonicalizeBase(Symbol label) {
	//    Symbol canonicalArgLabel = (Symbol)canonicalArgCache.get(label);
	//    if (canonicalArgLabel == null) {
	Symbol newLabel=removeDefaultArgAugmentation(label,new Nonterminal());
	Nonterminal parsedLabel =
	    treebank.parseNonterminal(newLabel);
	parsedLabel.base = treebank.getCanonical(parsedLabel.base);
	return parsedLabel.toSymbol();
    }
    //      if (argAugmentations.contains(augmentations.get(i))) {
  protected Symbol removeDefaultArgAugmentation(Symbol label,
                                         Nonterminal nonterminal) {
    Nonterminal parsedLabel = treebank.parseNonterminal(label, nonterminal);
    boolean removedSomething = false;
    SexpList augmentations = parsedLabel.augmentations;
    for (int i = 0; i < augmentations.length(); i++) {
	if (defaultArgAugmentation==augmentations.get(i)) {
	    i--; // move index to delimeter that preceded this arg augmentation
	    augmentations.remove(i);  // remove delimeter
	    augmentations.remove(i);  // remove arg augmentation
	    removedSomething = true;
	    i--; // decrement index again to offset increment in for loop
	}
    }
    return removedSomething ? parsedLabel.toSymbol() : label;
  }

  public void postProcess(Sexp tree) {
    //unrepairBaseNPs(tree);
      //    super.postProcess(tree);
      System.out.println("before remove basenps = " + tree);
    removeOnlyChildBaseNPs(tree);
       System.out.println("after remove basenps = " + tree);
    //    canonicalizeNonterminals(tree);
    stripArgAugAndCanonicalizeBases(tree);
  }

    // added on 4/18

  public Sexp identifyArguments(Sexp tree) {
    if (treebank.isPreterminal(tree))
      return tree;
    if (tree.isList()) {
      SexpList treeList = tree.list();
      int treeListLen = treeList.length();

      // first, make recursive call if not already at max recursion level
      for (int childIdx = 1; childIdx < treeListLen; childIdx++)
	identifyArguments(treeList.get(childIdx));

      // collect *all* candidate pattern lists in argContexts that apply to the
      // current parent: run through each key in argContexts map, and see if it
      // subsumes current parent; if so, add to allCandidatePatterns, which is
      // a list of lists
      Symbol parent = tree.list().first().symbol();
      Nonterminal parsedParent =
	treebank.parseNonterminal(parent, nonterminal1b);
      SexpList allCandidatePatterns = new SexpList();
      Iterator entries = argContexts.entrySet().iterator();
      while (entries.hasNext()) {
	Map.Entry entry = (Map.Entry)entries.next();
	Symbol key = (Symbol)entry.getKey();
	SexpList currPatternList = (SexpList)entry.getValue();
	Nonterminal parsedKey = treebank.parseNonterminal(key, nonterminal2b);
	if (parsedKey.subsumes(parsedParent)) {
	  allCandidatePatterns.add(currPatternList);
	}
      }

      int headIdx = headFinder.findHead(treeList);

      for (int patternIdx = 0; patternIdx < allCandidatePatterns.length();
	   patternIdx++) {
	SexpList candidatePatterns = allCandidatePatterns.listAt(patternIdx);

	Symbol headChild = (headIdx == 0 ? null :
			    treeList.get(headIdx).list().first().symbol());

	if (isCoordinatedPhrase(tree, headIdx))
	  return tree;

	// either the candidate pattern list has the form (head <int>) or
	// (head-left <list>) or (head-right <list>) or it's a list of actual
	// nonterminal labels
	if (candidatePatterns.first().symbol() == headSym ||
	    candidatePatterns.first().symbol() == headPreSym ||
	    candidatePatterns.first().symbol() == headPostSym) {
	  int argIdx = -1;
	  Symbol child = null;
	  boolean foundArg = false;

	  if (candidatePatterns.first().symbol() == headSym) {
	    String offsetStr = candidatePatterns.get(1).toString();
	    int headOffset = Integer.parseInt(offsetStr);
	    // IF there is a head and IF that head is not equal to parent (i.e.,
	    // if it's not a situation like (PP (PP ...) (CC and) (PP ...)) ) and
	    // if the headIdx plus the headOffset is still valid, then we've got
	    // an argument
	    argIdx = headIdx + headOffset;
	    if (headIdx > 0 &&
		treebank.getCanonical(headChild) != parent &&
		argIdx > 0 && argIdx < treeListLen) {
	      child = treeList.getChildLabel(argIdx);

	      // if the arg is actually a conjunction or punctuation,
	      // OR ANY KIND OF PRETERMINAL,
	      // if possible, find the first child to the left or right of
	      // argIdx that is not a preterminal
	      if (treebank.isPreterminal(child) ||
		  treebank.isConjunction(child) || treebank.isPunctuation(child)) {
		if (headOffset > 0) {
		  for (int i = argIdx + 1; i < treeListLen; i++) {
		    if (!treebank.isPreterminal(treeList.get(i))) {
		      argIdx = i;
		      child = treeList.getChildLabel(argIdx);
		      break;
		    }
		  }
		}
		else {
		  for (int i = argIdx - 1; i >= 0; i--) {
		    if (!treebank.isPreterminal(treeList.get(i))) {
		      argIdx = i;
		      child = treeList.getChildLabel(argIdx);
		      break;
		    }
		  }
		}
	      }
	      foundArg = !treebank.isPreterminal(treeList.get(argIdx));
	    }
	  }
	  else {
	    SexpList searchInstruction = candidatePatterns;
	    int searchInstructionLen = searchInstruction.length();
	    boolean leftSide =
	      searchInstruction.symbolAt(0) == headPreSym;
	    boolean leftToRight =
	      searchInstruction.symbolAt(1) == Constants.firstSym;
	    boolean negativeSearch =
	      searchInstruction.symbolAt(2) == Constants.notSym;
	    int searchSetStartIdx = negativeSearch ? 3 : 2;
	    if (headIdx > 0 && treebank.getCanonical(headChild) != parent) {
	    //if (headIdx > 0) {
	      int increment = leftToRight ? 1 : -1;
	      int startIdx = -1, endIdx = -1;
	      if (leftSide) {
		startIdx = leftToRight ?  1            :  headIdx - 1;
		endIdx   = leftToRight ?  headIdx - 1  :  1;
	      }
	      else {
		startIdx = leftToRight ? headIdx + 1      :  treeListLen - 1;
		endIdx   = leftToRight ? treeListLen - 1  :  headIdx + 1;
	      }
	      // start looking one after (or before) the head index for
	      // first occurrence of a symbol in the search set, which is
	      // comprised of the symbols at indices 2..searchInstructionLen
	      // in the list searchInstruction
	      SEARCH:
	      for (int i = startIdx;
		   leftToRight ? i <= endIdx : i >= endIdx; i += increment) {
		Symbol currChild = treeList.getChildLabel(i);
		Symbol noAugChild = treebank.stripAugmentation(currChild);
		for (int j = searchSetStartIdx; j < searchInstructionLen; j++) {
		  Symbol searchSym = searchInstruction.symbolAt(j);
		  //if (noAugChild == searchSym) {
		  if (currChild == searchSym) {
		    if (negativeSearch)
		      continue SEARCH;
		    else {
		      argIdx = i;
		      child = currChild;
		      foundArg = true;
		      break SEARCH;
		    }
		  }
		}
		// if no match for any nt in search set and this is
		// negative search, we've found what we're looking for
		if (negativeSearch) {
		  argIdx = i;
		  child = currChild;
		  foundArg = true;
		  break SEARCH;
		}
	      }
	    }
	  }

	  if (foundArg) {
	    Nonterminal parsedChild =
	      treebank.parseNonterminal(child, nonterminal1b);
            addArgAugmentation(child, parsedChild);
	    treeList.setChildLabel(argIdx, parsedChild.toSymbol());
	  }
	}
	else {
	  // the candidate pattern list is a list of actual nonterminal labels

	  // the following line means we only find arguments in situations
	  // where the head child's label is different from the parent label
	    //	  if (treebank.getCanonical(headChild) !=
	    //	      treebank.getCanonical(parent)) {
	  if (true) {
	    relabelArgChildren(treeList, headIdx, candidatePatterns);
	  }
	}
      }
    }
    return tree;
  }



}
