/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
package danbikel.parser.arabic;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

/**
 * Provides methods for language-specific processing of training parse trees.
 * Even though this subclass of {@link danbikel.parser.Training} is
 * in the default English language package, its primary purpose is simply
 * to fill in the {@link #argContexts}, {@link #semTagArgStopSet} and
 * {@link #nodesToPrune} data members using a metadata resource.  If this
 * capability is desired in another language package, this class may be
 * subclassed.
 * <p>
 * This class also re-defined the method
 * {@link danbikel.parser.lang.AbstractTraining#hasPossessiveChild(Sexp)}.
 */
public class Training6 extends danbikel.parser.arabic.Training2 {
    
    protected danbikel.parser.arabic.Treebank3 treebank = 
	(danbikel.parser.arabic.Treebank3)Language.treebank();    
    
    protected final Symbol NP = treebank.NPLabel();
    protected final Symbol PP = Symbol.add("PP");
    private Nonterminal nonterminal3 = new Nonterminal();

    public Training6() throws FileNotFoundException, IOException {
	super();
	System.err.println("in arabic Training 6");
	System.out.println("getCanonical NPB="+treebank.getCanonical(Symbol.get("NPB")));
	System.out.println("getCanonical NP1="+treebank.getCanonical(Symbol.get("NP1")));
	System.out.println("getCanonical NPA="+treebank.getCanonical(Symbol.get("NPA")));
	
    }
    
    
    public Sexp preProcess(Sexp tree) {
	//      System.out.println("before transformTags " + tree);
	transformTags(tree);
	//      System.out.println("after transformTags " + tree);
	prune(tree);
	System.out.println("before addNPNumbers\n    " + danbikel.parser.util.Util.prettyPrint(tree));
	addNPNumbers(tree);
	System.out.println("after addNPNumbers  \n    " + danbikel.parser.util.Util.prettyPrint(tree));
	addBaseNPs(tree);
	//repairBaseNPs(tree);
	//addGapInformation(tree);
	//relabelSubjectlessSentences(tree);
	removeNullElements(tree);
	raisePunctuation(tree);
	identifyArguments(tree);
	stripAugmentations(tree);
	System.out.println("after strip  \n    " + danbikel.parser.util.Util.prettyPrint(tree));
	return tree;
    }

    protected Sexp addNPNumbers(Sexp tree) {
	addNPNumbers(tree,0);
	return tree;
    }


    /* 
       (1)if I'm an NP:
            (a)if I have exactly one NP child
	       rename me NP+level
	       call recursively on that child with (level+1)
               call recursively on other children with level0
            (b) if I have exactly two children, and they're both NPs
	       rename me NPA
	       call recursivel on children with level=0
            (c) if none or >2 (does this ever happen)?
               call recursively on all children with level0
       (2) if I'm not an NP
               call recursively on all children with level0

    */

    protected void addNPNumbers(Sexp tree, int level) {
	if (treebank.isPreterminal(tree)) 
	    return;
	SexpList treeList=tree.list();
	Symbol parent = treeList.first().symbol();
	int treeListLen = treeList.length();	
	boolean parentIsNP=treebank.stripAugmentation(parent)==NP;
	boolean dominatesNP=false;
	int NPchild=-1;
	int numNPChildren=0;
	if (parentIsNP) {
	    for (int i = 1; i < treeListLen; i++) {
		Symbol childLabel=treeList.get(i).list().first().symbol();
		if (childLabel==NP) {
		    numNPChildren++;
		    if (NPchild==-1) {
			NPchild=i;
		    }
		}
	    }
	    if (numNPChildren>1) {
		NPchild=-1;
	    }
	    if (numNPChildren==1) {
		if (NPchild==-1) {
		    System.out.println("impossible situation");
		    System.out.println("numNPChildren="+numNPChildren);
		    System.out.println("NPchild="+NPchild);
		    System.out.println("tree="+tree);
		    System.exit(-1);
		}
		Nonterminal parsedParent =
		    treebank.parseNonterminal(parent, nonterminal3);
		//	    parsedParent.base=getNPSymbol(Symbol.get("NP"+(level-1));
		parsedParent.base=getNPSymbol(level);
		treeList.set(0, parsedParent.toSymbol());
	    } else if (numNPChildren==(treeListLen-1)) {
		// all children are NPs
		Nonterminal parsedParent =
		    treebank.parseNonterminal(parent, nonterminal3);
		//	    parsedParent.base=getNPSymbol(Symbol.get("NP"+(level-1));
		parsedParent.base=treebank.NPALabel();
		treeList.set(0, parsedParent.toSymbol());
	    }
	}

	for (int i = 1; i < treeListLen; i++) {
	    if (i==NPchild) {
		addNPNumbers(treeList.get(i),(level+1));
	    } else {
		addNPNumbers(treeList.get(i),0);
	    }
	}
    }

	    


    /*    
    protected boolean addNPNumbers(Sexp tree, int level) {
	System.out.println("level="+level+"tree="+tree);
	if (treebank.isPreterminal(tree)) 
	    return false;
	SexpList treeList=tree.list();
	Symbol parent = treeList.first().symbol();
	int treeListLen = treeList.length();	
	boolean parentIsNP=treebank.stripAugmentation(parent)==NP;
	boolean dominatesNP=false;
	if (parentIsNP) 
	    level++;
	for (int i = 1; i < treeListLen; i++) {
	    boolean childDominatesNP=
		addNPNumbers(treeList.get(i),level);
	    if (childDominatesNP) 
		dominatesNP=true;
	}
	if (dominatesNP && parentIsNP) {
	    System.out.println("adusting parent"+parent);
	    Nonterminal parsedParent =
		treebank.parseNonterminal(parent, nonterminal3);
	    //	    parsedParent.base=getNPSymbol(Symbol.get("NP"+(level-1));
	    parsedParent.base=getNPSymbol(level-1);
	    treeList.set(0, parsedParent.toSymbol());
	}
	return ( (parentIsNP || dominatesNP) ? true : false);
    }
    */
    protected Symbol getNPSymbol(int level) {
	if (level==0)
	    return treebank.NP0Label();
	else if (level==1) 
	    return treebank.NP1Label();
	else if (level==2) 
	    return treebank.NP2Label();
	else if (level==3) 
	    return treebank.NP3Label();
	else if (level==4) 
	    return treebank.NP4Label();
	else if (level==5) 
	    return treebank.NP5Label();
	else if (level==6) 
	    return treebank.NP6Label();
	else if (level==7) 
	    return treebank.NP7Label();
	else if (level==8) 
	    return treebank.NP8Label();
	else {
	    System.out.println("reached level"+level);
	    System.exit(-1);
	}
	return null;
    }


    // had to modify to include check if the parent is any NP except NPB
  /**
   * Handle case where an NP dominates a base NP and has no other children
   * (the base NP is an "only child" of the dominating NP).  This method
   * will effectively remove the base NP node, hooking up all its children
   * as the children of the parent NP.
   */
  protected void removeOnlyChildBaseNPs(Sexp tree) {
    Treebank treebank = Language.treebank();
    if (treebank.isPreterminal(tree))
      return;
    else if (tree.isList()) {
      SexpList treeList = tree.list();
      int treeListLen = treeList.length();
      for (int i = 1; i < treeListLen; i++) {
	Sexp currChild = treeList.get(i);
	boolean currChildRemoved = false;
	boolean hasOnlyChild = treeListLen == 2;
	if (hasOnlyChild &&
	    !treebank.isPreterminal(currChild) && currChild.isList()) {
	  Symbol parentLabel = treebank.stripAugmentation(treeList.symbolAt(0));
	  //	  if (parentLabel == treebank.NPLabel()) {
	  if (treebank.isNP(parentLabel) &&
	      !treebank.isBaseNP(parentLabel)) {
	    Symbol childLabel =
	      treebank.stripAugmentation(currChild.list().symbolAt(0));
	    if (treebank.isBaseNP(childLabel)) {
	      // we've got an NP dominating an only child base NP!
	      // first, remove baseNP from parent
	      currChildRemoved = true;
	      treeList.remove(i);
	      // next, make all baseNP's children become parent's children
	      SexpList childList = currChild.list();
	      int childListLen = childList.length();
	      for (int j = 1; j < childListLen; j++)
		treeList.add(childList.get(j));
	      // need to re-calculate cached length
	      treeListLen = treeList.length();
	    }
	  }
	}
	if (!currChildRemoved)
	  removeOnlyChildBaseNPs(currChild);
      }
    }
  }

  private final static String[] usageMsg = {
    "usage: [-tpnria] [-combine] <filename>\n" +
    "where\n\t" +
    "-t: transform tags\n\t" +
    "-p: prune trees\n\t" +
    "-z: add NPrenumbering\n\t" +
    "-b: add/relabel base NPs\n\t" +
    "-n: remove null elements\n\t" +
    "-r: raise punctuation\n\t" +
    "-i: identify arguments\n\t" +
    "-a: strip augmentations\n\t" +
    "-combine: by default, the tree will be printed to " +
    "System.out after each\n\t\ttransformation; " +
    "if this flag is present, all transformations will "+
    "be\n\t\tapplied and only the final, fully-" +
    "transformed tree will be printed"
  };

  private static void usage() {
    for (int i = 0; i < usageMsg.length; i++) {
      System.err.println(usageMsg[i]);
    }
    System.exit(1);
  }

  /** Test driver for this class. */
  public static void main(String[] args) {
    String filename = null;
    boolean transformTags = false;
    boolean npRenumbering=false;
    boolean prune = false;
    boolean addBaseNPs = false;
    boolean removeNullElements = false;
    boolean raisePunc = false;
    boolean idArgs = false;
    boolean stripAug = false;
    boolean combineTransformations = false;

    for (int i = 0; i < args.length; i++) {
      // for each arg that begins with a '-', examine each of its letters
      // and set booleans accordingly, except for "-combine" flag
      if (args[i].charAt(0) == '-') {
	if (args[i].equals("-combine"))
	  combineTransformations = true;
	else {
	  for (int charIdx = 1; charIdx < args[i].length(); charIdx++) {
	    char curr = args[i].charAt(charIdx);
	    switch (curr) {
	    case 't':
	      transformTags = true;
	      break;
	    case 'p':
	      prune = true;
	      break;
	    case 'z':
	      npRenumbering = true;
	      break;
	    case 'b':
	      addBaseNPs = true;
	      break;
	    case 'n':
	      removeNullElements = true;
	      break;
	    case 'r':
	      raisePunc = true;
	      break;
	    case 'i':
	      idArgs = true;
	      break;
	    case 'a':
	      stripAug = true;
	      break;
	    default:
	      System.err.println("illegal flag: " + curr);
	      usage();
	    }
	  }
	}
      }
      else
	filename = args[i];
    }

    if (filename == null) {
      usage();
    }

    danbikel.parser.arabic.Training6 training = 
	(danbikel.parser.arabic.Training6)Language.training();
    //training.printMetadata();

    try {
      SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
					    Constants.defaultFileBufsize);
      Sexp curr = null;
      while ((curr = Sexp.read(tok)) != null) {
	if (transformTags) {
	  curr = training.transformTags(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (prune) {
	  curr = training.prune(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (npRenumbering) {
	  curr = training.addNPNumbers(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (addBaseNPs) {
	  curr =training.addBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (removeNullElements) {
	  curr = training.removeNullElements(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (raisePunc) {
	  curr = training.raisePunctuation(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (idArgs) {
	  curr =training.identifyArguments(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (stripAug) {
	  curr = training.stripAugmentations(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (combineTransformations)
	  System.out.println(curr);
      }
    }
    catch (UnsupportedEncodingException uee) {
      System.err.println(uee);
    }
    catch (FileNotFoundException fnfe) {
      System.err.println(fnfe);
    }
    catch (IOException ioe) {
      System.err.println(ioe);
    }
  }
}

	

	    
    /*
  

    protected boolean isNPPP(SexpList treeList, Symbol parent) {
	if (treeList.length()!=3)
	    return false;
	if (treebank.stripAugmentation(parent)!=NP)
	    return false;
	// has two children, first is NP or NPB
	// second is PP
	//	System.out.println("getting symbol for child " + treeList.get(1));
	if (!(treebank.isNP(treeList.get(1).list().first().symbol()))) 
	    return false;
	//	System.out.println("getting symbol for child " + treeList.get(2));
	if (treebank.stripAugmentation(treeList.get(2).list().first().symbol())!=PP)
	    return false;
	return true;
    }

	

    protected Sexp addNPNumbers(Sexp tree, int level) {
	// don't need to look at grandparent context like with
	// addBaseNPs, so this is pretty straightforward

	if (treebank.isPreterminal(tree)) 
	    return tree;
	// any need to check for isList() here?
	SexpList treeList=tree.list();
	int treeListLen = treeList.length();	
	Symbol parent = treeList.first().symbol();
	if (isNPPP(treeList,parent)) {
	  Nonterminal parsedParent =
	      treebank.parseNonterminal(parent, nonterminal3);
	  parsedParent.base=Symbol.get("NP"+level);
	  treeList.set(0, parsedParent.toSymbol());
	  addNPNumbers(treeList.get(1),level++);
	  addNPNumbers(treeList.get(2),0);
	} else {
	    for (int i = 1; i < treeListLen; i++) 
		addNPNumbers(treeList.get(i),0);
	}
    return tree;
    }
    */




