/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
    package danbikel.parser.arabic;

import java.util.BitSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;
import danbikel.lisp.*;
import danbikel.parser.Language;
import danbikel.parser.Nonterminal;

/*
 * Provides data and methods speciifc to the structures found in the English
 * Treebank (the
 * <a target="_blank"
 * href="http://www.cis.upenn.edu/~treebank/">Penn Treebank</a>) or any other
 * treebank that conforms to the Treebank II annotation guidelines for
 * <a target="_blank" href="ftp://ftp.cis.upenn.edu/pub/treebank/doc/tagguide.ps.gz">part-of-speech tagging</a>
 * and
 * <a target="_blank"
 * href="ftp://ftp.cis.upenn.edu/pub/treebank/doc/manual/">bracketing</a>.
 */

/**
 * Provides data and methods speciifc to the structures found in the English
 * Treebank (the Penn Treebank) or any other treebank that conforms to the
 * Treebank II annotation guidelines for part-of-speech tagging and bracketing.
 */
public class Treebank3 extends danbikel.parser.arabic.Treebank {

    //  static final Symbol NP = Symbol.add("NP");
  static final Symbol NP0 = Symbol.add(NP + "0");
  static final Symbol NP1 = Symbol.add(NP + "1");
  static final Symbol NP2 = Symbol.add(NP + "2");
  static final Symbol NP3 = Symbol.add(NP + "3");
  static final Symbol NP4 = Symbol.add(NP + "4");
  static final Symbol NP5 = Symbol.add(NP + "5");
  static final Symbol NP6 = Symbol.add(NP + "6");
  static final Symbol NP7 = Symbol.add(NP + "7");
  static final Symbol NP8 = Symbol.add(NP + "8");
  static final Symbol NPA = Symbol.add(NP + "A");

    public Symbol NP0Label() { return NP0; }
    public Symbol NP1Label() { return NP1; }
    public Symbol NP2Label() { return NP2; }
    public Symbol NP3Label() { return NP3; }
    public Symbol NP4Label() { return NP4; }
    public Symbol NP5Label() { return NP5; }
    public Symbol NP6Label() { return NP6; }
    public Symbol NP7Label() { return NP7; }
    public Symbol NP8Label() { return NP8; }
    public Symbol NPALabel() { return NPA; }

    // why isn't this protected?
  private static Symbol[][] canonicalLabelMapData = {
    {baseNP, NP},
    {subjectlessS, S},
    {NP0, NP},
    {NP1, NP},
    {NP2, NP},
    {NP3, NP},
    {NP4, NP},
    {NP5, NP},
    {NP6, NP},
    {NP7, NP},
    {NP8, NP},
    {NPA, NP},
  };

  private static HashMap canonicalLabelMap =
    new HashMap(canonicalLabelMapData.length);
  static {
    for (int i = 0; i < canonicalLabelMapData.length; i++)
      canonicalLabelMap.put(canonicalLabelMapData[i][0],
                            canonicalLabelMapData[i][1]);
  }

  protected Symbol defaultGetCanonical(Symbol label) {
    for (int i = 0; i < nonterminalExceptionSet.length; i++)
      if (label == nonterminalExceptionSet[i])
	return label;
    Symbol strippedLabel = stripAugmentation(label);
    Symbol mapEntry = (Symbol)canonicalLabelMap.get(strippedLabel);
    return ((mapEntry == null) ? strippedLabel : mapEntry);
  }


    /*
  public Symbol getCanonical(Symbol label) {
    for (int i = 0; i < nonterminalExceptionSet.length; i++)
      if (label == nonterminalExceptionSet[i])
	return label;
    Symbol strippedLabel = stripAugmentation(label);
    Symbol mapEntry = (Symbol)canonicalLabelMap.get(strippedLabel);
    return ((mapEntry == null) ? strippedLabel : mapEntry);
  }

  public Symbol getCanonical(Symbol label, boolean stripAugmentations) {
    if (stripAugmentations)
      return getCanonical(label);
    else {
      Symbol mapEntry = (Symbol)canonicalLabelMap.get(label);
      return (mapEntry == null ? label : mapEntry);
    }
  }
    */

}
