/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
    package danbikel.parser.english;

import java.util.*;
import java.io.*;
import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.lisp.*;


public class Training2 extends danbikel.parser.english.Training {

      private Nonterminal nonterminal3 = new Nonterminal();

    static Symbol codeSymbol=Symbol.add("CODE");
    
  public Training2() throws FileNotFoundException, IOException {
      super();
      System.out.println("in english training 2");
  }

  public Sexp preProcess(Sexp tree) {
    prune(tree);
    addBaseNPs(tree);
    repairBaseNPs(tree);
    addGapInformation(tree);
    relabelSubjectlessSentences(tree);
    removeNullElements(tree);
    raisePunctuation(tree);
    identifyArguments(tree);
    //    stripAugmentations(tree);
    //    System.out.println("before stripIndices tree="+tree);
    stripIndices(tree);
    //    System.out.println("after stripIndices tree="+tree);
    fixSubjectlessSentences(tree);
    return tree;
  }

  public Sexp stripIndices(Sexp tree) {
    if (treebank.isPreterminal(tree))
      return tree;
    if (tree.isList()) {
      Symbol label = tree.list().first().symbol();
      tree.list().set(0, treebank.stripIndex(label, nonterminal3));
      int treeListLen = tree.list().length();
      for (int i = 1; i < treeListLen; i++) 
	stripIndices(tree.list().get(i));
    }
    return tree;
  }

  public void postProcess(Sexp tree) {
    removeOnlyChildBaseNPs(tree);
    System.out.println("after removeOnlyChildBaseNPs = " + tree);    
    //    canonicalizeNonterminals(tree);
    //    System.out.println("after canonicalizeNonterminals = " + tree);    
  }

//added later as special check for CODE.  shouldn't affect anything done earlier
    public boolean removeWord(Symbol word, Symbol tag, int idx, SexpList sentence,
			      SexpList tags, SexpList originalTags,
			      Set prunedPretermsPosSet,
			      Map prunedPretermsPosMap) {
	/*
	  return (tag != null ?
	  prunedPretermsPosSet.contains(tag) :
	  prunedPretermsPosMap.containsKey(word));
	*/
	if (tag==codeSymbol) 
	    return true;
	return super.removeWord(word,tag,idx,sentence,tags,originalTags,prunedPretermsPosSet,prunedPretermsPosMap);
    }
}
