package danbikel.parser.examine;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

public class ExamineATBBaseNPs extends ExamineBaseNPs {

    /* no use of the bothTags stuff. Instead assume the
       ATB is the combined version with 
       (pos gloss lemma vocalized unvocalized)
       just strip off the case stuff
    */

    public ExamineATBBaseNPs() {
	super(false);
    }

    protected void  getContextInfo(SexpList treeList,ContextInfo contextInfo) {
	//	System.out.println("getting contextInfo for " + treeList);
	int treeListLen=treeList.length();

	contextInfo.siblingsAllPreterminals=true;
	contextInfo.reducedTags=new String[treeListLen-1];
	contextInfo.fullTags= new String[treeListLen-1];
	contextInfo.words=new String[treeListLen-1];
	contextInfo.siblingsAllPreterminals=true;

	contextInfo.glosses=new String[treeListLen-1];
	contextInfo.lemmas=new String[treeListLen-1];
	contextInfo.vocalized=new String[treeListLen-1];

	contextInfo.parent=treeList.symbolAt(0);
	contextInfo.parentIsNP=Language.treebank().isNP(treeList.first().symbol());
	contextInfo.filename=currentFilename;
	contextInfo.sentNum=currentSentNum;

	for (int i = 1; i < treeList.length(); i++) {
	    Sexp child=treeList.get(i);
	    if (!(Language.treebank().isPreterminal(child))) {
		contextInfo.siblingsAllPreterminals=false;
		contextInfo.fullTags[i-1]=child.list().symbolAt(0).toString();
		contextInfo.reducedTags[i-1]=contextInfo.fullTags[i-1];
		contextInfo.words[i-1]="(nt)";
		contextInfo.glosses[i-1]="(nt)";
		contextInfo.lemmas[i-1]="(nt)";
		contextInfo.vocalized[i-1]="(nt)";

	    } else {
		contextInfo.fullTags[i-1]=child.list().symbolAt(0).toString();
		contextInfo.glosses[i-1]=child.list().symbolAt(1).toString();		
		contextInfo.lemmas[i-1]=child.list().symbolAt(2).toString();		
		contextInfo.vocalized[i-1]=child.list().symbolAt(3).toString();		
		contextInfo.words[i-1]=child.list().symbolAt(4).toString();
		getReducedTags(contextInfo,i);

		
	    }
	}
    }

    public void getReducedTags(ContextInfo contextInfo, int i) {
	String parts[]=contextInfo.fullTags[i-1].split("\\+");
	String reduced=null;
	for (int j=0; j<parts.length; j++) {
	    if (parts[j].indexOf("CASE")==-1 &&
		parts[j].indexOf("NSUFF")==-1) {
		if (reduced==null) {
		    reduced=parts[j];
		} else {
		    reduced+="+"+parts[j];
		}
	    }
	}
	if (reduced==null) {
	    System.out.println("warning no reduced tag for " + contextInfo.fullTags[i-1]);
	}
	contextInfo.reducedTags[i-1]=reduced;
    }



		
    public void showEntries(String name, ArrayList entries) {
	if (entries.size()==0)
	    return;
	System.out.println();
	int maxWordsLen=-1;
	int maxReducedTagsLen=-1;
	int maxFullTagsLen=-1;

	for (int i=0; i<entries.size(); i++) {
	    ContextInfo entry=(ContextInfo)entries.get(i);
	    String wordsStr="";
	    String reducedTagsStr="";
	    String fullTagsStr="";
	    for (int j=0; j<entry.words.length; j++) {
		wordsStr+=entry.words[j]+" ";
		reducedTagsStr+=entry.reducedTags[j]+" ";
		if (bothTags)
		    fullTagsStr+=entry.fullTags[j]+" ";
	    }
	    if (wordsStr.length()>maxWordsLen) 
		maxWordsLen=wordsStr.length();
	    if (reducedTagsStr.length()>maxReducedTagsLen)
		maxReducedTagsLen=reducedTagsStr.length();
	    if (fullTagsStr.length()>maxFullTagsLen)
		maxFullTagsLen=fullTagsStr.length();
	}
	maxWordsLen=(maxWordsLen>20 ? 20 : maxWordsLen);
	maxReducedTagsLen=(maxReducedTagsLen>30 ? 30 : maxReducedTagsLen);
	maxFullTagsLen=(maxFullTagsLen>60 ? 60 : maxFullTagsLen);

	System.out.println(name);
	for (int i=0; i<entries.size(); i++) {
	    ContextInfo entry=(ContextInfo)entries.get(i);
	    String wordsStr="";
	    String reducedTagsStr="";
	    String fullTagsStr="";
	    String lemmaStr="";
	    String glossStr="";
	    String vocalizedStr="";
	    for (int j=0; j<entry.words.length; j++) {
		wordsStr+=entry.words[j]+" ";
		reducedTagsStr+=entry.reducedTags[j]+" ";
		fullTagsStr+=entry.fullTags[j]+" ";
		lemmaStr+=convertBack(entry.lemmas[j])+" ";
		glossStr+=convertBack(entry.glosses[j])+" ";
		vocalizedStr+=convertBack(entry.vocalized[j])+" ";
	    }
	    System.out.println(danbikel.parser.util.SKUtil.leftJustify(4,i+") ")+ 
			       danbikel.parser.util.SKUtil.leftJustify(maxWordsLen,wordsStr)+
			       danbikel.parser.util.SKUtil.leftJustify(maxReducedTagsLen,reducedTagsStr)+
			       danbikel.parser.util.SKUtil.leftJustify(maxFullTagsLen,fullTagsStr) +
			       danbikel.parser.util.SKUtil.leftJustify(20,entry.filename+"/"+entry.sentNum));
	    System.out.println("    "+
			       danbikel.parser.util.SKUtil.leftJustify(30,lemmaStr) +
			       danbikel.parser.util.SKUtil.leftJustify(30,glossStr)+
			       danbikel.parser.util.SKUtil.leftJustify(30,vocalizedStr));
	}
    }

    String convertBack(String str) {
	str=str.replaceAll("\\-","\\(");
	str=str.replaceAll("\\=","\\)");
	str=str.replaceAll("@","_");	
	return str;
    }
	
    public static void main(String[] args) {
	ArrayList dir=new ArrayList();
	dir.add(new File(args[0]));
	
	//	System.out.println("added"+args[1]+"to dir");
	List files=danbikel.parser.util.SKUtil.expandDirectories(dir,defaultExtension,false,false);
	try {
	    ExamineATBBaseNPs examineATBBaseNPs=new ExamineATBBaseNPs();
	    examineATBBaseNPs.process(files);
	}
	catch (FileNotFoundException fnfe) {
	System.err.println(fnfe);
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    
	}
    }

}

