package danbikel.parser.examine;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

public class ExamineVPs extends ExamineConstituent {

    int countVPwithVhead=0;

    public ExamineVPs(boolean bothTags) {
	super(bothTags);
    }

    boolean isVerb(String tag) {
	if (tag.equals("VBP") ||
	    tag.equals("VBN") ||
	    tag.equals("VBD") ||
	    tag.equals("VB")) {
	    return true;
	}
	return false;
    }

    boolean containsVerb(ContextInfo contextInfo) {
	for (int i=0; i<contextInfo.reducedTags.length; i++) {
	    if (isVerb(contextInfo.reducedTags[i]))
		return true;
	}
	return false;
    }

    String getKey(ContextInfo contextInfo) {
	/*
	for (int i=0; i<contextInfo.reducedTags.length; i++) {
	    System.out.println("tags["+i+"]="+contextInfo.reducedTags[i]);
	}
	*/  
    
	String key="";
	for (int i=0; i<contextInfo.reducedTags.length; i++) {
		key+=contextInfo.reducedTags[i]+" ";
	}
	if (key.startsWith("NN NP-SBJ[PRP$] NP-OBJ")) {
	    key="NN NP-SBJ[PRP$] NP-OBJ";
	} else if (key.startsWith("NNS NP-SBJ[PRP$] NP-OBJ")) {
	    key="NNS NP-SBJ[PRP$] NP-OBJ";	    
	}

	return key;
    }

    void adjustContext(Sexp tree, ContextInfo contextInfo) {
	//	System.out.println("tree="+tree);
	if (contextInfo.reducedTags.length<2 || 
	    !contextInfo.reducedTags[1].equals("NP-SBJ")) {
	    return;
	}
	//	System.out.println("Reducedtags[1] is NP-SBJ");
	//	System.out.println("tree.list().get(2)="+tree.list().get(2));
	ArrayList children=danbikel.parser.util.Util.collectWordObjects(tree.list().get(2));
	/*
	System.out.println("children of subject " + tree + 
			       " length = " + children.size());
	*/
	String reduced="";
	String full="";
	for (int j=0; j<children.size(); j++) {
	    Word w=(Word)children.get(j);
	    String tagStr=w.tag().toString();
	    if (j>0) {
		reduced+=" ";
		full+=" ";
	    }
	    reduced+=getNewTag(tagStr);
	    full+=getOldTagKey(tagStr);
	}
	contextInfo.reducedTags[1]+="["+reduced+"]";
	contextInfo.fullTags[1]+="["+full+"]";		
	//		System.out.println("word="+w.word() + " " + w.tag());
    }


	    
    protected void process(Sexp tree) {
	if (Language.treebank().isPreterminal(tree)) {
	  return;
      }
	else if (tree.isList()) {
	  SexpList treeList = tree.list();
	  ContextInfo contextInfo=null;
	  String parentStr=treeList.symbolAt(0).toString();
	  if (parentStr.startsWith("VP")) {
	      contextInfo=new ContextInfo();
	      getContextInfo(treeList,contextInfo);
	      adjustContext(treeList,contextInfo);
	      if (!containsVerb(contextInfo)) {
		  String key=getKey(contextInfo);
		  ArrayList entries=(ArrayList)map.get(key);
		  if (entries==null) {
		      map.put(key,new ArrayList());
		      entries=(ArrayList)map.get(key);
		  }
		  entries.add(contextInfo);
		  count++;
	      } else {
		  countVPwithVhead++;
	      }
	     
	  }
	  for (int i = 1; i < treeList.length(); i++) {
	      process(treeList.get(i));
	  }
	}
    }

    protected void showResults() {
	System.out.println("num VPs with V head = " + countVPwithVhead);
	super.showResults();
    }

    public static void main(String[] args) {
	boolean bothTags= (args[0].equals("true")) ? true : false;
	String listOfWords=args[1];
	ArrayList dir=new ArrayList();
	dir.add(new File(args[2]));
	
	//	System.out.println("added"+args[1]+"to dir");
	List files=danbikel.parser.util.SKUtil.expandDirectories(dir,defaultExtension,false,false);
	try {
	    ExamineVPs examineVPs=new ExamineVPs(bothTags);
	    examineVPs.process(files);
	}
	catch (FileNotFoundException fnfe) {
	System.err.println(fnfe);
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    
	}
    }
}
