package danbikel.parser.examine2;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

class ContextInfo {
    String filename=null;
    int sentNum=0;
    int childNum=-1;
    Symbol parent=null;
    boolean parentIsNP=false;
    boolean siblingsAllPreterminals=false;
    String[] fullTags=null;
    String[] reducedTags=null;

    String[] words=null;

    String[] glosses=null;
    String[] lemmas=null;
    String[] vocalized=null;

    String[] vocShrunk=null;
    String[] posShrunk=null;

    static final int posGoldX=0;
    static final int vocShrunkX=1;
    static final int posShrunkX=2;
    static final int glossX=3;
    static final int lemmaX=4;
    static final int vocX=5;
    static final int unvocX=6;

    public static boolean debug=false;


    protected void  fill(SexpList treeList) {
	//	System.out.println("getting contextInfo for " + treeList);
	int treeListLen=treeList.length();

	siblingsAllPreterminals=true;
	reducedTags=new String[treeListLen-1];
	fullTags=new String[treeListLen-1];
	words=new String[treeListLen-1];
	siblingsAllPreterminals=true;

	parent=treeList.symbolAt(0);
	parentIsNP=Language.treebank().isNP(treeList.first().symbol());


	for (int i = 1; i < treeList.length(); i++) {
	    Sexp child=treeList.get(i);
	    if (!(Language.treebank().isPreterminal(child))) {
		siblingsAllPreterminals=false;
		fullTags[i-1]=child.list().symbolAt(0).toString();
		reducedTags[i-1]=reducedTags[i-1];
		words[i-1]="(nt)";
	    } else {
		fullTags[i-1]=child.list().symbolAt(0).toString();
		reducedTags[i-1]=reducedTags[i-1];
		words[i-1]=child.list().symbolAt(1).toString();
	    }
	}
    }

    void  fillFromCombinedATB(Sexp tree) {
	if (debug)
	    System.err.println("fillFromCombinedATB " + tree);
	SexpList treeList=tree.list();
	int treeListLen=treeList.length();

	siblingsAllPreterminals=true;
	reducedTags=new String[treeListLen-1];
	fullTags= new String[treeListLen-1];
	words=new String[treeListLen-1];
	siblingsAllPreterminals=true;

	glosses=new String[treeListLen-1];
	lemmas=new String[treeListLen-1];
	vocalized=new String[treeListLen-1];

	vocShrunk=new String[treeListLen-1];
	posShrunk=new String[treeListLen-1];

	parent=treeList.symbolAt(0);
	parentIsNP=Language.treebank().isNP(treeList.first().symbol());


	for (int i = 1; i < treeList.length(); i++) {
	    Sexp child=treeList.get(i);
	    
	    if (Language.treebank().isPreterminal(child)) {
		if (debug)
		    System.err.println("is preterminal" + child);
		fullTags[i-1]=child.list().symbolAt(posGoldX).toString();
		glosses[i-1]=child.list().symbolAt(glossX).toString();		
		lemmas[i-1]=child.list().symbolAt(lemmaX).toString();		
		vocalized[i-1]=child.list().symbolAt(vocX).toString();		
		words[i-1]=child.list().symbolAt(unvocX).toString();
		reducedTags[i-1]=reducedTagFromCombinedATB(fullTags[i-1]);
		vocShrunk[i-1]=child.list().symbolAt(vocShrunkX).toString();
		posShrunk[i-1]=child.list().symbolAt(posShrunkX).toString();		
	    } else if (Language.treebank().isNullElementPreterminal(child)) {
		if (debug)
		    System.err.println("is null element " + child);
		fullTags[i-1]=child.list().symbolAt(posGoldX).toString();
		reducedTags[i-1]=fullTags[i-1];
		words[i-1]=child.list().symbolAt(0).toString();
		glosses[i-1]="(ec)";
		lemmas[i-1]="(ec)";
		vocalized[i-1]="(ec)";
	    } else {
		if (debug)
		    System.err.println("is nonterminal " + child);
		siblingsAllPreterminals=false;
		fullTags[i-1]=child.list().symbolAt(posGoldX).toString();
		reducedTags[i-1]=reducedNTFromCombinedATB(fullTags[i-1]);
		words[i-1]="(nt)";
		glosses[i-1]="(nt)";
		lemmas[i-1]="(nt)";
		vocalized[i-1]="(nt)";
	    }
	}
    }



    String reducedTagFromCombinedATB(String fullTag) {
	//	System.err.println("reducing " + fullTag);
	
	String parts[]=fullTag.split("\\+");
	String reduced=null;
	for (int j=0; j<parts.length; j++) {
	    String rp=reducePart(parts[j]);
	    if (rp!=null) {
		if (reduced==null) {
		    reduced=rp;
		} else {
		    reduced+="+"+rp;
		}
	    }
	}
	if (reduced==null) {
	    System.err.println("warning no reduced tag for " + fullTag);
	}
	//	System.err.println("reduced " + fullTag + " to " + reduced);
	return reduced;
    }

		//	    if (parts[j].indexOf("CASE")==-1 &&
		//		parts[j].indexOf("NSUFF")==-1) {

    String reducePart(String part) {
	if (part.indexOf("CASE")>-1) 
	    return null;
	if (part.indexOf("NSUFF")>-1)
	    return null;
	if (part.startsWith("POSS_PRON"))
	    return "POSS_PRON";
	if (part.startsWith("PVSUFF_SUBJ"))
	    return "PVSUFF_SUBJ";
	if (part.startsWith("DEM_PRON")) 
	    return "DEM_PRON";
	return part;
    }
	


    String reducedNTFromCombinedATB(String fullTag) {

	/*
	System.err.println("reducing " + fullTag);
	String parts[]=fullTag.split("[\\-\\+\\=]");
	System.out.println("numParts="+parts.length);
	String reduced=null;
	String ret=parts[0];
	for (int j=1; j<parts.length; j+=2) {
	    System.err.println("parts["+j+"]="+parts[j]);
	    System.err.println("parts["+j+1+"]="+parts[j+1]);
	    String delim=parts[j];
	    String aug=parts[j+1];
	    if (delim.equals("=")) {
		ret+="="+aug;
	    } else if (delim.equals("+")) {
		System.err.println("unexpected + " + fullTag);
		ret+="+"+aug;
	    } else if (!Text.isAllDigits(aug)) {
		ret+="-"+aug;
	    }
	}
	return ret;
	*/
	// above not correct, should use regex
	// for now just strip index

	//	System.err.println("reducing " + fullTag);
	int lastDash=fullTag.lastIndexOf("-");
	if (lastDash==-1) {
	    return fullTag;
	}
	String lastAug=fullTag.substring(lastDash+1);
	if (Text.isAllDigits(lastAug)) {
	    String ret=fullTag.substring(0,lastDash);
	    //	    System.err.println("reducing "+fullTag+" to " + ret);
	    return ret;
	}
	//	System.err.println("returning full tag, not all digits " + lastAug);
	return fullTag;

    }
	
    boolean containsVerb() {
	for (int i=0; i<fullTags.length; i++) {
	    if (isVerbTag(fullTags[i])) {
		//		System.err.println("isVerbTag checking true " + fullTags[i]);
		return true;
	    } 
	}
	return false;
    }

  public boolean isVerbTag(String tag) {
      String stripped=stripAugmentation(tag);
      //      System.err.println("tag="+tag+" stripped="+stripped);
      if (stripped.equals(IV) ||
	  stripped.equals(IVPASS) ||
	  stripped.equals(PV) ||
	  stripped.equals(PVPASS) ||
	  stripped.equals(CV) ||
	  stripped.equals(DV)) {
	  //	  System.out.println("isVerbTag " + tag + " true");
	  return true;
      }
      //      System.out.println("isVerbTag " + tag + " false");
      return false;
  }

    static final String IV="IV";
    static final String IVPASS = "IV_PASS";
  static final String PV = "PV";
  static final String PVPASS = "PV_PASS";
  static final String CV = "CV";
  static final String NOUNPROP = "NOUN_PROP";
  static final String NOUN = "NOUN";
  static final String ADJ = "ADJ";
  static final String PVSUFF = "PVSUFF";
  static final String IVSUFF = "IVSUFF";
  static final String NUM = "NUM";
  static final String DV = "DV";


  public String stripAugmentation(String labelStr) {
    String retSymbol=null;
    if (labelStr.indexOf("+IV_PASS+")>-1 ||
	labelStr.endsWith("+IV_PASS")) {
	retSymbol=IVPASS;
    } else if (labelStr.indexOf("+IV+")>-1 ||
	labelStr.endsWith("+IV")) {
	retSymbol=IV;
    } else if (labelStr.indexOf("+PV_PASS+")>-1 ||
	labelStr.endsWith("+PV_PASS")) {
	retSymbol=PVPASS;
    } else if (labelStr.indexOf("+PV+")>-1 ||
	labelStr.endsWith("+PV")) {
	retSymbol=PV;
    } else if (labelStr.indexOf("+CV+")>-1 ||
	labelStr.endsWith("+CV")) {
	retSymbol=CV;
    } else if (labelStr.indexOf("NOUN_PROP")>-1) {
	retSymbol=NOUNPROP;
    } else if (labelStr.indexOf("NOUN")>-1) {
	retSymbol=NOUN;
    } else if (labelStr.indexOf("ADJ")>-1) {
	retSymbol=ADJ;
    } else if (labelStr.startsWith("PVSUFF_DO")) {
	retSymbol=PVSUFF;
    } else if (labelStr.startsWith("IVSUFF_DO")) {
	retSymbol=IVSUFF;
    } else if (labelStr.startsWith("NUM")) {
	retSymbol=NUM;
    } else {
	int labelStrLen = labelStr.length();
	int augmentationIdx = -1;
	
	for (int i = 0; i < labelStrLen; i++) {
	    if (labelStr.substring(i,i+1).equals("-") ||
		labelStr.substring(i,i+1).equals("+") ||
		labelStr.substring(i,i+1).equals("=")) {
		augmentationIdx = i;
		break;
	    }
	}
	retSymbol=((augmentationIdx == -1) ? labelStr :
		   labelStr.substring(0, augmentationIdx));
    }
    //    System.out.println("stripAugmentation" + label + " = " + retSymbol);
    return retSymbol;
  }
    
    String keyFromReducedTag() {
	String key="";
	for (int i=0; i<reducedTags.length; i++) {
	    key+=reducedTags[i]+" ";
	}	
	return key.trim();
    }

}


