package danbikel.parser.examine2;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;


/*

go through each unvocalized word.
for each one, want to know how many core POS tags.
And for each core tag, # of different (lemma, vocalized) choices

unvocalizedMap: key=unvocalizedWord
     entry is itself a map, callit posMap

unvocalizedPosMap: key=unvocalized+corePostag
      entry is itself a map, callit lemmaVocalizedMap

unvocalizedPosLemmaVocalizedMap: key= unvocalizedWord+corePosTag+Lemma+Vocalized
      entry is a unvocalizedPosLemmaVocalized structure


and from other direction, want to know

lemmaMap: key= lemma
entry  is itself a map, callit lemmaPosMap

lemmaPosMap: key=lemma+corePosTag
entry is itself a map, callit lemmaPosUnvocalized

lemmaPosUnvocalized: key=lemma+corePosTag+Unvocalized
entry is a LemmaPosUnvocalized


*/

// direction1
class UnvocalizedMapEntry {
    int num;
    String unvocalized;
    java.util.HashMap posMap=new java.util.HashMap();
    
}

class UnvocalizedPosMapEntry {
    int num;
    String pos;
    java.util.HashMap lemmaMap=new java.util.HashMap();
}

class UnvocalizedPosLemmaMapEntry {
    int num;
    String lemma;
    java.util.HashMap vocalizedMap=new java.util.HashMap();
}

class UnvocalizedPosLemmaVocalizedMapEntry {
    String vocalized;
    String gloss;
    int num;
    // and then also keep here context information
}


// direction2
class LemmaMapEntry {
    int num;
    String lemma;
    java.util.HashMap posMap=new java.util.HashMap();
    
}

class LemmaPosMapEntry {
    int num;
    String pos;  // can either be posShrunk or posGold
    java.util.HashMap unvocalizedMap=new java.util.HashMap();
}

class LemmaPosUnvocalizedMapEntry {
    String unvocalized;
    int num;
    // and then also keep here context information
    java.util.HashMap vocalizedMap=new java.util.HashMap();
}

class LemmaPosUnvocalizedVocalizedMapEntry {
    String fullPos;
    String vocalized;  // can either be vocShrunk or voc
    String gloss;
    int num;
}
    



public class ExamineATB extends ExamineCore {


    int countVPwithVHead=0;
    CollectResults headlessVPs=new CollectResults();

    java.util.HashMap unvocalizedMap=null;
    java.util.HashMap lemmaMap=null;

    static final int posGold=0;
    static final int vocShrunk=1;
    static final int posShrunk=2;
    static final int gloss=3;
    static final int lemma=4;
    static final int voc=5;
    static final int unvoc=6;

    static final int posForMap2=posShrunk;
    static final int wordForMap2=vocShrunk;

    static String outputDir="/nlp/home/beatrice/parsing/arabic/analysis";


    public ExamineATB() {
	unvocalizedMap=new java.util.HashMap(); 
	lemmaMap=new java.util.HashMap();

	
    }

    protected void process(Sexp tree) {

      if (Language.treebank().isNullElementPreterminal(tree)) {
	  return;
      } else   if (Language.treebank().isPreterminal(tree)) {
	  /*
	    SexpList treeList = tree.list();
	    Symbol currLabel = treeList.symbolAt(0);
	    treeList.set(0, treebank.getCanonical(currLabel));
	  */
	  return;
      }
      else if (tree.isList()) {
	  SexpList treeList = tree.list();
	  ContextInfo contextInfo=null;
	  String parentStr=treeList.symbolAt(0).toString();
	  boolean isNP=Language.treebank().isNP(treeList.first().symbol());
	  boolean isVP=parentStr.startsWith("VP");
	  if (hasOnePreterminalBesidesPunc(treeList)) {
	      contextInfo=new ContextInfo();
	      contextInfo.fillFromCombinedATB(tree);
	      contextInfo.filename=currentFilename;
	      contextInfo.sentNum=currentSentNum;
	      //	      storePreterminalInfo(contextInfo);
	      if (isVP) {
		  processVP(contextInfo,tree);
	      }
	  }
	  for (int i = 1; i < treeList.length(); i++) {
	      process(treeList.get(i));
	  }
      } else {
      System.err.print("not list or preterminal"+tree);
      }
  }

    void processVP(ContextInfo contextInfo, Sexp tree) {
	//	System.err.println("processVP " + treeList);
	  SexpList treeList = tree.list();

	if (contextInfo.containsVerb()) {
	    countVPwithVHead++;
	    //	    System.err.println("containsVerb true " + treeList);
	    return;
	}
	//	System.err.println("containsVerb false");
	//	System.err.println("verbless VP"+treeList);
	String key1=contextInfo.keyFromReducedTag();
	String key2=getKeyForVP(contextInfo,treeList);
	if (!key1.equals(key2)) {
	    //	    System.err.println("key adjusted from " + key1 + " to "+ key2);
	} else {
	    //	    System.err.println("key =  "+ key2);
	}
	headlessVPs.enterIntoMap(key2, contextInfo);
    }

    String getKeyForVP(ContextInfo contextInfo, SexpList treeList) {
	String key=contextInfo.keyFromReducedTag();
	if (contextInfo.reducedTags.length<2 || 
	    !contextInfo.reducedTags[1].equals("NP-SBJ")) {
	    return key;
	}
	//	ContextInfo.debug=true;
	ContextInfo subjectContextInfo=new ContextInfo();

	subjectContextInfo.fillFromCombinedATB(treeList.get(2));
	String subjectReduced=subjectContextInfo.keyFromReducedTag();
	if (subjectReduced.equals("-NONE-")) {
	    subjectReduced="";
	}
	
	//	System.err.println("subjectContext="+treeList.get(2) + " subjectReduced="+subjectReduced);
	//	ContextInfo.debug=false;
	/*
	ArrayList children=danbikel.parser.util.Util.collectWordObjects(treeList.get(2));
	String childrenTags="";
	for (int j=0; j<children.size(); j++) {	
	    Word w=(Word)children.get(j);
	    childrenTags+=(j>0 ? " " : "") + w.tag().toString();
	}
	*/
	key=contextInfo.reducedTags[0]+" ";
	//	key+=contextInfo.reducedTags[1]+"["+childrenTags+"]"+" ";
	key+=contextInfo.reducedTags[1]+"["+subjectReduced+"]"+" ";
	for (int i=2; i<contextInfo.reducedTags.length; i++) {
	    key+=contextInfo.reducedTags[i]+" ";
	}	
	return key;
    }

	

    void storePreterminalInfo(ContextInfo contextInfo) {
	/*	
	for (int i=0; i<contextInfo.fullTags.length; i++) {
	    System.out.println("got info for " + 
			       "fullTag="+contextInfo.fullTags[i]+
			       " reducedTag="+contextInfo.reducedTags[i]+
			       " word="+contextInfo.words[i]+
			       " gloss="+contextInfo.glosses[i]+
			       " lemma="+contextInfo.lemmas[i]+
			       " vocalized="+contextInfo.vocalized[i]);
	}

	*/
	for (int i=0; i<contextInfo.words.length; i++) {
	    if (!contextInfo.words[i].equals("(nt)")) {
		storeInMap1(contextInfo,i);
		storeInMap2(contextInfo,i);
	    }
	}

    }

    void storeInMap1(ContextInfo contextInfo, int i) {
	
	java.util.HashMap posMap=null;

	String unvocalizedMapKey="u="+contextInfo.words[i];
	String unvocalizedPosMapKey=unvocalizedMapKey+"p="+contextInfo.reducedTags[i];
	String unvocalizedPosLemmaMapKey=unvocalizedPosMapKey+"l="+contextInfo.lemmas[i];
	String unvocalizedPosLemmaVocalizedMapKey=unvocalizedPosLemmaMapKey+"v="+contextInfo.vocalized[i];
	
	UnvocalizedMapEntry unvocalizedMapEntry=
	    (UnvocalizedMapEntry)unvocalizedMap.get(unvocalizedMapKey);
	if (unvocalizedMapEntry==null) {
	    unvocalizedMap.put(unvocalizedMapKey,new UnvocalizedMapEntry());
	    unvocalizedMapEntry=(UnvocalizedMapEntry)unvocalizedMap.get(unvocalizedMapKey);
	    unvocalizedMapEntry.unvocalized=contextInfo.words[i];
	}
	unvocalizedMapEntry.num++;
	
	UnvocalizedPosMapEntry unvocalizedPosMapEntry=
	    (UnvocalizedPosMapEntry)unvocalizedMapEntry.posMap.get(unvocalizedPosMapKey);
	if (unvocalizedPosMapEntry==null) {
	    unvocalizedMapEntry.posMap.put(unvocalizedPosMapKey,new UnvocalizedPosMapEntry());
	    unvocalizedPosMapEntry=(UnvocalizedPosMapEntry)unvocalizedMapEntry.posMap.get(unvocalizedPosMapKey);
	    unvocalizedPosMapEntry.pos=contextInfo.reducedTags[i];
	}
	unvocalizedPosMapEntry.num++;

	UnvocalizedPosLemmaMapEntry unvocalizedPosLemmaMapEntry=
	    (UnvocalizedPosLemmaMapEntry)unvocalizedPosMapEntry.lemmaMap.get(unvocalizedPosLemmaMapKey);
	if (unvocalizedPosLemmaMapEntry==null) {
	    unvocalizedPosMapEntry.lemmaMap.put(unvocalizedPosLemmaMapKey,new UnvocalizedPosLemmaMapEntry());
	    unvocalizedPosLemmaMapEntry=(UnvocalizedPosLemmaMapEntry)unvocalizedPosMapEntry.lemmaMap.get(unvocalizedPosLemmaMapKey);
	    unvocalizedPosLemmaMapEntry.lemma=contextInfo.lemmas[i];
	}
	unvocalizedPosLemmaMapEntry.num++;

	UnvocalizedPosLemmaVocalizedMapEntry unvocalizedPosLemmaVocalizedMapEntry=
	    (UnvocalizedPosLemmaVocalizedMapEntry)unvocalizedPosLemmaMapEntry.vocalizedMap.get(unvocalizedPosLemmaVocalizedMapKey);
	if (unvocalizedPosLemmaVocalizedMapEntry==null) {
	    unvocalizedPosLemmaMapEntry.vocalizedMap.put(unvocalizedPosLemmaVocalizedMapKey,
					      new UnvocalizedPosLemmaVocalizedMapEntry());
	    unvocalizedPosLemmaVocalizedMapEntry=
		(UnvocalizedPosLemmaVocalizedMapEntry)
		unvocalizedPosLemmaMapEntry.vocalizedMap.get(unvocalizedPosLemmaVocalizedMapKey);		    
	    unvocalizedPosLemmaVocalizedMapEntry.vocalized=contextInfo.vocalized[i];
	    unvocalizedPosLemmaVocalizedMapEntry.gloss=contextInfo.glosses[i];
	}
	unvocalizedPosLemmaVocalizedMapEntry.num++;
    }
	/*
	  unvocalized: (15)   1-15
          space:       (1)    16
	  num:         (5)    17-21
	  space:       (1)    22
          pos:         (15)   23-37
          space:       (1)    38
          num:         (5)    39-43  
          lemma:       (10)   44-53
	  space:       (1)    54
          num:         (5)    55-59
	  space:       (1)    60
	  vocalized:   (15)   61-75
          space:       (1)    76
          gloss:       (20)   77-96
	*/

    static int unvocalizedLength=15;
    static int numLength=5;
    static int spaceLength=1;
    static int posLength=25;
    static int lemmaLength=20;
    static int vocalizedLength=15;
    static int glossLength=20;


    void showBreakdown1() 
    throws IOException,FileNotFoundException{
	
	BufferedWriter out=new
		BufferedWriter(new 
		    OutputStreamWriter(new 
				       FileOutputStream(outputDir+"map1")));


	out.write("RESULTS 1\n");		      
	Iterator unvocalizedMapKeys=unvocalizedMap.keySet().iterator();
	
	boolean onNewLine=false;
	while (unvocalizedMapKeys.hasNext()) {	
	    String unvocalizedMapKey=(String)unvocalizedMapKeys.next();
	    UnvocalizedMapEntry unvocalizedMapEntry=
		(UnvocalizedMapEntry)unvocalizedMap.get(unvocalizedMapKey);	    
	    out.write(danbikel.parser.util.SKUtil.leftJustify(unvocalizedLength,unvocalizedMapEntry.unvocalized) + 
			     " "+
			     danbikel.parser.util.SKUtil.rightJustify(numLength,"("+unvocalizedMapEntry.num+")")+
			     " ");
	    if (unvocalizedMapEntry.posMap.size()>1) {
		out.write("\n");
		onNewLine=true;
	    } else {
		onNewLine=false;
	    }

	    int i=0;
	    Iterator unvocalizedPosMapKeys=unvocalizedMapEntry.posMap.keySet().iterator();
	    while (unvocalizedPosMapKeys.hasNext()) {	
		String unvocalizedPosMapKey=(String)unvocalizedPosMapKeys.next();
		UnvocalizedPosMapEntry unvocalizedPosMapEntry=
		    (UnvocalizedPosMapEntry)unvocalizedMapEntry.posMap.get(unvocalizedPosMapKey);
		if (onNewLine) {
		    out.write(danbikel.parser.util.SKUtil.leftJustify(unvocalizedLength+1+numLength+1," "));
		}
		out.write(danbikel.parser.util.SKUtil.leftJustify(posLength,unvocalizedPosMapEntry.pos)+
			  " "+
			  danbikel.parser.util.SKUtil.rightJustify(numLength,"("+unvocalizedPosMapEntry.num+")")+
				 " ");
		if (unvocalizedPosMapEntry.lemmaMap.size()>1) {
		    out.write("\n");
		    onNewLine=true;
		} else {
		    onNewLine=false;
		}
		/*		    System.out.println(danbikel.parser.util.SKUtil.leftJustify(5,"P"+i+")")+
				   danbikel.parser.util.SKUtil.leftJustify(20," ")+
				   danbikel.parser.util.SKUtil.leftJustify(15,unvocalizedPosMapEntry.pos) + 
				   " total=" + unvocalizedPosMapEntry.num + 
				   " #differentLemmaVocalized="+unvocalizedPosMapEntry.lemmaVocalizedMap.size());
		*/
		i++;

		int j=0;
		Iterator unvocalizedPosLemmaMapKeys=
		    unvocalizedPosMapEntry.lemmaMap.keySet().iterator();
		while (unvocalizedPosLemmaMapKeys.hasNext()) {	
		    String unvocalizedPosLemmaMapKey=(String)unvocalizedPosLemmaMapKeys.next();
		    UnvocalizedPosLemmaMapEntry unvocalizedPosLemmaMapEntry=
			(UnvocalizedPosLemmaMapEntry)
			unvocalizedPosMapEntry.lemmaMap.get(unvocalizedPosLemmaMapKey);
		    if (onNewLine) {
			System.out.print(danbikel.parser.util.SKUtil.leftJustify(unvocalizedLength+1+numLength+1+
										 posLength+1+numLength+1," "));
		    }   
		    System.out.print(danbikel.parser.util.SKUtil.leftJustify(lemmaLength,unvocalizedPosLemmaMapEntry.lemma) + 
				       " "+
				       danbikel.parser.util.SKUtil.rightJustify(numLength,"("+unvocalizedPosLemmaMapEntry.num+")")+
				       " ");
		    if (unvocalizedPosLemmaMapEntry.vocalizedMap.size()>1) {
			System.out.println();
			onNewLine=true;
		    } else {
			onNewLine=false;
		    }
		    j++;

		    int k=0;
		    Iterator unvocalizedPosLemmaVocalizedMapKeys=
			unvocalizedPosLemmaMapEntry.vocalizedMap.keySet().iterator();
		    while (unvocalizedPosLemmaVocalizedMapKeys.hasNext()) {	
			String unvocalizedPosLemmaVocalizedMapKey=(String)unvocalizedPosLemmaVocalizedMapKeys.next();
			UnvocalizedPosLemmaVocalizedMapEntry unvocalizedPosLemmaVocalizedMapEntry=
			    (UnvocalizedPosLemmaVocalizedMapEntry)
			    unvocalizedPosLemmaMapEntry.vocalizedMap.get(unvocalizedPosLemmaVocalizedMapKey);
			if (onNewLine) {
			    System.out.print(danbikel.parser.util.SKUtil.leftJustify(unvocalizedLength+1+numLength+1+
										     posLength+1+numLength+1 +
										     lemmaLength+1+numLength+1," "));
			}   
			System.out.println(danbikel.parser.util.SKUtil.leftJustify(vocalizedLength,unvocalizedPosLemmaVocalizedMapEntry.vocalized) + 
					   " "+
					   danbikel.parser.util.SKUtil.leftJustify(numLength,"("+unvocalizedPosLemmaVocalizedMapEntry.num+")") + 
					   " "+
					   danbikel.parser.util.SKUtil.leftJustify(glossLength,unvocalizedPosLemmaVocalizedMapEntry.gloss));
			onNewLine=true;
			j++;
		    }
		}
	    }
	}
	out.close();
    }




		    /*
		    System.out.println(danbikel.parser.util.SKUtil.leftJustify(5," ")+
				       danbikel.parser.util.SKUtil.leftJustify(5,"L"+j+")")+
				       danbikel.parser.util.SKUtil.leftJustify(12," ")+
				       danbikel.parser.util.SKUtil.leftJustify(15," ")+
				       danbikel.parser.util.SKUtil.leftJustify(15,unvocalizedPosLemmaVocalizedMapEntry.lemma) + 
				       " "+
				       danbikel.parser.util.SKUtil.leftJustify(15,unvocalizedPosLemmaVocalizedMapEntry.vocalized) + 
				       " "+
				       danbikel.parser.util.SKUtil.leftJustify(15,unvocalizedPosLemmaVocalizedMapEntry.gloss));
 		    */
    String getPosForMap2(ContextInfo contextInfo,int i) {
	return (
		(posForMap2==posShrunk) ?
		contextInfo.posShrunk[i] :
		contextInfo.reducedTags[i]);
    }

    String getWordForMap2(ContextInfo contextInfo,int i) {
	return (
		(wordForMap2==vocShrunk) ?
		contextInfo.vocShrunk[i] :
		contextInfo.words[i]);
    }
	    

    void storeInMap2(ContextInfo contextInfo, int i) {
	
	String lemmaMapKey="l="+contextInfo.lemmas[i];
	// String lemmaPosMapKey=lemmaMapKey+"p="+contextInfo.reducedTags[i];
	// String lemmaPosMapKey=lemmaMapKey+"p="+getPosForMap2(contextInfo,i);
	String lemmaPosMapKey=lemmaMapKey+"p="+contextInfo.posShrunk[i];

	// String lemmaPosUnvocalizedMapKey=lemmaPosMapKey+"u="+contextInfo.words[i];
	String lemmaPosUnvocalizedMapKey=lemmaPosMapKey+"u="+contextInfo.vocShrunk[i];

	// String lemmaPosUnvocalizedMapKey=lemmaPosMapKey+"u="+getWordForMap2(contextInfo,i);
	String lemmaPosUnvocalizedVocalizedMapKey=lemmaPosUnvocalizedMapKey+"v="+contextInfo.vocalized[i]+"fp="+contextInfo.fullTags[i];
	LemmaMapEntry lemmaMapEntry=
	    (LemmaMapEntry)lemmaMap.get(lemmaMapKey);
	if (lemmaMapEntry==null) {
	    lemmaMap.put(lemmaMapKey,new LemmaMapEntry());
	    lemmaMapEntry=(LemmaMapEntry)lemmaMap.get(lemmaMapKey);
	    lemmaMapEntry.lemma=contextInfo.lemmas[i];
	}
	lemmaMapEntry.num++;
	
	LemmaPosMapEntry lemmaPosMapEntry=
	    (LemmaPosMapEntry)lemmaMapEntry.posMap.get(lemmaPosMapKey);
	if (lemmaPosMapEntry==null) {
	    lemmaMapEntry.posMap.put(lemmaPosMapKey,new LemmaPosMapEntry());
	    lemmaPosMapEntry=(LemmaPosMapEntry)lemmaMapEntry.posMap.get(lemmaPosMapKey);
	    //	    lemmaPosMapEntry.pos=contextInfo.reducedTags[i];
	    lemmaPosMapEntry.pos=contextInfo.posShrunk[i];
	}
	lemmaPosMapEntry.num++;
	
	LemmaPosUnvocalizedMapEntry lemmaPosUnvocalizedMapEntry=
	    (LemmaPosUnvocalizedMapEntry)lemmaPosMapEntry.unvocalizedMap.get(lemmaPosUnvocalizedMapKey);
	if (lemmaPosUnvocalizedMapEntry==null) {
	    lemmaPosMapEntry.unvocalizedMap.put(lemmaPosUnvocalizedMapKey,
						new LemmaPosUnvocalizedMapEntry());
	    lemmaPosUnvocalizedMapEntry=
		(LemmaPosUnvocalizedMapEntry)
		lemmaPosMapEntry.unvocalizedMap.get(lemmaPosUnvocalizedMapKey);		    
	    //	    lemmaPosUnvocalizedMapEntry.unvocalized=contextInfo.words[i];
	    lemmaPosUnvocalizedMapEntry.unvocalized=contextInfo.vocShrunk[i];
	}
	lemmaPosUnvocalizedMapEntry.num++;

	LemmaPosUnvocalizedVocalizedMapEntry lemmaPosUnvocalizedVocalizedMapEntry = 
	    (LemmaPosUnvocalizedVocalizedMapEntry)lemmaPosUnvocalizedMapEntry.vocalizedMap.get(lemmaPosUnvocalizedVocalizedMapKey);
	if (lemmaPosUnvocalizedVocalizedMapEntry==null) {
	    lemmaPosUnvocalizedMapEntry.vocalizedMap.put(lemmaPosUnvocalizedVocalizedMapKey,
						new LemmaPosUnvocalizedVocalizedMapEntry());
	    lemmaPosUnvocalizedVocalizedMapEntry = 
	    (LemmaPosUnvocalizedVocalizedMapEntry)lemmaPosUnvocalizedMapEntry.vocalizedMap.get(lemmaPosUnvocalizedVocalizedMapKey);
	    lemmaPosUnvocalizedVocalizedMapEntry.vocalized=contextInfo.vocalized[i];
	    lemmaPosUnvocalizedVocalizedMapEntry.fullPos=contextInfo.fullTags[i];
	    lemmaPosUnvocalizedVocalizedMapEntry.gloss=contextInfo.glosses[i];
	}
	lemmaPosUnvocalizedVocalizedMapEntry.num++;
    }

    void showBreakdown2() 
	throws IOException,FileNotFoundException{

	BufferedWriter out=new
		BufferedWriter(new 
		    OutputStreamWriter(new 
				       FileOutputStream(outputDir+"map2")));

	out.write("RESULTS 2\n");
		      
	Iterator lemmaMapKeys=lemmaMap.keySet().iterator();
	
	boolean onNewLine=false;
	while (lemmaMapKeys.hasNext()) {	
	    String lemmaMapKey=(String)lemmaMapKeys.next();
	    LemmaMapEntry lemmaMapEntry=
		(LemmaMapEntry)lemmaMap.get(lemmaMapKey);	    
	    out.write(danbikel.parser.util.SKUtil.leftJustify(lemmaLength,lemmaMapEntry.lemma)+
			       " "+
			       danbikel.parser.util.SKUtil.rightJustify(numLength,"("+lemmaMapEntry.num+")")+
			       " ");
	    if (lemmaMapEntry.posMap.size()>1) {
		out.write("\n");
		onNewLine=true;
	    } else {
		onNewLine=false;
	    }

	    int i=0;
	    Iterator lemmaPosMapKeys=lemmaMapEntry.posMap.keySet().iterator();
	    while (lemmaPosMapKeys.hasNext()) {	
		String lemmaPosMapKey=(String)lemmaPosMapKeys.next();
		LemmaPosMapEntry lemmaPosMapEntry=
		    (LemmaPosMapEntry)lemmaMapEntry.posMap.get(lemmaPosMapKey);
		if (onNewLine) {
		    out.write(danbikel.parser.util.SKUtil.leftJustify(lemmaLength+1+numLength+1," "));
		}
		System.out.print(danbikel.parser.util.SKUtil.leftJustify(posLength,lemmaPosMapEntry.pos)+
				   " "+
				   danbikel.parser.util.SKUtil.rightJustify(numLength,"("+lemmaPosMapEntry.num+")"));
		if (lemmaPosMapEntry.unvocalizedMap.size()>1) {
		    out.write("\n");
		    onNewLine=true;
		} else {
		    onNewLine=false;
		}
		
		Iterator lemmaPosUnvocalizedMapKeys=
		    lemmaPosMapEntry.unvocalizedMap.keySet().iterator();
		while (lemmaPosUnvocalizedMapKeys.hasNext()) {	
		    String lemmaPosUnvocalizedMapKey=(String)lemmaPosUnvocalizedMapKeys.next();
		    LemmaPosUnvocalizedMapEntry lemmaPosUnvocalizedMapEntry=
			(LemmaPosUnvocalizedMapEntry)
			lemmaPosMapEntry.unvocalizedMap.get(lemmaPosUnvocalizedMapKey);
		    if (onNewLine) {
			out.write(danbikel.parser.util.SKUtil.leftJustify(lemmaLength+1+numLength+1+
										 posLength+1+numLength," "));
		    }
		    out.write(danbikel.parser.util.SKUtil.leftJustify(unvocalizedLength,lemmaPosUnvocalizedMapEntry.unvocalized)+
			      " "+
			      danbikel.parser.util.SKUtil.leftJustify(numLength,"("+lemmaPosUnvocalizedMapEntry.num+")") + 
			      " ");

		    onNewLine=true;

		    
		    Iterator lemmaPosUnvocalizedVocalizedMapKeys=
			lemmaPosUnvocalizedMapEntry.vocalizedMap.keySet().iterator();
		    while (lemmaPosUnvocalizedVocalizedMapKeys.hasNext()) {	
			String lemmaPosUnvocalizedVocalizedMapKey=(String)lemmaPosUnvocalizedVocalizedMapKeys.next();
			LemmaPosUnvocalizedVocalizedMapEntry lemmaPosUnvocalizedVocalizedMapEntry=
			    (LemmaPosUnvocalizedVocalizedMapEntry)
			    lemmaPosUnvocalizedMapEntry.vocalizedMap.get(lemmaPosUnvocalizedVocalizedMapKey);
			out.write(danbikel.parser.util.SKUtil.leftJustify(lemmaLength+1+numLength+1+
										 posLength+1+numLength," "));			
			out.write(danbikel.parser.util.SKUtil.leftJustify(vocalizedLength, lemmaPosUnvocalizedVocalizedMapEntry.vocalized)+
				  " " +
				  lemmaPosUnvocalizedVocalizedMapEntry.fullPos+
				  " " + 
				  lemmaPosUnvocalizedVocalizedMapEntry.gloss+
				  "\n");
		    }
		}
	    }
	}
	out.close();
    }


    static boolean hasOnePreterminal(SexpList treeList) {
	for (int i = 1; i < treeList.length(); i++) {
	    Sexp child=treeList.get(i);
	    if (Language.treebank().isPreterminal(child)) 
		return true;
	}
	return false;
    }

    static boolean hasOnePreterminalBesidesPunc(SexpList treeList) {
	for (int i = 1; i < treeList.length(); i++) {
	    Sexp child=treeList.get(i);
	    if (Language.treebank().isPreterminal(child)) {
		if (!child.list().symbolAt(ContextInfo.posGoldX).toString().equals("PUNC")) {
		    return true;
		}
	    }
	}
	return false;
    }


    protected void showResults()
	throws FileNotFoundException,IOException {
	//	showBreakdown();
	showResultsVP();
    }

    protected void showBreakdown()
	throws IOException,FileNotFoundException{
	showBreakdown1();
	showBreakdown2();
    }
    protected void showResultsVP() {
	System.err.println("sorting headlessVP");
	headlessVPs.sortResults();
	System.err.println("showing headless VP");
	System.out.println("#headless VPs="+headlessVPs.total()+ " # with head = "  +countVPwithVHead);
	headlessVPs.showResults();
	
    }

    public static void main(String[] args) {
	ArrayList dir=new ArrayList();
	dir.add(new File(args[0]));
	
	//	System.out.println("added"+args[1]+"to dir");
	List files=danbikel.parser.util.SKUtil.expandDirectories(dir,defaultExtension,false,false);
	try {
	    ExamineATB examineATB=new ExamineATB();
	    examineATB.process(files);
	}
	catch (FileNotFoundException fnfe) {
	System.err.println(fnfe);
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    
	}
    }
}




