package danbikel.parser.examine2;

import danbikel.parser.Constants;
import danbikel.parser.HeadFinder;
import danbikel.parser.Language;
import danbikel.parser.Treebank;
import danbikel.parser.Settings;
import danbikel.parser.Nonterminal;
import danbikel.parser.Word;
import danbikel.parser.Words;
import danbikel.util.*;
import danbikel.lisp.*;
import java.util.*;
import java.io.*;

public class ModifyUnifiedATB extends ExamineATB {

    String oldDir="/nlp/home/beatrice/arabic/corpus/ATB/ANNAHAR/release/ATB3-v2.0/data/combined/";
    String newDir="/nlp/home/beatrice/arabic/corpus/ATB/ANNAHAR/release/ATB3-v2.0/data/combined-modified1/";

    public ModifyUnifiedATB(int mode) {
	super();
    }



    protected void process(Sexp tree) {

      if (Language.treebank().isNullElementPreterminal(tree)) {
	  return;
      } else   if (Language.treebank().isPreterminal(tree)) {
	  /*
	    SexpList treeList = tree.list();
	    Symbol currLabel = treeList.symbolAt(0);
	    treeList.set(0, treebank.getCanonical(currLabel));
	  */
	  return;
      }
      else if (tree.isList()) {
	  SexpList treeList = tree.list();
	  ContextInfo contextInfo=null;
	  String parentStr=treeList.symbolAt(0).toString();
	  boolean isNP=Language.treebank().isNP(treeList.first().symbol());
	  boolean isVP=parentStr.startsWith("VP");
	  if (hasOnePreterminalBesidesPunc(treeList)) {
	      contextInfo=new ContextInfo();
	      contextInfo.fillFromCombinedATB(tree);
	      contextInfo.filename=currentFilename;
	      contextInfo.sentNum=currentSentNum;
	      //	      storePreterminalInfo(contextInfo);
	      if (isVP) {
		  processVP(contextInfo,treeList);
	      }
	  }
	  for (int i = 1; i < treeList.length(); i++) {
	      process(treeList.get(i));
	  }
      } else {
      System.err.print("not list or preterminal"+tree);
      }
  }


    //make this take an index for whether to look at index 0 or index 1
    // and do the same for the other PRT cases
    boolean isLayosCase(ContextInfo contextInfo, Sexp tree, int childX) {
	if (!contextInfo.reducedTags[childX].equals("NEG_PART+PVSUFF_SUBJ"))
	    return false;
	if (!contextInfo.vocalized[childX].endsWith("layos+a") ) {
	    System.err.println("NEG_PART+PVSUFF_SUBJ but not layos+a");
	    return false;
	}
	String oldTreeStr=tree.toString();	    
	Sexp child=tree.list().get(childX+1); // parent is index 0
	String newPosGoldStr=child.list().symbolAt(ContextInfo.posGoldX).toString().replaceAll("NEG_PART","PV");
	String newPosShrunkStr=child.list().symbolAt(ContextInfo.posShrunkX).toString().replaceAll("NEG_PART","PV");
	child.list().set(ContextInfo.posGoldX,Symbol.add(newPosGoldStr));
	child.list().set(ContextInfo.posShrunkX,Symbol.add(newPosShrunkStr));
	//	child.list().set(ContextInfo.glossX,Symbol.add("not be + he/it [verb]"));

	/*	if (childX==1) {
	    	    System.err.println("fixup isLayosCase childX=1" + "\nfrom="+oldTreeStr+"\nto " + tree.toString());
	}
	*/
	return true;
    }


    boolean firstWordIsDeverbal(ContextInfo contextInfo, Sexp tree, int childX) {
	if (!contextInfo.reducedTags[childX].equals("NOUN") &&
	    !contextInfo.reducedTags[childX].equals("ADJ") &&
	    !contextInfo.reducedTags[childX].equals("NO_FUNC") &&
	    !contextInfo.reducedTags[childX].equals("DET+ADJ") &&
	    !contextInfo.reducedTags[childX].equals("DET+NOUN") &&
	    !contextInfo.reducedTags[childX].startsWith("PVSUFF_DO") &&
	    !contextInfo.reducedTags[childX].startsWith("IVSUFF_DO") &&
	    !contextInfo.reducedTags[childX].startsWith("NOUN_PROP") &&
	    !contextInfo.reducedTags[childX].startsWith("PREP") && // yuch!  PREP NP-SBJ[] NP-OBJ 
	    !contextInfo.reducedTags[childX].startsWith("PART") &&  // PART NP-SBJ[NP SBAR-TMP] SBAR 
	    !contextInfo.reducedTags[childX].startsWith("ADV"))  // ADV NP-SBJ[] SBAR 
	    return false;
	String oldTreeStr=tree.toString();
	Sexp child=tree.list().get(childX+1);
	child.list().set(ContextInfo.posGoldX,Symbol.add("DV"));
	child.list().set(ContextInfo.posShrunkX,Symbol.add("DV"));
	
	//	System.err.println("fixup firstWordIsDeverbal" + "\nfrom="+oldTreeStr+"\nto " + tree.toString());	
	return true;
    }

	
    void processVP(ContextInfo contextInfo, Sexp tree) {
	  SexpList treeList = tree.list();

	  String oldTreeStr=tree.toString();

	if (contextInfo.containsVerb()) {
	    //	    System.err.println("containsVerb true " + treeList);
	    return;
	}
	//	System.err.println("processVP " + treeList);
	String key=contextInfo.keyFromReducedTag();
	if (key.indexOf("CONJ")>-1) 
	    return;

	int childX=0;
	if (contextInfo.reducedTags[0].equals("PRT")) {
	    childX=1;
	    if (contextInfo.reducedTags[1].equals("PRT")) {
		childX=2;
	    }
	} else if (contextInfo.reducedTags[0].equals("ADVP")) {
	    // for ADVP NO_FUNC NP-SBJ SBAR-NOM-OBJ
	    childX=1;
	} else if (contextInfo.reducedTags[0].equals("PUNC")) {
	    // PUNC NO_FUNC NP-SBJ NP-ADV
	    childX=1;
	    if (contextInfo.reducedTags[1].equals("PRT")) {
		// PUNC PRT NO_FUNC NP-SBJ NP-OBJ ADVP NP PP-PRP PUNC
		childX=2;
	    }
	} else if (contextInfo.reducedTags[0].equals("VERB_PART")) {
	    //  VERB_PART NO_FUNC NP-SBJ NP-OBJ
	    childX=1;
	}




	if (isLayosCase(contextInfo,tree,childX))
	    return;
	if (firstWordIsDeverbal(contextInfo,tree,childX))
	    return;
	
	System.err.println("remaining headless  " + 
			   getKeyForVP(contextInfo,treeList)+ "\n   " +
			   tree);
	
	    /*	
	int indexToChange=-1;
	if (contextInfo.reducedTags[0].equals("NOUN") ||
	    contextInfo.reducedTags[0].equals("ADJ") ||
	    contextInfo.reducedTags[0].equals("NO_FUNC"))
	//	System.err.println("containsVerb false");
	//	System.err.println("verbless VP"+treeList);
	String key1=contextInfo.keyFromReducedTag();
	String key2=getKeyForVP(contextInfo,treeList);
	if (!key1.equals(key2)) {
	    //	    System.err.println("key adjusted from " + key1 + " to "+ key2);
	} else {
	    //	    System.err.println("key =  "+ key2);
	}
	headlessVPs.enterIntoMap(key2, contextInfo);
	*/
    }

    public void process(List files) 
	throws FileNotFoundException,IOException {
	
	int i=0;
	for (Iterator fileIter = files.iterator(); fileIter.hasNext(); ) {
	    String baseFileName=(String)fileIter.next();
	    //	    String inFileName=baseFileName+defaultExtension;
	    if (i++ % 50 ==0) 
		System.err.println(i);

	    process(baseFileName);
	}
	//	showResults();

    }


    public void process(String filename)
	throws FileNotFoundException,IOException {
	//	System.err.println("reading file"+filename);
	int index=filename.lastIndexOf("/");
	currentFilename=filename.substring(index+1);
	currentSentNum=0;
	String outFilename=filename.replaceAll(oldDir,newDir);
	System.out.println("outFilename="+outFilename);
	BufferedWriter outWriter= new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
				       FileOutputStream(outFilename)));
	
	try {
	    //	    Training3 training = (Training3)Language.training();
	    SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
						  Constants.defaultFileBufsize);
	    Sexp curr = null;
	    while ((curr = Sexp.read(tok)) != null) {
		//		System.out.println("curr="+curr);
		process(curr);
		outWriter.write(curr+"\n");
		currentSentNum++;
	    
	    }
	    outWriter.close();
	}

	catch (UnsupportedEncodingException uee) {
	    System.err.println(uee);
	}
    }


    public static void main(String[] args) {
	String filename = null;
	ArrayList dir=new ArrayList();
	int mode=Integer.parseInt(args[0]);
	System.err.print ("using mode"+mode);
	dir.add(new File(args[1]));
	//	System.out.println("added"+args[0]+"to dir");
	List files=danbikel.parser.util.SKUtil.expandDirectories(dir,defaultExtension,false,false);
	//	String newDir=args[2];
	try {
	    ModifyUnifiedATB modifyUnifiedATB=new ModifyUnifiedATB(mode);
	    modifyUnifiedATB.process(files);
	}
	catch (FileNotFoundException fnfe) {
	    System.err.println(fnfe);
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    
	}
    }
}

