package danbikel.parser.parsedebug;

import java.io.*;
import java.util.*;
import java.util.regex.*;
import java.text.DecimalFormat;

import danbikel.lisp.Sexp;
import danbikel.lisp.SexpList;
import danbikel.lisp.Symbol;
import danbikel.lisp.SexpTokenizer;
import danbikel.parser.util.Util;
import danbikel.parser.Treebank;
import danbikel.parser.Language;
import danbikel.util.IntCounter;


class ParseTreeNode {
    private static Treebank treebank = Language.treebank();
    private final static String className = ParseTreeNode.class.getName();

    public Symbol label;
    public Symbol text;
    
    public int leftIdx;
    public int rightIdx;
    public int leftIdxDeleted;
    public int rightIdxDeleted;

    Sexp nodeInOriginalTree;

    boolean deleted;
    
    public List children;
    static ArrayList deleteLabels;
    
    public static void setDeleteLabels(ArrayList deleteLabelsA) {
	deleteLabels=deleteLabelsA;
	System.out.println("ParseTreeNode:deleteLabels:");
	for (int i=0; i<deleteLabels.size(); i++) {
	    System.out.println(i+") "+deleteLabels.get(i));
	}
	/*
	System.out.println("contains test " + deleteLabels.contains(Symbol.get(",")));
	String text=",";
	System.out.println("contains test2 " + deleteLabels.contains(Symbol.get(text)));
	*/
    }

    public ParseTreeNode(Sexp tree) {
	this(tree, new IntCounter(), new IntCounter());
    }

    private ParseTreeNode(Sexp tree, IntCounter wordCounter,
			  IntCounter wordCounterDeleted) {
	
	//	System.out.println("parseTreeNode tree="+tree);
	if (tree.isSymbol())
	    throw new IllegalArgumentException(className + 
					       ": constructor argument " +
					       "\"" + tree 
					       + "\" is of wrong type: " +
					       tree.getClass().getName());
	nodeInOriginalTree=tree;

	leftIdx = wordCounter.get();
	leftIdxDeleted = wordCounterDeleted.get();

	if (treebank.isPreterminal(tree)) {
	    children = Collections.EMPTY_LIST;
	    text=tree.list().get(1).symbol();
	    label = tree.list().get(0).symbol();
	    wordCounter.increment();
	    rightIdx = wordCounter.get();
	    if (!deleteLabels.contains(label)) {
		wordCounterDeleted.increment();
		rightIdxDeleted = wordCounterDeleted.get();
		deleted=false;
	    } else {
		rightIdxDeleted = -1;
		deleted=true;
	    }
	    
	    return;
	}
	
	if (tree.isList()) {
	    SexpList treeList = tree.list();
	    int treeListLen = treeList.length();
	    int firstIndex=1;
	    if (treeList.get(0).isSymbol()) {
		children=new ArrayList(treeListLen-1);
		label = treeList.get(0).symbol();
	    } else {
		System.err.println("WARNING: first element not symbol"+
				   treeList.get(0)+
				   " treeListLen="+treeListLen);
		
		//		label=Symbol.add("zero");
		label=Symbol.add("");
		firstIndex=0;
		children=new ArrayList(treeListLen);		
	    }
	    text=null;
	    deleted=false; // should check!

	    for (int i=firstIndex; i<treeListLen; i++) {
		ParseTreeNode child=new ParseTreeNode(treeList.get(i), 
						      wordCounter,
						      wordCounterDeleted);
		children.add(child);
	    }
	    
	    rightIdx = wordCounter.get();
	    rightIdxDeleted = wordCounterDeleted.get();
	    return;
	}

	// shouldn't ever reach this point!
	System.err.println(className + ": something very bad happened");
    }

    public boolean isPreterminal() { 
	return children == Collections.EMPTY_LIST; 
    }


    public String toString() {
	StringBuffer sb=new StringBuffer();
	sb.append("["+leftIdx+","+rightIdx+"]"+
		  "["+leftIdxDeleted+","+rightIdxDeleted+"]");
	if (isPreterminal()) {
	    sb.append(" ("+label+" "+text+")");
	} else {
	    sb.append(" ("+label+"#children="+children.size());
	}
	return sb.toString();
    }
}


