/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
package danbikel.parser.ppceme;

import java.util.*;
import java.io.*;
import danbikel.lisp.*;
import danbikel.parser.*;


public class Training extends danbikel.parser.lang.AbstractTraining2 {


    /** Data member returned by the accessor method of the same name. */
    private Symbol startSym = Symbol.add("*START*");
    /** Data member returned by the accessor method of the same name. */
    private Symbol stopSym = Symbol.add("*STOP*");
    /** Data member returned by the accessor method of the same name. */
    private Word startWord = Words.get(startSym, startSym);
    /** Data member returned by the accessor method of the same name. */
    private Word stopWord = Words.get(stopSym, stopSym);
    /** Data member returned by the accessor method of the same name. */
    private Symbol topSym = Symbol.add("*TOP*");
    /** Data member returned by the accessor method of the same name. */
    private Word topWord = Words.get(topSym, topSym);




    
    private static String[] pruneNodesDuplicateElements = {"``","''",".","CODE"};
    private static Set pruneNodesDuplicate=new HashSet();
    static {
	for (int i=0; i<pruneNodesDuplicateElements.length; i++) 
	    pruneNodesDuplicate.add(Symbol.add(pruneNodesDuplicateElements[i]));
    }
    
    public boolean removeWord(Symbol word, Symbol tag, int idx, SexpList sentence,
			      SexpList tags, SexpList originalTags,
			      Set prunedPretermsPosSet,
			      Map prunedPretermsPosMap) {
	/*
	  return (tag != null ?
	  prunedPretermsPosSet.contains(tag) :
	  prunedPretermsPosMap.containsKey(word));
	*/
	//    return prunedPretermsPosMap.containsKey(word);
	return pruneNodesDuplicate.contains(tag);
    }
    
    
    public Training() throws FileNotFoundException, IOException {
	System.err.println("ppceme Training constructor");
	startSym = Symbol.add("*START*");
	stopSym = Symbol.add("*STOP*");
	startWord = Words.get(startSym, startSym);
	stopWord = Words.get(stopSym, stopSym);
	topSym = Symbol.add("*TOP*");
	topWord = Words.get(topSym, topSym);

	String language = Settings.get(Settings.language);
	String metadataResource = Settings.get(metadataPropertyPrefix + language);
	InputStream is = Settings.getFileOrResourceAsStream(this.getClass(),
							    metadataResource);
	int bufSize = Constants.defaultFileBufsize;
	SexpTokenizer metadataTok =
	    new SexpTokenizer(is, Language.encoding(), bufSize);
	readMetadata(metadataTok);
	printMetadata();
	System.err.println("skipPostProcessing="+skipPostProcessing);

    }


    // re-defining the accessor methods for the changed start, stop and top
    // data members (and their associated Word objects)
    public Symbol startSym() { return startSym; }
    public Word startWord() { return startWord; }
    public Symbol stopSym() { return stopSym; }
    public Word stopWord() { return stopWord; }
    public Symbol topSym() { return topSym; }
    public Word topWord() { return topWord; }


    public Sexp preProcess(Sexp tree) {
	stripDollarSigns(tree);
	prune(tree);
	addBaseNPs(tree);
	repairBaseNPs(tree);
	//    addGapInformation(tree);
	//    relabelSubjectlessSentences(tree);
	removeNullElements(tree);
	raisePunctuation(tree);
	identifyArguments(tree);
	stripAugmentations(tree);
	//fixSubjectlessSentences(tree);  this method now in English lang. package
	return tree;
    }


    public Sexp relabelSubjectlessSentences(Sexp tree) {
	return tree;
    }

    public void postProcess(Sexp tree) {
	//unrepairBaseNPs(tree);
	//    super.postProcess(tree);
	removeOnlyChildBaseNPs(tree);
	System.out.println("after remove basenps = " + tree);
	//    canonicalizeNonterminals(tree);

	//raiseCodeNodes(tree);
	//System.out.println("after raiseCodeNodes " + tree);
	stripArgAugAndCanonicalizeBases(tree);
	System.out.println("after stripArgAugAndCanonicalizeBases" + tree);
    }

  protected static void usage() {
      System.err.println("use it right!");
    System.exit(1);
  }

  /** Test driver for this class. */
  public static void main(String[] args) {
    String filename = null;
    boolean stripDollarSigns = false;
    boolean prune = false;
    boolean addBaseNPs = false;
    boolean repairBaseNPs= false;
    boolean removeNullElements = false;
    boolean raisePunc = false;
    boolean idArgs = false;
    boolean stripAug = false;
    boolean combineTransformations = false;

    for (int i = 0; i < args.length; i++) {
      // for each arg that begins with a '-', examine each of its letters
      // and set booleans accordingly, except for "-combine" flag
      if (args[i].charAt(0) == '-') {
	if (args[i].equals("-combine"))
	  combineTransformations = true;
	else {
	  for (int charIdx = 1; charIdx < args[i].length(); charIdx++) {
	    char curr = args[i].charAt(charIdx);
	    switch (curr) {
	    case 't':
	      stripDollarSigns = true;
	      break;
	    case 'p':
	      prune = true;
	      break;
	    case 'b':
	      addBaseNPs = true;
	      break;
	    case 'c':
	      repairBaseNPs = true;
	      break;
	    case 'n':
	      removeNullElements = true;
	      break;
	    case 'r':
	      raisePunc = true;
	      break;
	    case 'i':
	      idArgs = true;
	      break;
	    case 'a':
	      stripAug = true;
	      break;
	    default:
	      System.err.println("illegal flag: " + curr);
	      usage();
	    }
	  }
	}
      }
      else
	filename = args[i];
    }

    if (filename == null) {
      usage();
    }

    Training2 training = (Training2)Language.training();
    //training.printMetadata();

    try {
      SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
					    Constants.defaultFileBufsize);
      Sexp curr = null;
      int sentNum=0;
      while ((curr = Sexp.read(tok)) != null) {
	  String skipStr = training.skip(curr);
	  if (skipStr != null) {
	      System.err.println(": skipping tree No. " +
				 (sentNum + 1) + ": " + skipStr + 
				 " tree="+curr);
	      continue;
	  }
	if (stripDollarSigns) {
	  curr = training.stripDollarSigns(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (prune) {
	  curr = training.prune(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (addBaseNPs) {
	  curr =training.addBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (repairBaseNPs) {
	  curr =training.repairBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (removeNullElements) {
	  curr = training.removeNullElements(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (raisePunc) {
	  curr = training.raisePunctuation(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (idArgs) {
	  curr =training.identifyArguments(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (stripAug) {
	  curr = training.stripAugmentations(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (combineTransformations)
	  System.out.println(curr);
      }
      sentNum++;
    }
    catch (UnsupportedEncodingException uee) {
      System.err.println(uee);
    }
    catch (FileNotFoundException fnfe) {
      System.err.println(fnfe);
    }
    catch (IOException ioe) {
      System.err.println(ioe);
    }
  }

}
