/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
package danbikel.parser.ppceme;

import java.util.*;
import java.io.*;
import danbikel.lisp.*;
import danbikel.parser.*;


public class Training12 extends Training9 {

    private Nonterminal nonterminal6 = new Nonterminal();
    protected Symbol PRNAugmentation=Symbol.add("PRN");

    public Training12() throws FileNotFoundException, IOException {
	super();
	System.err.println("ppceme Training12 constructor");
    }
    
    public Sexp preProcess(Sexp tree) {
	separateFuncTags(tree,true);
	convertCONJP(tree);
        //      verifyCONJP(tree);
	stripDollarSigns(tree);
	prune(tree);
	//addBaseNPs(tree);
	//repairBaseNPs(tree);
	//    addGapInformation(tree);
	//    relabelSubjectlessSentences(tree);
	removeNullElements(tree);
	raisePunctuation(tree);
	identifyArguments(tree);
	// stripAugmentations(tree);
	stripIndices(tree);
	//fixSubjectlessSentences(tree);  this method now in English lang. package
	return tree;
    }

    public void postProcess(Sexp tree) {
	if (skipPostProcessing)
	    return;
        System.out.println("before fixCONJP = " + tree);
        fixCONJP(tree);
        System.out.println("after fixCONJP = " + tree);
        combineFuncTags(tree,true);
        System.out.println("after combineFuncTags = " + tree);

	//unrepairBaseNPs(tree);
	//    super.postProcess(tree);
	//removeOnlyChildBaseNPs(tree);
	//System.out.println("after remove basenps = " + tree);
	//    canonicalizeNonterminals(tree);

	raiseCodeNodes(tree);
	System.out.println("after raiseCodeNodes " + tree);
	stripArgAugAndCanonicalizeBases(tree);
	System.out.println("after stripArgAugAndCanonicalizeBases" + tree);
    }


    /* could make it just do the children, on the assumption that
       the root of a tree will never have PRN. But want to make this
       more general, to allow other func tags to be separated,
       and so should allow even the root to be separated
    */

    public Sexp separateFuncTags(Sexp tree,boolean justPRN) {
        //      System.out.println("separateFuncTags start=\n"+tree);
        Sexp tree2=separateFuncTags(null,-1,tree,justPRN);
        //      System.out.println("separateFuncTags end=\n"+tree);
        return tree2;
    }
        
    public Sexp separateFuncTags(Sexp grandparent, int parentIdx, 
                                    Sexp tree, boolean justPRN) {
        if (treebank.isPreterminal(tree)) {
            SexpList treeList = tree.list();
            Symbol parent = treeList.first().symbol();
            Nonterminal parsedParent =
                treebank.parseNonterminal(parent, nonterminal6);
            boolean isPRN=parsedParent.augmentations.contains(PRNAugmentation);
            if (isPRN) {
                System.err.println("warning: PRN on preterminal"+tree+" grandparent="+grandparent);
            }
            return tree;
        }
        if (tree.isList()) {
            SexpList treeList = tree.list();
            int treeListLen = treeList.length();
            if (tree.list().first().isList())
                System.err.println("error: tree " + tree + " has bad format");      
            Symbol parent = treeList.first().symbol();
            Nonterminal parsedParent =
                treebank.parseNonterminal(parent, nonterminal6);
            boolean isPRN=parsedParent.augmentations.contains(PRNAugmentation);
            if (isPRN) {
                treebank.removeAugmentation(nonterminal6,PRNAugmentation);
                Symbol parentWithoutPRN=nonterminal6.toSymbol();
                if (grandparent!=null) {
                    SexpList newParent = new SexpList(2);
                    treeList.set(0, parentWithoutPRN);
                    newParent.add(PRNAugmentation).add(tree);
                    grandparent.list().set(parentIdx, newParent);
                } else {
                    System.out.println("grandParent is null, not sure what to do");
                    System.out.println("tree="+tree);
                    treeList.set(0,PRNAugmentation);
                    SexpList newNode=new SexpList(treeListLen);
                    newNode.add(parentWithoutPRN); 
                    for (int j = 1; j < treeListLen; j++)
                        newNode.add(treeList.get(j));
                    for (int j = treeListLen - 1; j >= 1; j--)
                        treeList.remove(j);
                    treeList.add(newNode);
                    
                }
            }
            for (int i = 1; i < treeListLen; i++)
                separateFuncTags(tree, i, treeList.get(i),true);
        }
        return tree;
    }


    protected void combineFuncTags(Sexp tree,boolean justPRN) {
        System.out.println("combineFuncTags start=\n"+tree);
        combineFuncTags(null,-1,tree,justPRN);
        System.out.println("combineFuncTags end=\n"+tree);
    }

    protected void combineFuncTags(Sexp grandparent, int parentIdx,
                                   Sexp tree, boolean justPRN) {
        if (treebank.isPreterminal(tree))
            return;
        if (tree.isList()) {
            SexpList treeList = tree.list();
            int treeListLen = treeList.length();
            Symbol parent = treeList.first().symbol();
            boolean isPRN=parent.equals(PRNAugmentation);
            if (isPRN) {
                if (treeListLen!=2) {
                    System.err.println("something's wrong: PRN with non-only child");
                }
                Sexp child=treeList.get(1);
                Symbol childSymbol=child.list().first().symbol();
                Nonterminal parsedChild=
                    treebank.parseNonterminal(childSymbol,nonterminal6);
                treebank.addAugmentation(parsedChild,PRNAugmentation);
                Symbol parsedChildWithPRN=parsedChild.toSymbol();

                if (grandparent!=null) {
                    /* 
                       grandparent -> parent(PRN) -> child (with PRN removed)
                       to
                       grandparent ->                child (with PRN put back)
                    */
                    child.list().set(0,parsedChildWithPRN);
                    grandparent.list().set(parentIdx,child);
                } else {
                    System.out.println("grandParent is null, not sure what to do");
                    System.out.println("tree="+tree);
                    /*
                      root of tree is PRN
                      parent(PRN) -> child (with PRN removed) -> grandchildren
                      parent(child's tags+PRN)                -> grandchildren
                    */

                    treeList.set(0, parsedChildWithPRN);
                    treeList.remove(1);
                    for (int j=1; j<child.list().length(); j++) {
                        treeList.add(child.list().get(j));
                    }
                }

            }
            for (int i = 1; i < treeListLen; i++) {
                combineFuncTags(tree, i, treeList.get(i),true);
            }
        }
    }

  /** Test driver for this class. */
  public static void main(String[] args) {
    String filename = null;
    boolean splitPRN=true;
    boolean convertConj=false;
    boolean stripDollarSigns = false;
    boolean prune = false;
    //    boolean addBaseNPs = false;
    //boolean repairBaseNPs= false;
    boolean removeNullElements = false;
    boolean raisePunc = false;
    boolean idArgs = false;
    boolean stripAug = false;
    boolean combineTransformations = false;

    for (int i = 0; i < args.length; i++) {
      // for each arg that begins with a '-', examine each of its letters
      // and set booleans accordingly, except for "-combine" flag
      if (args[i].charAt(0) == '-') {
	if (args[i].equals("-combine"))
	  combineTransformations = true;
	else {
	  for (int charIdx = 1; charIdx < args[i].length(); charIdx++) {
	    char curr = args[i].charAt(charIdx);
	    switch (curr) {
	    case 'd':
	      convertConj = true;
	      break;
	    case 'e':
	      splitPRN= true;
	      break;
	    case 't':
	      stripDollarSigns = true;
	      break;
	    case 'p':
	      prune = true;
	      break;
	      /*
	    case 'b':
	      addBaseNPs = true;
	      break;
	    case 'c':
	      repairBaseNPs = true;
	      break;
	      */
	    case 'n':
	      removeNullElements = true;
	      break;
	    case 'r':
	      raisePunc = true;
	      break;
	    case 'i':
	      idArgs = true;
	      break;
	    case 'a':
	      stripAug = true;
	      break;
	    default:
	      System.err.println("illegal flag: " + curr);
	      usage();
	    }
	  }
	}
      }
      else
	filename = args[i];
    }

    if (filename == null) {
      usage();
    }

    Training9 training = (Training9)Language.training();
    //training.printMetadata();

    try {
      SexpTokenizer tok = new SexpTokenizer(filename, Language.encoding(),
					    Constants.defaultFileBufsize);
      Sexp curr = null;
      int sentNum=0;
      while ((curr = Sexp.read(tok)) != null) {
	  String skipStr = training.skip(curr);
	  if (skipStr != null) {
	      System.err.println(": skipping tree No. " +
				 (sentNum + 1) + ": " + skipStr + 
				 " tree="+curr);
	      continue;
	  }
	  System.out.println("----------------------------");
	  System.out.println(curr);
	if (splitPRN) {
	  curr = training.separateFuncTags(curr,true);
	  if (!combineTransformations)
	    System.out.println(curr);
	}

	if (convertConj) {
	  curr = training.convertCONJP(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (stripDollarSigns) {
	  curr = training.stripDollarSigns(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (prune) {
	  curr = training.prune(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	/*
	if (addBaseNPs) {
	  curr =training.addBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (repairBaseNPs) {
	  curr =training.repairBaseNPs(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	*/
	if (removeNullElements) {
	  curr = training.removeNullElements(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (raisePunc) {
	  curr = training.raisePunctuation(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (idArgs) {
	  curr =training.identifyArguments(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (stripAug) {
	  curr = training.stripAugmentations(curr);
	  if (!combineTransformations)
	    System.out.println(curr);
	}
	if (combineTransformations)
	  System.out.println(curr);
      }
      System.out.println("----------------------------");
      sentNum++;
    }
    catch (UnsupportedEncodingException uee) {
      System.err.println(uee);
    }
    catch (FileNotFoundException fnfe) {
      System.err.println(fnfe);
    }
    catch (IOException ioe) {
      System.err.println(ioe);
    }
  }

}
