package danbikel.parser.util;

import java.io.*;
import java.util.*;
import danbikel.util.*;
import danbikel.lisp.*;
import danbikel.parser.Constants;
import danbikel.parser.Language;
import danbikel.parser.util.*;

public class RaiseNodes{
    private static final int bufSize = 81920;
    
    // data members used by raisePunctuation
    private SexpList addToRaise = new SexpList();
    private SexpList raise = new SexpList();

    protected Set nodesToRaise = new HashSet();

    static String oldExt=".parsed.nn";
    // static String oldExt=".parsed";
    
    Symbol sym= Symbol.add("SYM");
    
    public RaiseNodes() {}

    void setNodesToRaise(ArrayList nodesArrayList) {
	for (int i=0; i<nodesArrayList.size(); i++) {
	    String nodeStr=(String)nodesArrayList.get(i);
	    //	    System.out.println("adding " + nodeStr+" to nodesToRaise");
	    nodesToRaise.add(Symbol.add(nodeStr));
	}
    }


    public Sexp raisePunctuation(Sexp tree) {
	/*
	  leftRaise.clear();
	  rightRaise.clear();
	  raisePunctuation(tree, leftRaise, rightRaise);
	*/
	//	System.out.println("raisePunctuation " + tree);
	raise.clear();
	raisePunctuation(tree, raise, Constants.LEFT,true);
	if (raise.size()>0) {
	    SexpList treeList = tree.list();
	    for (int raiseIdx = 0; raiseIdx < raise.length(); raiseIdx++) {
		int i=treeList.length();
		System.out.println("raised out left " + 
				   raiseIdx + ")" + 
				   raise.get(raiseIdx) + " putting back in");
		treeList.add(i, raise.get(raiseIdx));
	    }
	}
	raise.clear();
	raisePunctuation(tree, raise, Constants.RIGHT,true);
	if (raise.size()>0) {
	    SexpList treeList = tree.list();
	    for (int raiseIdx = 0; raiseIdx < raise.length(); raiseIdx++) {
		System.out.println("raised out right " + 
				   raiseIdx + ")" + 
				   raise.get(raiseIdx) + " putting back in");
		treeList.add(1, raise.get(raiseIdx));
	    }
	}
	return tree;
    }
    
    private void raisePunctuation(Sexp tree, SexpList raise, boolean direction, boolean atTop) {
	/*
	System.out.println("raisePunctuation " + tree + 
			   "\n   raise = "+ raise +
			   "\n   direction="+direction + 
			   "\n   atTop="+atTop);
	*/
	if (tree.isSymbol() || Language.treebank().isPreterminal(tree))
	    return;
	if (allChildrenPuncToRaise(tree)) {
	    System.err.println("RaiseNodes "+
			       ": warning: all children are punctuation to raise\n" +
			       "\t" + tree);
	    return;
	}
	
	// if tree is a list with at least two children (i.e., of length 3)
	if (tree.isList()) {
	    SexpList treeList = tree.list();
	    
	    boolean leftToRight = direction == Constants.LEFT;
	    int startIdx = (leftToRight ? 1 : treeList.length() - 1);
	    int endIdx = (leftToRight ? treeList.length() - 1 : 1);
	    int increment = (leftToRight ? 1 : -1);
	    for (int i = startIdx;
		 (leftToRight && i <= endIdx) || (!leftToRight && i >= endIdx);
		 i += increment) {
		// we're occasionally removing items from tree, so recalculate these
		startIdx = (leftToRight ? 1 : treeList.length() - 1);
		endIdx = (leftToRight ? treeList.length() - 1 : 1);
		Sexp currChild = treeList.get(i);
		
		raisePunctuation(currChild, raise, direction,false);
		
		// if it's the last child that we're visiting
		if (i == endIdx) {
		    // while there's ending punctuation, remove it and add it
		    // to the raise queue (in queue order, hence the call to
		    // reverse, since we're grabbing nodes from the outside in)
		    addToRaise.clear();
		    while (treeList.length() > 1 &&
			   isNodeToRaise(treeList.get(endIdx))) {
			    addToRaise.add(treeList.remove(endIdx));
			    endIdx = (leftToRight ? treeList.length() - 1 : 1);
		    }
		    // set i to the new last element index (not strictly necessary)
		    if (raise.addAll(addToRaise.reverse()))
			i = endIdx;
		}
		// if it's not the last child we're visiting at this level and
		// there are raise requests enqueued, we can oblige by
		// inserting the punctuation just after the current node
		else if (raise.length() != 0) {
		    // since items naturally shift rightward when we use the add method,
		    // to add "after" current node in a right-to-left traversal requires
		    // keeping add index the same (i.e., an increment of zero)
		    int insertIncrement = (leftToRight ? increment : 0);
		    i += insertIncrement;
		    for (int raiseIdx = 0; raiseIdx < raise.length(); raiseIdx++) {
			treeList.add(i, raise.get(raiseIdx));
			if (leftToRight)
			    i += insertIncrement;
		    }
		    endIdx = (leftToRight ? treeList.length() - 1 : 1);
		    i -= insertIncrement; // offset increment for enclosing for loop
		    raise.clear();
		}
	    }
	}
    }

    private boolean allChildrenPuncToRaise(Sexp tree) {
	int treeLen = tree.list().length();
	for (int i = 1; i < treeLen; i++)
	    if (isNodeToRaise(tree.list().get(i)) == false)
		return false;
	return true;
    }
    
    public boolean isNodeToRaise(Sexp preterm) {
	if (!Language.treebank().isPreterminal(preterm))
	    return false;
	if (preterm.list().symbolAt(0)==sym) {
	    System.out.println("not raising " + preterm);
	    return false;
	}
	return (nodesToRaise.contains(preterm.list().symbolAt(1)));
    }
    
    public void process(InputStream inStream,
			       BufferedWriter out) 
	throws IOException {
	
	Sexp tree = null;
	String enc = System.getProperty("file.encoding");
	SexpTokenizer tok = new 
	    SexpTokenizer(inStream, enc, bufSize);
	for (int i = 1; ((tree = Sexp.read(tok)) != null); i++) {
	    raisePunctuation(tree);
	    out.write(tree+"\n");
	}

	out.close();
	inStream.close();
    }


    public void process(List files) 
	throws IOException {
	for (Iterator fileIter = files.iterator(); fileIter.hasNext(); ) {
	    String baseFileName=(String)fileIter.next();
	    String inFileName=baseFileName+oldExt;
	    String outFileName=baseFileName+oldExt+".raised";

	    System.out.println("inFileName="+inFileName);
	    System.out.println("outFileName="+outFileName);
	    InputStream inStream=null;
	    inStream = new FileInputStream(inFileName);
	    BufferedWriter out=null;

	    out = new 
		BufferedWriter(new 
		    OutputStreamWriter(new 
			FileOutputStream(outFileName),
				       "ISO-8859-1"));
	    process(inStream,
		    out);
	}
    }


    public static List expandDirectories(List files,String fileExt, 
					 boolean trim, boolean expandDir) {
	List expanded = new ArrayList(files.size());
	int extlen = fileExt.length();	
	
	for (Iterator fileIter = files.iterator(); fileIter.hasNext();) {
	    File file = (File) fileIter.next();
	    String fname=file.toString();
	    
	    if (!file.exists())
		System.err.println("Warning: "+file+" does not exist");
	    else {
		if (file.isDirectory()) {
		    String[] dir = file.list();
		    for (int i=0; i<dir.length; i++)
			if (dir[i].endsWith(fileExt)) {
			    if (trim) {
				dir[i]=
				    dir[i].substring(0,dir[i].length()-extlen);
			    } 
			    expanded.add(new File(file, dir[i]).toString());
			} else if ((new File(file, dir[i])).isDirectory()) {
			    if (expandDir) {
				List lst = new ArrayList(1);
				lst.add(new File(file, dir[i]).toString());
				expanded.addAll(expandDirectories(lst,
								  fileExt,
								  trim,
								  expandDir));
			    } 
			    
			}
		}
		else {
		    if (trim) {
			fname=fname.substring(0,fname.length()-extlen);
		    } 
		    expanded.add(fname);
		}
	    }
	}
	return expanded;
    }


    public static void main(String[] args) {

	RaiseNodes raiseNodes=new RaiseNodes();
	ArrayList nodesArrayList=new ArrayList();
	nodesArrayList.add(new String("\""));
	nodesArrayList.add(new String("'"));
	nodesArrayList.add(new String("!"));
	nodesArrayList.add(new String("?"));
	nodesArrayList.add(new String("."));
	//	nodesArrayList.add(new String("''"));
	raiseNodes.setNodesToRaise(nodesArrayList);
	ArrayList dir=new ArrayList();
	dir.add(new File(args[0]));
	List files=expandDirectories(dir,oldExt,true,false);
	try {
	    raiseNodes.process(files);
	}
	catch (Exception e) {
	    System.err.println(e);
	    System.exit(-1);
	}
    }
}
