/* *           Copyright (c) 2004, Daniel M. Bikel.
 *                         All rights reserved.
 * 
 *                Developed at the University of Pennsylvania
 *                Institute for Research in Cognitive Science
 *                    3401 Walnut Street
 *                    Philadelphia, Pennsylvania 19104
 * 			
 * 
 * For research or educational purposes only.  Do not redistribute.  For
 * complete license details, please read the file LICENSE that accompanied
 * this software.
 * 
 * DISCLAIMER
 * 
 * Daniel M. Bikel makes no representations or warranties about the suitability of
 * the Software, either express or implied, including but not limited to the
 * implied warranties of merchantability, fitness for a particular purpose, or
 * non-infringement. Daniel M. Bikel shall not be liable for any damages suffered
 * by Licensee as a result of using, modifying or distributing the Software or its
 * derivatives.
 * 
 */
package danbikel.parser.util;

import danbikel.lisp.*;
import danbikel.parser.*;
import java.util.*;
import java.io.*;

/**
 * Contains basic utility functions for <code>Sexp</code> objects that
 * represent parse trees.
 * @author Dan Bikel
 */
public class SKUtil {

    private Nonterminal nonterminal3 = new Nonterminal();

    private SKUtil() {}


    public static boolean treeHasNoRoot(Sexp tree) {
	if (tree.isList()) {
	    return (!tree.list().get(0).isSymbol());
	} 
	return false;
    }

    public static boolean treeHasRoot(Sexp tree) {
	if (tree.isList()) {
	    return (tree.list().get(0).isSymbol());
	} 
	return false;
    }

    public static boolean treeIsNull(Sexp tree) {
	if (tree.isSymbol() &&
	    tree.symbol().toString().equals("null"))
	    return true;
	else 
	    return false;
    }


    public static Sexp adjustForParens(Sexp tree) {
	boolean stripOuterParens = (tree.list().length() == 1 &&
				    tree.list().get(0).isList());
	if (stripOuterParens)
	    tree = tree.list().get(0);
	
	return tree;
    }

	
    public static List expandDirectories(List files,String fileExt, 
					 boolean trim, boolean expandDir) {
	List expanded = new ArrayList(files.size());
	int extlen = fileExt.length();	
	
	for (Iterator fileIter = files.iterator(); fileIter.hasNext();) {
	    File file = (File) fileIter.next();
	    String fname=file.toString();
	    
	    if (!file.exists())
		System.err.println("Warning: "+file+" does not exist");
	    else {
		if (file.isDirectory()) {
		    String[] dir = file.list();
		    for (int i=0; i<dir.length; i++)
			if (dir[i].endsWith(fileExt)) {
			    if (trim) {
				dir[i]=
				    dir[i].substring(0,dir[i].length()-extlen);
			    } 
			    expanded.add(new File(file, dir[i]).toString());
			} else if ((new File(file, dir[i])).isDirectory()) {
			    if (expandDir) {
				List lst = new ArrayList(1);
				lst.add(new File(file, dir[i]).toString());
				expanded.addAll(expandDirectories(lst,
								  fileExt,
								  trim,
								  expandDir));
			    } 
			    
			}
		}
		else {
		    if (trim) {
			fname=fname.substring(0,fname.length()-extlen);
		    } 
		    expanded.add(fname);
		}
	    }
	}
	return expanded;
    }



    static final String blanks="                                                                                                                                                              ";    
    static public String leftJustify(int len, String t) {
	if (t.length()>len) {
	    return t;
	} else {
	    //	    System.err.println("len="+len+" t="+t);
	    return (t+blanks.substring(0,len-t.length()));
	}
    }
    static public String leftJustify(int len, int num) {
	    return leftJustify(len,num+"");
    }    
    
    static public String rightJustify(int len, int num) {
	return rightJustify(len,num+"");
    }
    
    static public String rightJustify(int len, String t) {
	if (t.length()>len) {
	    return t;
	} else {
	    return (blanks.substring(0,len-t.length())+t);
	}
    }



}
